/* hashmap.c generated by valac 0.56.3, the Vala compiler
 * generated from hashmap.vala, do not modify */

/* hashmap.vala
 *
 * Copyright (C) 1995-1997  Peter Mattis, Spencer Kimball and Josh MacDonald
 * Copyright (C) 1997-2000  GLib Team and others
 * Copyright (C) 2007-2009  Jürg Billeter
 * Copyright (C) 2009-2014  Maciej Piechotka
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include "gee.h"
#include <glib.h>
#include <glib-object.h>

#define GEE_HASH_MAP_MIN_SIZE 11
#define GEE_HASH_MAP_MAX_SIZE 13845163

typedef struct _GeeHashMapNode GeeHashMapNode;

#define GEE_FUNCTIONS_TYPE_HASH_DATA_FUNC_CLOSURE (gee_functions_hash_data_func_closure_get_type ())
#define GEE_FUNCTIONS_HASH_DATA_FUNC_CLOSURE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_FUNCTIONS_TYPE_HASH_DATA_FUNC_CLOSURE, GeeFunctionsHashDataFuncClosure))
#define GEE_FUNCTIONS_HASH_DATA_FUNC_CLOSURE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_FUNCTIONS_TYPE_HASH_DATA_FUNC_CLOSURE, GeeFunctionsHashDataFuncClosureClass))
#define GEE_FUNCTIONS_IS_HASH_DATA_FUNC_CLOSURE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_FUNCTIONS_TYPE_HASH_DATA_FUNC_CLOSURE))
#define GEE_FUNCTIONS_IS_HASH_DATA_FUNC_CLOSURE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_FUNCTIONS_TYPE_HASH_DATA_FUNC_CLOSURE))
#define GEE_FUNCTIONS_HASH_DATA_FUNC_CLOSURE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_FUNCTIONS_TYPE_HASH_DATA_FUNC_CLOSURE, GeeFunctionsHashDataFuncClosureClass))

typedef struct _GeeFunctionsHashDataFuncClosure GeeFunctionsHashDataFuncClosure;
typedef struct _GeeFunctionsHashDataFuncClosureClass GeeFunctionsHashDataFuncClosureClass;

#define GEE_FUNCTIONS_TYPE_EQUAL_DATA_FUNC_CLOSURE (gee_functions_equal_data_func_closure_get_type ())
#define GEE_FUNCTIONS_EQUAL_DATA_FUNC_CLOSURE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_FUNCTIONS_TYPE_EQUAL_DATA_FUNC_CLOSURE, GeeFunctionsEqualDataFuncClosure))
#define GEE_FUNCTIONS_EQUAL_DATA_FUNC_CLOSURE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_FUNCTIONS_TYPE_EQUAL_DATA_FUNC_CLOSURE, GeeFunctionsEqualDataFuncClosureClass))
#define GEE_FUNCTIONS_IS_EQUAL_DATA_FUNC_CLOSURE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_FUNCTIONS_TYPE_EQUAL_DATA_FUNC_CLOSURE))
#define GEE_FUNCTIONS_IS_EQUAL_DATA_FUNC_CLOSURE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_FUNCTIONS_TYPE_EQUAL_DATA_FUNC_CLOSURE))
#define GEE_FUNCTIONS_EQUAL_DATA_FUNC_CLOSURE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_FUNCTIONS_TYPE_EQUAL_DATA_FUNC_CLOSURE, GeeFunctionsEqualDataFuncClosureClass))

typedef struct _GeeFunctionsEqualDataFuncClosure GeeFunctionsEqualDataFuncClosure;
typedef struct _GeeFunctionsEqualDataFuncClosureClass GeeFunctionsEqualDataFuncClosureClass;
enum  {
	GEE_HASH_MAP_0_PROPERTY,
	GEE_HASH_MAP_K_TYPE,
	GEE_HASH_MAP_K_DUP_FUNC,
	GEE_HASH_MAP_K_DESTROY_FUNC,
	GEE_HASH_MAP_V_TYPE,
	GEE_HASH_MAP_V_DUP_FUNC,
	GEE_HASH_MAP_V_DESTROY_FUNC,
	GEE_HASH_MAP_SIZE_PROPERTY,
	GEE_HASH_MAP_READ_ONLY_PROPERTY,
	GEE_HASH_MAP_KEYS_PROPERTY,
	GEE_HASH_MAP_VALUES_PROPERTY,
	GEE_HASH_MAP_ENTRIES_PROPERTY,
	GEE_HASH_MAP_NUM_PROPERTIES
};
static GParamSpec* gee_hash_map_properties[GEE_HASH_MAP_NUM_PROPERTIES];
#define _gee_functions_hash_data_func_closure_unref0(var) ((var == NULL) ? NULL : (var = (gee_functions_hash_data_func_closure_unref (var), NULL)))
#define _gee_functions_equal_data_func_closure_unref0(var) ((var == NULL) ? NULL : (var = (gee_functions_equal_data_func_closure_unref (var), NULL)))
#define _gee_hash_map_node_free0(var) ((var == NULL) ? NULL : (var = (gee_hash_map_node_free (var), NULL)))

#define GEE_HASH_MAP_TYPE_NODE_ITERATOR (gee_hash_map_node_iterator_get_type ())
#define GEE_HASH_MAP_NODE_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_HASH_MAP_TYPE_NODE_ITERATOR, GeeHashMapNodeIterator))
#define GEE_HASH_MAP_NODE_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_HASH_MAP_TYPE_NODE_ITERATOR, GeeHashMapNodeIteratorClass))
#define GEE_HASH_MAP_IS_NODE_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_HASH_MAP_TYPE_NODE_ITERATOR))
#define GEE_HASH_MAP_IS_NODE_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_HASH_MAP_TYPE_NODE_ITERATOR))
#define GEE_HASH_MAP_NODE_ITERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_HASH_MAP_TYPE_NODE_ITERATOR, GeeHashMapNodeIteratorClass))

typedef struct _GeeHashMapNodeIterator GeeHashMapNodeIterator;
typedef struct _GeeHashMapNodeIteratorClass GeeHashMapNodeIteratorClass;

#define GEE_HASH_MAP_TYPE_MAP_ITERATOR (gee_hash_map_map_iterator_get_type ())
#define GEE_HASH_MAP_MAP_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_HASH_MAP_TYPE_MAP_ITERATOR, GeeHashMapMapIterator))
#define GEE_HASH_MAP_MAP_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_HASH_MAP_TYPE_MAP_ITERATOR, GeeHashMapMapIteratorClass))
#define GEE_HASH_MAP_IS_MAP_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_HASH_MAP_TYPE_MAP_ITERATOR))
#define GEE_HASH_MAP_IS_MAP_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_HASH_MAP_TYPE_MAP_ITERATOR))
#define GEE_HASH_MAP_MAP_ITERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_HASH_MAP_TYPE_MAP_ITERATOR, GeeHashMapMapIteratorClass))

typedef struct _GeeHashMapMapIterator GeeHashMapMapIterator;
typedef struct _GeeHashMapMapIteratorClass GeeHashMapMapIteratorClass;

#define GEE_HASH_MAP_TYPE_KEY_SET (gee_hash_map_key_set_get_type ())
#define GEE_HASH_MAP_KEY_SET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_HASH_MAP_TYPE_KEY_SET, GeeHashMapKeySet))
#define GEE_HASH_MAP_KEY_SET_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_HASH_MAP_TYPE_KEY_SET, GeeHashMapKeySetClass))
#define GEE_HASH_MAP_IS_KEY_SET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_HASH_MAP_TYPE_KEY_SET))
#define GEE_HASH_MAP_IS_KEY_SET_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_HASH_MAP_TYPE_KEY_SET))
#define GEE_HASH_MAP_KEY_SET_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_HASH_MAP_TYPE_KEY_SET, GeeHashMapKeySetClass))

typedef struct _GeeHashMapKeySet GeeHashMapKeySet;
typedef struct _GeeHashMapKeySetClass GeeHashMapKeySetClass;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))

#define GEE_HASH_MAP_TYPE_VALUE_COLLECTION (gee_hash_map_value_collection_get_type ())
#define GEE_HASH_MAP_VALUE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_HASH_MAP_TYPE_VALUE_COLLECTION, GeeHashMapValueCollection))
#define GEE_HASH_MAP_VALUE_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_HASH_MAP_TYPE_VALUE_COLLECTION, GeeHashMapValueCollectionClass))
#define GEE_HASH_MAP_IS_VALUE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_HASH_MAP_TYPE_VALUE_COLLECTION))
#define GEE_HASH_MAP_IS_VALUE_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_HASH_MAP_TYPE_VALUE_COLLECTION))
#define GEE_HASH_MAP_VALUE_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_HASH_MAP_TYPE_VALUE_COLLECTION, GeeHashMapValueCollectionClass))

typedef struct _GeeHashMapValueCollection GeeHashMapValueCollection;
typedef struct _GeeHashMapValueCollectionClass GeeHashMapValueCollectionClass;

#define GEE_HASH_MAP_TYPE_ENTRY_SET (gee_hash_map_entry_set_get_type ())
#define GEE_HASH_MAP_ENTRY_SET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_HASH_MAP_TYPE_ENTRY_SET, GeeHashMapEntrySet))
#define GEE_HASH_MAP_ENTRY_SET_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_HASH_MAP_TYPE_ENTRY_SET, GeeHashMapEntrySetClass))
#define GEE_HASH_MAP_IS_ENTRY_SET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_HASH_MAP_TYPE_ENTRY_SET))
#define GEE_HASH_MAP_IS_ENTRY_SET_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_HASH_MAP_TYPE_ENTRY_SET))
#define GEE_HASH_MAP_ENTRY_SET_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_HASH_MAP_TYPE_ENTRY_SET, GeeHashMapEntrySetClass))

typedef struct _GeeHashMapEntrySet GeeHashMapEntrySet;
typedef struct _GeeHashMapEntrySetClass GeeHashMapEntrySetClass;
typedef struct _GeeFunctionsHashDataFuncClosurePrivate GeeFunctionsHashDataFuncClosurePrivate;
typedef struct _GeeFunctionsEqualDataFuncClosurePrivate GeeFunctionsEqualDataFuncClosurePrivate;

#define GEE_HASH_MAP_TYPE_ENTRY (gee_hash_map_entry_get_type ())
#define GEE_HASH_MAP_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_HASH_MAP_TYPE_ENTRY, GeeHashMapEntry))
#define GEE_HASH_MAP_ENTRY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_HASH_MAP_TYPE_ENTRY, GeeHashMapEntryClass))
#define GEE_HASH_MAP_IS_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_HASH_MAP_TYPE_ENTRY))
#define GEE_HASH_MAP_IS_ENTRY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_HASH_MAP_TYPE_ENTRY))
#define GEE_HASH_MAP_ENTRY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_HASH_MAP_TYPE_ENTRY, GeeHashMapEntryClass))

typedef struct _GeeHashMapEntry GeeHashMapEntry;
typedef struct _GeeHashMapEntryClass GeeHashMapEntryClass;
typedef struct _GeeHashMapEntryPrivate GeeHashMapEntryPrivate;
enum  {
	GEE_HASH_MAP_ENTRY_0_PROPERTY,
	GEE_HASH_MAP_ENTRY_K_TYPE,
	GEE_HASH_MAP_ENTRY_K_DUP_FUNC,
	GEE_HASH_MAP_ENTRY_K_DESTROY_FUNC,
	GEE_HASH_MAP_ENTRY_V_TYPE,
	GEE_HASH_MAP_ENTRY_V_DUP_FUNC,
	GEE_HASH_MAP_ENTRY_V_DESTROY_FUNC,
	GEE_HASH_MAP_ENTRY_KEY_PROPERTY,
	GEE_HASH_MAP_ENTRY_VALUE_PROPERTY,
	GEE_HASH_MAP_ENTRY_READ_ONLY_PROPERTY,
	GEE_HASH_MAP_ENTRY_NUM_PROPERTIES
};
static GParamSpec* gee_hash_map_entry_properties[GEE_HASH_MAP_ENTRY_NUM_PROPERTIES];
typedef struct _GeeHashMapKeySetPrivate GeeHashMapKeySetPrivate;
enum  {
	GEE_HASH_MAP_KEY_SET_0_PROPERTY,
	GEE_HASH_MAP_KEY_SET_K_TYPE,
	GEE_HASH_MAP_KEY_SET_K_DUP_FUNC,
	GEE_HASH_MAP_KEY_SET_K_DESTROY_FUNC,
	GEE_HASH_MAP_KEY_SET_V_TYPE,
	GEE_HASH_MAP_KEY_SET_V_DUP_FUNC,
	GEE_HASH_MAP_KEY_SET_V_DESTROY_FUNC,
	GEE_HASH_MAP_KEY_SET_SIZE_PROPERTY,
	GEE_HASH_MAP_KEY_SET_READ_ONLY_PROPERTY,
	GEE_HASH_MAP_KEY_SET_NUM_PROPERTIES
};
static GParamSpec* gee_hash_map_key_set_properties[GEE_HASH_MAP_KEY_SET_NUM_PROPERTIES];

#define GEE_HASH_MAP_TYPE_KEY_ITERATOR (gee_hash_map_key_iterator_get_type ())
#define GEE_HASH_MAP_KEY_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_HASH_MAP_TYPE_KEY_ITERATOR, GeeHashMapKeyIterator))
#define GEE_HASH_MAP_KEY_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_HASH_MAP_TYPE_KEY_ITERATOR, GeeHashMapKeyIteratorClass))
#define GEE_HASH_MAP_IS_KEY_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_HASH_MAP_TYPE_KEY_ITERATOR))
#define GEE_HASH_MAP_IS_KEY_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_HASH_MAP_TYPE_KEY_ITERATOR))
#define GEE_HASH_MAP_KEY_ITERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_HASH_MAP_TYPE_KEY_ITERATOR, GeeHashMapKeyIteratorClass))

typedef struct _GeeHashMapKeyIterator GeeHashMapKeyIterator;
typedef struct _GeeHashMapKeyIteratorClass GeeHashMapKeyIteratorClass;
typedef struct _GeeHashMapValueCollectionPrivate GeeHashMapValueCollectionPrivate;
enum  {
	GEE_HASH_MAP_VALUE_COLLECTION_0_PROPERTY,
	GEE_HASH_MAP_VALUE_COLLECTION_K_TYPE,
	GEE_HASH_MAP_VALUE_COLLECTION_K_DUP_FUNC,
	GEE_HASH_MAP_VALUE_COLLECTION_K_DESTROY_FUNC,
	GEE_HASH_MAP_VALUE_COLLECTION_V_TYPE,
	GEE_HASH_MAP_VALUE_COLLECTION_V_DUP_FUNC,
	GEE_HASH_MAP_VALUE_COLLECTION_V_DESTROY_FUNC,
	GEE_HASH_MAP_VALUE_COLLECTION_SIZE_PROPERTY,
	GEE_HASH_MAP_VALUE_COLLECTION_READ_ONLY_PROPERTY,
	GEE_HASH_MAP_VALUE_COLLECTION_NUM_PROPERTIES
};
static GParamSpec* gee_hash_map_value_collection_properties[GEE_HASH_MAP_VALUE_COLLECTION_NUM_PROPERTIES];

#define GEE_HASH_MAP_TYPE_VALUE_ITERATOR (gee_hash_map_value_iterator_get_type ())
#define GEE_HASH_MAP_VALUE_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_HASH_MAP_TYPE_VALUE_ITERATOR, GeeHashMapValueIterator))
#define GEE_HASH_MAP_VALUE_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_HASH_MAP_TYPE_VALUE_ITERATOR, GeeHashMapValueIteratorClass))
#define GEE_HASH_MAP_IS_VALUE_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_HASH_MAP_TYPE_VALUE_ITERATOR))
#define GEE_HASH_MAP_IS_VALUE_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_HASH_MAP_TYPE_VALUE_ITERATOR))
#define GEE_HASH_MAP_VALUE_ITERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_HASH_MAP_TYPE_VALUE_ITERATOR, GeeHashMapValueIteratorClass))

typedef struct _GeeHashMapValueIterator GeeHashMapValueIterator;
typedef struct _GeeHashMapValueIteratorClass GeeHashMapValueIteratorClass;
typedef struct _GeeHashMapEntrySetPrivate GeeHashMapEntrySetPrivate;
enum  {
	GEE_HASH_MAP_ENTRY_SET_0_PROPERTY,
	GEE_HASH_MAP_ENTRY_SET_K_TYPE,
	GEE_HASH_MAP_ENTRY_SET_K_DUP_FUNC,
	GEE_HASH_MAP_ENTRY_SET_K_DESTROY_FUNC,
	GEE_HASH_MAP_ENTRY_SET_V_TYPE,
	GEE_HASH_MAP_ENTRY_SET_V_DUP_FUNC,
	GEE_HASH_MAP_ENTRY_SET_V_DESTROY_FUNC,
	GEE_HASH_MAP_ENTRY_SET_SIZE_PROPERTY,
	GEE_HASH_MAP_ENTRY_SET_READ_ONLY_PROPERTY,
	GEE_HASH_MAP_ENTRY_SET_NUM_PROPERTIES
};
static GParamSpec* gee_hash_map_entry_set_properties[GEE_HASH_MAP_ENTRY_SET_NUM_PROPERTIES];

#define GEE_HASH_MAP_TYPE_ENTRY_ITERATOR (gee_hash_map_entry_iterator_get_type ())
#define GEE_HASH_MAP_ENTRY_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_HASH_MAP_TYPE_ENTRY_ITERATOR, GeeHashMapEntryIterator))
#define GEE_HASH_MAP_ENTRY_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_HASH_MAP_TYPE_ENTRY_ITERATOR, GeeHashMapEntryIteratorClass))
#define GEE_HASH_MAP_IS_ENTRY_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_HASH_MAP_TYPE_ENTRY_ITERATOR))
#define GEE_HASH_MAP_IS_ENTRY_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_HASH_MAP_TYPE_ENTRY_ITERATOR))
#define GEE_HASH_MAP_ENTRY_ITERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_HASH_MAP_TYPE_ENTRY_ITERATOR, GeeHashMapEntryIteratorClass))

typedef struct _GeeHashMapEntryIterator GeeHashMapEntryIterator;
typedef struct _GeeHashMapEntryIteratorClass GeeHashMapEntryIteratorClass;
typedef struct _GeeHashMapNodeIteratorPrivate GeeHashMapNodeIteratorPrivate;
enum  {
	GEE_HASH_MAP_NODE_ITERATOR_0_PROPERTY,
	GEE_HASH_MAP_NODE_ITERATOR_K_TYPE,
	GEE_HASH_MAP_NODE_ITERATOR_K_DUP_FUNC,
	GEE_HASH_MAP_NODE_ITERATOR_K_DESTROY_FUNC,
	GEE_HASH_MAP_NODE_ITERATOR_V_TYPE,
	GEE_HASH_MAP_NODE_ITERATOR_V_DUP_FUNC,
	GEE_HASH_MAP_NODE_ITERATOR_V_DESTROY_FUNC,
	GEE_HASH_MAP_NODE_ITERATOR_READ_ONLY_PROPERTY,
	GEE_HASH_MAP_NODE_ITERATOR_VALID_PROPERTY,
	GEE_HASH_MAP_NODE_ITERATOR_NUM_PROPERTIES
};
static GParamSpec* gee_hash_map_node_iterator_properties[GEE_HASH_MAP_NODE_ITERATOR_NUM_PROPERTIES];
typedef struct _GeeHashMapKeyIteratorPrivate GeeHashMapKeyIteratorPrivate;
enum  {
	GEE_HASH_MAP_KEY_ITERATOR_0_PROPERTY,
	GEE_HASH_MAP_KEY_ITERATOR_K_TYPE,
	GEE_HASH_MAP_KEY_ITERATOR_K_DUP_FUNC,
	GEE_HASH_MAP_KEY_ITERATOR_K_DESTROY_FUNC,
	GEE_HASH_MAP_KEY_ITERATOR_V_TYPE,
	GEE_HASH_MAP_KEY_ITERATOR_V_DUP_FUNC,
	GEE_HASH_MAP_KEY_ITERATOR_V_DESTROY_FUNC,
	GEE_HASH_MAP_KEY_ITERATOR_NUM_PROPERTIES
};
static GParamSpec* gee_hash_map_key_iterator_properties[GEE_HASH_MAP_KEY_ITERATOR_NUM_PROPERTIES];
typedef struct _GeeHashMapMapIteratorPrivate GeeHashMapMapIteratorPrivate;
enum  {
	GEE_HASH_MAP_MAP_ITERATOR_0_PROPERTY,
	GEE_HASH_MAP_MAP_ITERATOR_K_TYPE,
	GEE_HASH_MAP_MAP_ITERATOR_K_DUP_FUNC,
	GEE_HASH_MAP_MAP_ITERATOR_K_DESTROY_FUNC,
	GEE_HASH_MAP_MAP_ITERATOR_V_TYPE,
	GEE_HASH_MAP_MAP_ITERATOR_V_DUP_FUNC,
	GEE_HASH_MAP_MAP_ITERATOR_V_DESTROY_FUNC,
	GEE_HASH_MAP_MAP_ITERATOR_MUTABLE_PROPERTY,
	GEE_HASH_MAP_MAP_ITERATOR_READ_ONLY_PROPERTY,
	GEE_HASH_MAP_MAP_ITERATOR_NUM_PROPERTIES
};
static GParamSpec* gee_hash_map_map_iterator_properties[GEE_HASH_MAP_MAP_ITERATOR_NUM_PROPERTIES];
typedef struct _GeeHashMapValueIteratorPrivate GeeHashMapValueIteratorPrivate;
enum  {
	GEE_HASH_MAP_VALUE_ITERATOR_0_PROPERTY,
	GEE_HASH_MAP_VALUE_ITERATOR_K_TYPE,
	GEE_HASH_MAP_VALUE_ITERATOR_K_DUP_FUNC,
	GEE_HASH_MAP_VALUE_ITERATOR_K_DESTROY_FUNC,
	GEE_HASH_MAP_VALUE_ITERATOR_V_TYPE,
	GEE_HASH_MAP_VALUE_ITERATOR_V_DUP_FUNC,
	GEE_HASH_MAP_VALUE_ITERATOR_V_DESTROY_FUNC,
	GEE_HASH_MAP_VALUE_ITERATOR_NUM_PROPERTIES
};
static GParamSpec* gee_hash_map_value_iterator_properties[GEE_HASH_MAP_VALUE_ITERATOR_NUM_PROPERTIES];
typedef struct _GeeHashMapEntryIteratorPrivate GeeHashMapEntryIteratorPrivate;
enum  {
	GEE_HASH_MAP_ENTRY_ITERATOR_0_PROPERTY,
	GEE_HASH_MAP_ENTRY_ITERATOR_K_TYPE,
	GEE_HASH_MAP_ENTRY_ITERATOR_K_DUP_FUNC,
	GEE_HASH_MAP_ENTRY_ITERATOR_K_DESTROY_FUNC,
	GEE_HASH_MAP_ENTRY_ITERATOR_V_TYPE,
	GEE_HASH_MAP_ENTRY_ITERATOR_V_DUP_FUNC,
	GEE_HASH_MAP_ENTRY_ITERATOR_V_DESTROY_FUNC,
	GEE_HASH_MAP_ENTRY_ITERATOR_NUM_PROPERTIES
};
static GParamSpec* gee_hash_map_entry_iterator_properties[GEE_HASH_MAP_ENTRY_ITERATOR_NUM_PROPERTIES];
#define _vala_assert(expr, msg) if G_LIKELY (expr) ; else g_assertion_message_expr (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);
#define _vala_return_if_fail(expr, msg) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return; }
#define _vala_return_val_if_fail(expr, msg, val) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return val; }
#define _vala_warn_if_fail(expr, msg) if G_LIKELY (expr) ; else g_warn_message (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);

struct _GeeHashMapPrivate {
	GType k_type;
	GBoxedCopyFunc k_dup_func;
	GDestroyNotify k_destroy_func;
	GType v_type;
	GBoxedCopyFunc v_dup_func;
	GDestroyNotify v_destroy_func;
	gint _array_size;
	gint _nnodes;
	GeeHashMapNode** _nodes;
	gint _nodes_length1;
	gint __nodes_size_;
	GeeFunctionsHashDataFuncClosure* _key_hash_func;
	GeeFunctionsEqualDataFuncClosure* _key_equal_func;
	GeeFunctionsEqualDataFuncClosure* _value_equal_func;
	GeeSet* _keys;
	GeeCollection* _values;
	GeeSet* _entries;
	gint _stamp;
};

struct _GeeHashMapNode {
	gpointer key;
	gpointer value;
	GeeHashMapNode* next;
	guint key_hash;
	GeeMapEntry* entry;
};

struct _GeeFunctionsHashDataFuncClosure {
	GTypeInstance parent_instance;
	volatile int ref_count;
	GeeFunctionsHashDataFuncClosurePrivate * priv;
	GeeHashDataFunc func;
	gpointer func_target;
	GDestroyNotify func_target_destroy_notify;
};

struct _GeeFunctionsHashDataFuncClosureClass {
	GTypeClass parent_class;
	void (*finalize) (GeeFunctionsHashDataFuncClosure *self);
};

struct _GeeFunctionsEqualDataFuncClosure {
	GTypeInstance parent_instance;
	volatile int ref_count;
	GeeFunctionsEqualDataFuncClosurePrivate * priv;
	GeeEqualDataFunc func;
	gpointer func_target;
	GDestroyNotify func_target_destroy_notify;
};

struct _GeeFunctionsEqualDataFuncClosureClass {
	GTypeClass parent_class;
	void (*finalize) (GeeFunctionsEqualDataFuncClosure *self);
};

struct _GeeHashMapEntry {
	GeeMapEntry parent_instance;
	GeeHashMapEntryPrivate * priv;
};

struct _GeeHashMapEntryClass {
	GeeMapEntryClass parent_class;
};

struct _GeeHashMapEntryPrivate {
	GType k_type;
	GBoxedCopyFunc k_dup_func;
	GDestroyNotify k_destroy_func;
	GType v_type;
	GBoxedCopyFunc v_dup_func;
	GDestroyNotify v_destroy_func;
	GeeHashMapNode* _node;
};

struct _GeeHashMapKeySet {
	GeeAbstractSet parent_instance;
	GeeHashMapKeySetPrivate * priv;
};

struct _GeeHashMapKeySetClass {
	GeeAbstractSetClass parent_class;
};

struct _GeeHashMapKeySetPrivate {
	GType k_type;
	GBoxedCopyFunc k_dup_func;
	GDestroyNotify k_destroy_func;
	GType v_type;
	GBoxedCopyFunc v_dup_func;
	GDestroyNotify v_destroy_func;
	GeeHashMap* _map;
};

struct _GeeHashMapValueCollection {
	GeeAbstractCollection parent_instance;
	GeeHashMapValueCollectionPrivate * priv;
};

struct _GeeHashMapValueCollectionClass {
	GeeAbstractCollectionClass parent_class;
};

struct _GeeHashMapValueCollectionPrivate {
	GType k_type;
	GBoxedCopyFunc k_dup_func;
	GDestroyNotify k_destroy_func;
	GType v_type;
	GBoxedCopyFunc v_dup_func;
	GDestroyNotify v_destroy_func;
	GeeHashMap* _map;
};

struct _GeeHashMapEntrySet {
	GeeAbstractSet parent_instance;
	GeeHashMapEntrySetPrivate * priv;
};

struct _GeeHashMapEntrySetClass {
	GeeAbstractSetClass parent_class;
};

struct _GeeHashMapEntrySetPrivate {
	GType k_type;
	GBoxedCopyFunc k_dup_func;
	GDestroyNotify k_destroy_func;
	GType v_type;
	GBoxedCopyFunc v_dup_func;
	GDestroyNotify v_destroy_func;
	GeeHashMap* _map;
};

struct _GeeHashMapNodeIterator {
	GObject parent_instance;
	GeeHashMapNodeIteratorPrivate * priv;
	GeeHashMap* _map;
	gint _index;
	GeeHashMapNode* _node;
	GeeHashMapNode* _next;
	gint _stamp;
};

struct _GeeHashMapNodeIteratorClass {
	GObjectClass parent_class;
	gboolean (*get_read_only) (GeeHashMapNodeIterator* self);
};

struct _GeeHashMapNodeIteratorPrivate {
	GType k_type;
	GBoxedCopyFunc k_dup_func;
	GDestroyNotify k_destroy_func;
	GType v_type;
	GBoxedCopyFunc v_dup_func;
	GDestroyNotify v_destroy_func;
};

struct _GeeHashMapKeyIterator {
	GeeHashMapNodeIterator parent_instance;
	GeeHashMapKeyIteratorPrivate * priv;
};

struct _GeeHashMapKeyIteratorClass {
	GeeHashMapNodeIteratorClass parent_class;
};

struct _GeeHashMapKeyIteratorPrivate {
	GType k_type;
	GBoxedCopyFunc k_dup_func;
	GDestroyNotify k_destroy_func;
	GType v_type;
	GBoxedCopyFunc v_dup_func;
	GDestroyNotify v_destroy_func;
};

struct _GeeHashMapMapIterator {
	GeeHashMapNodeIterator parent_instance;
	GeeHashMapMapIteratorPrivate * priv;
};

struct _GeeHashMapMapIteratorClass {
	GeeHashMapNodeIteratorClass parent_class;
};

struct _GeeHashMapMapIteratorPrivate {
	GType k_type;
	GBoxedCopyFunc k_dup_func;
	GDestroyNotify k_destroy_func;
	GType v_type;
	GBoxedCopyFunc v_dup_func;
	GDestroyNotify v_destroy_func;
};

struct _GeeHashMapValueIterator {
	GeeHashMapNodeIterator parent_instance;
	GeeHashMapValueIteratorPrivate * priv;
};

struct _GeeHashMapValueIteratorClass {
	GeeHashMapNodeIteratorClass parent_class;
};

struct _GeeHashMapValueIteratorPrivate {
	GType k_type;
	GBoxedCopyFunc k_dup_func;
	GDestroyNotify k_destroy_func;
	GType v_type;
	GBoxedCopyFunc v_dup_func;
	GDestroyNotify v_destroy_func;
};

struct _GeeHashMapEntryIterator {
	GeeHashMapNodeIterator parent_instance;
	GeeHashMapEntryIteratorPrivate * priv;
};

struct _GeeHashMapEntryIteratorClass {
	GeeHashMapNodeIteratorClass parent_class;
};

struct _GeeHashMapEntryIteratorPrivate {
	GType k_type;
	GBoxedCopyFunc k_dup_func;
	GDestroyNotify k_destroy_func;
	GType v_type;
	GBoxedCopyFunc v_dup_func;
	GDestroyNotify v_destroy_func;
};

static gint GeeHashMap_private_offset;
static gpointer gee_hash_map_parent_class = NULL;
static gint GeeHashMapEntry_private_offset;
static gpointer gee_hash_map_entry_parent_class = NULL;
static gint GeeHashMapKeySet_private_offset;
static gpointer gee_hash_map_key_set_parent_class = NULL;
static gint GeeHashMapValueCollection_private_offset;
static gpointer gee_hash_map_value_collection_parent_class = NULL;
static gint GeeHashMapEntrySet_private_offset;
static gpointer gee_hash_map_entry_set_parent_class = NULL;
static gint GeeHashMapNodeIterator_private_offset;
static gpointer gee_hash_map_node_iterator_parent_class = NULL;
static gint GeeHashMapKeyIterator_private_offset;
static gpointer gee_hash_map_key_iterator_parent_class = NULL;
static GeeTraversableIface * gee_hash_map_key_iterator_gee_traversable_parent_iface = NULL;
static GeeIteratorIface * gee_hash_map_key_iterator_gee_iterator_parent_iface = NULL;
static gint GeeHashMapMapIterator_private_offset;
static gpointer gee_hash_map_map_iterator_parent_class = NULL;
static GeeMapIteratorIface * gee_hash_map_map_iterator_gee_map_iterator_parent_iface = NULL;
static gint GeeHashMapValueIterator_private_offset;
static gpointer gee_hash_map_value_iterator_parent_class = NULL;
static GeeTraversableIface * gee_hash_map_value_iterator_gee_traversable_parent_iface = NULL;
static GeeIteratorIface * gee_hash_map_value_iterator_gee_iterator_parent_iface = NULL;
static gint GeeHashMapEntryIterator_private_offset;
static gpointer gee_hash_map_entry_iterator_parent_class = NULL;
static GeeTraversableIface * gee_hash_map_entry_iterator_gee_traversable_parent_iface = NULL;
static GeeIteratorIface * gee_hash_map_entry_iterator_gee_iterator_parent_iface = NULL;

static void gee_hash_map_node_free (GeeHashMapNode * self);
 G_GNUC_INTERNAL gpointer gee_functions_hash_data_func_closure_ref (gpointer instance);
 G_GNUC_INTERNAL void gee_functions_hash_data_func_closure_unref (gpointer instance);
 G_GNUC_INTERNAL GParamSpec* gee_functions_param_spec_hash_data_func_closure (const gchar* name,
                                                             const gchar* nick,
                                                             const gchar* blurb,
                                                             GType object_type,
                                                             GParamFlags flags);
 G_GNUC_INTERNAL void gee_functions_value_set_hash_data_func_closure (GValue* value,
                                                     gpointer v_object) G_GNUC_UNUSED ;
 G_GNUC_INTERNAL void gee_functions_value_take_hash_data_func_closure (GValue* value,
                                                      gpointer v_object);
 G_GNUC_INTERNAL gpointer gee_functions_value_get_hash_data_func_closure (const GValue* value) G_GNUC_UNUSED ;
 G_GNUC_INTERNAL GType gee_functions_hash_data_func_closure_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
 G_GNUC_INTERNAL gpointer gee_functions_equal_data_func_closure_ref (gpointer instance);
 G_GNUC_INTERNAL void gee_functions_equal_data_func_closure_unref (gpointer instance);
 G_GNUC_INTERNAL GParamSpec* gee_functions_param_spec_equal_data_func_closure (const gchar* name,
                                                              const gchar* nick,
                                                              const gchar* blurb,
                                                              GType object_type,
                                                              GParamFlags flags);
 G_GNUC_INTERNAL void gee_functions_value_set_equal_data_func_closure (GValue* value,
                                                      gpointer v_object) G_GNUC_UNUSED ;
 G_GNUC_INTERNAL void gee_functions_value_take_equal_data_func_closure (GValue* value,
                                                       gpointer v_object);
 G_GNUC_INTERNAL gpointer gee_functions_value_get_equal_data_func_closure (const GValue* value) G_GNUC_UNUSED ;
 G_GNUC_INTERNAL GType gee_functions_equal_data_func_closure_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
 G_GNUC_INTERNAL GeeFunctionsHashDataFuncClosure* gee_functions_hash_data_func_closure_new (GType g_type,
                                                                           GBoxedCopyFunc g_dup_func,
                                                                           GDestroyNotify g_destroy_func,
                                                                           GeeHashDataFunc func,
                                                                           gpointer func_target,
                                                                           GDestroyNotify func_target_destroy_notify);
 G_GNUC_INTERNAL GeeFunctionsHashDataFuncClosure* gee_functions_hash_data_func_closure_construct (GType object_type,
                                                                                 GType g_type,
                                                                                 GBoxedCopyFunc g_dup_func,
                                                                                 GDestroyNotify g_destroy_func,
                                                                                 GeeHashDataFunc func,
                                                                                 gpointer func_target,
                                                                                 GDestroyNotify func_target_destroy_notify);
 G_GNUC_INTERNAL GeeFunctionsEqualDataFuncClosure* gee_functions_equal_data_func_closure_new (GType g_type,
                                                                             GBoxedCopyFunc g_dup_func,
                                                                             GDestroyNotify g_destroy_func,
                                                                             GeeEqualDataFunc func,
                                                                             gpointer func_target,
                                                                             GDestroyNotify func_target_destroy_notify);
 G_GNUC_INTERNAL GeeFunctionsEqualDataFuncClosure* gee_functions_equal_data_func_closure_construct (GType object_type,
                                                                                   GType g_type,
                                                                                   GBoxedCopyFunc g_dup_func,
                                                                                   GDestroyNotify g_destroy_func,
                                                                                   GeeEqualDataFunc func,
                                                                                   gpointer func_target,
                                                                                   GDestroyNotify func_target_destroy_notify);
 G_GNUC_INTERNAL GeeHashMap* gee_hash_map_new_with_closures (GType k_type,
                                            GBoxedCopyFunc k_dup_func,
                                            GDestroyNotify k_destroy_func,
                                            GType v_type,
                                            GBoxedCopyFunc v_dup_func,
                                            GDestroyNotify v_destroy_func,
                                            GeeFunctionsHashDataFuncClosure* key_hash_func,
                                            GeeFunctionsEqualDataFuncClosure* key_equal_func,
                                            GeeFunctionsEqualDataFuncClosure* value_equal_func);
 G_GNUC_INTERNAL GeeHashMap* gee_hash_map_construct_with_closures (GType object_type,
                                                  GType k_type,
                                                  GBoxedCopyFunc k_dup_func,
                                                  GDestroyNotify k_destroy_func,
                                                  GType v_type,
                                                  GBoxedCopyFunc v_dup_func,
                                                  GDestroyNotify v_destroy_func,
                                                  GeeFunctionsHashDataFuncClosure* key_hash_func,
                                                  GeeFunctionsEqualDataFuncClosure* key_equal_func,
                                                  GeeFunctionsEqualDataFuncClosure* value_equal_func);
static GeeHashMapNode** gee_hash_map_lookup_node (GeeHashMap* self,
                                           gconstpointer key);
static gboolean gee_hash_map_real_has_key (GeeAbstractMap* base,
                                    gconstpointer key);
static gboolean gee_hash_map_real_has (GeeAbstractMap* base,
                                gconstpointer key,
                                gconstpointer value);
static gpointer gee_hash_map_real_get (GeeAbstractMap* base,
                                gconstpointer key);
static void gee_hash_map_real_set (GeeAbstractMap* base,
                            gconstpointer key,
                            gconstpointer value);
static GeeHashMapNode* gee_hash_map_node_new (gpointer k,
                                       gpointer v,
                                       guint hash);
static inline void gee_hash_map_resize (GeeHashMap* self);
static gboolean gee_hash_map_real_unset (GeeAbstractMap* base,
                                  gconstpointer key,
                                  gpointer* value);
static inline gboolean gee_hash_map_unset_helper (GeeHashMap* self,
                                    gconstpointer key,
                                    gpointer* value);
static void gee_hash_map_real_clear (GeeAbstractMap* base);
static GeeMapIterator* gee_hash_map_real_map_iterator (GeeAbstractMap* base);
static GeeHashMapMapIterator* gee_hash_map_map_iterator_new (GType k_type,
                                                      GBoxedCopyFunc k_dup_func,
                                                      GDestroyNotify k_destroy_func,
                                                      GType v_type,
                                                      GBoxedCopyFunc v_dup_func,
                                                      GDestroyNotify v_destroy_func,
                                                      GeeHashMap* map);
static GeeHashMapMapIterator* gee_hash_map_map_iterator_construct (GType object_type,
                                                            GType k_type,
                                                            GBoxedCopyFunc k_dup_func,
                                                            GDestroyNotify k_destroy_func,
                                                            GType v_type,
                                                            GBoxedCopyFunc v_dup_func,
                                                            GDestroyNotify v_destroy_func,
                                                            GeeHashMap* map);
static GType gee_hash_map_node_iterator_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
static GType gee_hash_map_map_iterator_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
 G_GNUC_INTERNAL GeeFunctionsHashDataFuncClosure* gee_hash_map_get_key_hash_func_closure (GeeHashMap* self);
 G_GNUC_INTERNAL GeeFunctionsEqualDataFuncClosure* gee_hash_map_get_key_equal_func_closure (GeeHashMap* self);
static GeeHashMapKeySet* gee_hash_map_key_set_new (GType k_type,
                                            GBoxedCopyFunc k_dup_func,
                                            GDestroyNotify k_destroy_func,
                                            GType v_type,
                                            GBoxedCopyFunc v_dup_func,
                                            GDestroyNotify v_destroy_func,
                                            GeeHashMap* map);
static GeeHashMapKeySet* gee_hash_map_key_set_construct (GType object_type,
                                                  GType k_type,
                                                  GBoxedCopyFunc k_dup_func,
                                                  GDestroyNotify k_destroy_func,
                                                  GType v_type,
                                                  GBoxedCopyFunc v_dup_func,
                                                  GDestroyNotify v_destroy_func,
                                                  GeeHashMap* map);
static GType gee_hash_map_key_set_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
static GeeHashMapValueCollection* gee_hash_map_value_collection_new (GType k_type,
                                                              GBoxedCopyFunc k_dup_func,
                                                              GDestroyNotify k_destroy_func,
                                                              GType v_type,
                                                              GBoxedCopyFunc v_dup_func,
                                                              GDestroyNotify v_destroy_func,
                                                              GeeHashMap* map);
static GeeHashMapValueCollection* gee_hash_map_value_collection_construct (GType object_type,
                                                                    GType k_type,
                                                                    GBoxedCopyFunc k_dup_func,
                                                                    GDestroyNotify k_destroy_func,
                                                                    GType v_type,
                                                                    GBoxedCopyFunc v_dup_func,
                                                                    GDestroyNotify v_destroy_func,
                                                                    GeeHashMap* map);
static GType gee_hash_map_value_collection_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
static GeeHashMapEntrySet* gee_hash_map_entry_set_new (GType k_type,
                                                GBoxedCopyFunc k_dup_func,
                                                GDestroyNotify k_destroy_func,
                                                GType v_type,
                                                GBoxedCopyFunc v_dup_func,
                                                GDestroyNotify v_destroy_func,
                                                GeeHashMap* map);
static GeeHashMapEntrySet* gee_hash_map_entry_set_construct (GType object_type,
                                                      GType k_type,
                                                      GBoxedCopyFunc k_dup_func,
                                                      GDestroyNotify k_destroy_func,
                                                      GType v_type,
                                                      GBoxedCopyFunc v_dup_func,
                                                      GDestroyNotify v_destroy_func,
                                                      GeeHashMap* map);
static GType gee_hash_map_entry_set_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
static void gee_hash_map_set_key_hash_func (GeeHashMap* self,
                                     GeeHashDataFunc value,
                                     gpointer value_target);
static void gee_hash_map_set_key_equal_func (GeeHashMap* self,
                                      GeeEqualDataFunc value,
                                      gpointer value_target);
static void gee_hash_map_set_value_equal_func (GeeHashMap* self,
                                        GeeEqualDataFunc value,
                                        gpointer value_target);
static void gee_hash_map_node_instance_init (GeeHashMapNode * self);
static GType gee_hash_map_entry_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
static GeeMapEntry* gee_hash_map_entry_entry_for (GType k_type,
                                           GBoxedCopyFunc k_dup_func,
                                           GDestroyNotify k_destroy_func,
                                           GType v_type,
                                           GBoxedCopyFunc v_dup_func,
                                           GDestroyNotify v_destroy_func,
                                           GeeHashMapNode* node);
static GeeHashMapEntry* gee_hash_map_entry_new (GType k_type,
                                         GBoxedCopyFunc k_dup_func,
                                         GDestroyNotify k_destroy_func,
                                         GType v_type,
                                         GBoxedCopyFunc v_dup_func,
                                         GDestroyNotify v_destroy_func,
                                         GeeHashMapNode* node);
static GeeHashMapEntry* gee_hash_map_entry_construct (GType object_type,
                                               GType k_type,
                                               GBoxedCopyFunc k_dup_func,
                                               GDestroyNotify k_destroy_func,
                                               GType v_type,
                                               GBoxedCopyFunc v_dup_func,
                                               GDestroyNotify v_destroy_func,
                                               GeeHashMapNode* node);
static void gee_hash_map_entry_finalize (GObject * obj);
static GType gee_hash_map_entry_get_type_once (void);
static void _vala_gee_hash_map_entry_get_property (GObject * object,
                                            guint property_id,
                                            GValue * value,
                                            GParamSpec * pspec);
static void _vala_gee_hash_map_entry_set_property (GObject * object,
                                            guint property_id,
                                            const GValue * value,
                                            GParamSpec * pspec);
static GeeIterator* gee_hash_map_key_set_real_iterator (GeeAbstractCollection* base);
static GeeHashMapKeyIterator* gee_hash_map_key_iterator_new (GType k_type,
                                                      GBoxedCopyFunc k_dup_func,
                                                      GDestroyNotify k_destroy_func,
                                                      GType v_type,
                                                      GBoxedCopyFunc v_dup_func,
                                                      GDestroyNotify v_destroy_func,
                                                      GeeHashMap* map);
static GeeHashMapKeyIterator* gee_hash_map_key_iterator_construct (GType object_type,
                                                            GType k_type,
                                                            GBoxedCopyFunc k_dup_func,
                                                            GDestroyNotify k_destroy_func,
                                                            GType v_type,
                                                            GBoxedCopyFunc v_dup_func,
                                                            GDestroyNotify v_destroy_func,
                                                            GeeHashMap* map);
static GType gee_hash_map_key_iterator_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
static gboolean gee_hash_map_key_set_real_add (GeeAbstractCollection* base,
                                        gconstpointer key);
static void gee_hash_map_key_set_real_clear (GeeAbstractCollection* base);
static gboolean gee_hash_map_key_set_real_remove (GeeAbstractCollection* base,
                                           gconstpointer key);
static gboolean gee_hash_map_key_set_real_contains (GeeAbstractCollection* base,
                                             gconstpointer key);
static void gee_hash_map_key_set_finalize (GObject * obj);
static GType gee_hash_map_key_set_get_type_once (void);
static void _vala_gee_hash_map_key_set_get_property (GObject * object,
                                              guint property_id,
                                              GValue * value,
                                              GParamSpec * pspec);
static void _vala_gee_hash_map_key_set_set_property (GObject * object,
                                              guint property_id,
                                              const GValue * value,
                                              GParamSpec * pspec);
static GeeIterator* gee_hash_map_value_collection_real_iterator (GeeAbstractCollection* base);
static GeeHashMapValueIterator* gee_hash_map_value_iterator_new (GType k_type,
                                                          GBoxedCopyFunc k_dup_func,
                                                          GDestroyNotify k_destroy_func,
                                                          GType v_type,
                                                          GBoxedCopyFunc v_dup_func,
                                                          GDestroyNotify v_destroy_func,
                                                          GeeHashMap* map);
static GeeHashMapValueIterator* gee_hash_map_value_iterator_construct (GType object_type,
                                                                GType k_type,
                                                                GBoxedCopyFunc k_dup_func,
                                                                GDestroyNotify k_destroy_func,
                                                                GType v_type,
                                                                GBoxedCopyFunc v_dup_func,
                                                                GDestroyNotify v_destroy_func,
                                                                GeeHashMap* map);
static GType gee_hash_map_value_iterator_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
static gboolean gee_hash_map_value_collection_real_add (GeeAbstractCollection* base,
                                                 gconstpointer value);
static void gee_hash_map_value_collection_real_clear (GeeAbstractCollection* base);
static gboolean gee_hash_map_value_collection_real_remove (GeeAbstractCollection* base,
                                                    gconstpointer value);
static gboolean gee_hash_map_value_collection_real_contains (GeeAbstractCollection* base,
                                                      gconstpointer value);
static void gee_hash_map_value_collection_finalize (GObject * obj);
static GType gee_hash_map_value_collection_get_type_once (void);
static void _vala_gee_hash_map_value_collection_get_property (GObject * object,
                                                       guint property_id,
                                                       GValue * value,
                                                       GParamSpec * pspec);
static void _vala_gee_hash_map_value_collection_set_property (GObject * object,
                                                       guint property_id,
                                                       const GValue * value,
                                                       GParamSpec * pspec);
static GeeIterator* gee_hash_map_entry_set_real_iterator (GeeAbstractCollection* base);
static GeeHashMapEntryIterator* gee_hash_map_entry_iterator_new (GType k_type,
                                                          GBoxedCopyFunc k_dup_func,
                                                          GDestroyNotify k_destroy_func,
                                                          GType v_type,
                                                          GBoxedCopyFunc v_dup_func,
                                                          GDestroyNotify v_destroy_func,
                                                          GeeHashMap* map);
static GeeHashMapEntryIterator* gee_hash_map_entry_iterator_construct (GType object_type,
                                                                GType k_type,
                                                                GBoxedCopyFunc k_dup_func,
                                                                GDestroyNotify k_destroy_func,
                                                                GType v_type,
                                                                GBoxedCopyFunc v_dup_func,
                                                                GDestroyNotify v_destroy_func,
                                                                GeeHashMap* map);
static GType gee_hash_map_entry_iterator_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
static gboolean gee_hash_map_entry_set_real_add (GeeAbstractCollection* base,
                                          GeeMapEntry* entry);
static void gee_hash_map_entry_set_real_clear (GeeAbstractCollection* base);
static gboolean gee_hash_map_entry_set_real_remove (GeeAbstractCollection* base,
                                             GeeMapEntry* entry);
static gboolean gee_hash_map_entry_set_real_contains (GeeAbstractCollection* base,
                                               GeeMapEntry* entry);
static void gee_hash_map_entry_set_finalize (GObject * obj);
static GType gee_hash_map_entry_set_get_type_once (void);
static void _vala_gee_hash_map_entry_set_get_property (GObject * object,
                                                guint property_id,
                                                GValue * value,
                                                GParamSpec * pspec);
static void _vala_gee_hash_map_entry_set_set_property (GObject * object,
                                                guint property_id,
                                                const GValue * value,
                                                GParamSpec * pspec);
static gboolean gee_hash_map_node_iterator_get_read_only (GeeHashMapNodeIterator* self);
static GeeHashMapNodeIterator* gee_hash_map_node_iterator_construct (GType object_type,
                                                              GType k_type,
                                                              GBoxedCopyFunc k_dup_func,
                                                              GDestroyNotify k_destroy_func,
                                                              GType v_type,
                                                              GBoxedCopyFunc v_dup_func,
                                                              GDestroyNotify v_destroy_func,
                                                              GeeHashMap* map);
static GeeHashMapNodeIterator* gee_hash_map_node_iterator_construct_from_iterator (GType object_type,
                                                                            GType k_type,
                                                                            GBoxedCopyFunc k_dup_func,
                                                                            GDestroyNotify k_destroy_func,
                                                                            GType v_type,
                                                                            GBoxedCopyFunc v_dup_func,
                                                                            GDestroyNotify v_destroy_func,
                                                                            GeeHashMapNodeIterator* iter);
static gboolean gee_hash_map_node_iterator_next (GeeHashMapNodeIterator* self);
static gboolean gee_hash_map_node_iterator_has_next (GeeHashMapNodeIterator* self);
static gboolean gee_hash_map_node_iterator_get_valid (GeeHashMapNodeIterator* self);
static void gee_hash_map_node_iterator_finalize (GObject * obj);
static GType gee_hash_map_node_iterator_get_type_once (void);
static void _vala_gee_hash_map_node_iterator_get_property (GObject * object,
                                                    guint property_id,
                                                    GValue * value,
                                                    GParamSpec * pspec);
static void _vala_gee_hash_map_node_iterator_set_property (GObject * object,
                                                    guint property_id,
                                                    const GValue * value,
                                                    GParamSpec * pspec);
static GeeHashMapKeyIterator* gee_hash_map_key_iterator_new_from_iterator (GType k_type,
                                                                    GBoxedCopyFunc k_dup_func,
                                                                    GDestroyNotify k_destroy_func,
                                                                    GType v_type,
                                                                    GBoxedCopyFunc v_dup_func,
                                                                    GDestroyNotify v_destroy_func,
                                                                    GeeHashMapKeyIterator* iter);
static GeeHashMapKeyIterator* gee_hash_map_key_iterator_construct_from_iterator (GType object_type,
                                                                          GType k_type,
                                                                          GBoxedCopyFunc k_dup_func,
                                                                          GDestroyNotify k_destroy_func,
                                                                          GType v_type,
                                                                          GBoxedCopyFunc v_dup_func,
                                                                          GDestroyNotify v_destroy_func,
                                                                          GeeHashMapKeyIterator* iter);
static gpointer gee_hash_map_key_iterator_real_get (GeeIterator* base);
static void gee_hash_map_key_iterator_real_remove (GeeIterator* base);
static gboolean gee_hash_map_key_iterator_real_foreach (GeeTraversable* base,
                                                 GeeForallFunc f,
                                                 gpointer f_target);
static GeeIterator** gee_hash_map_key_iterator_real_tee (GeeTraversable* base,
                                                  guint forks,
                                                  gint* result_length1);
static GType gee_hash_map_key_iterator_get_type_once (void);
static void _vala_gee_hash_map_key_iterator_get_property (GObject * object,
                                                   guint property_id,
                                                   GValue * value,
                                                   GParamSpec * pspec);
static void _vala_gee_hash_map_key_iterator_set_property (GObject * object,
                                                   guint property_id,
                                                   const GValue * value,
                                                   GParamSpec * pspec);
static gpointer gee_hash_map_map_iterator_real_get_key (GeeMapIterator* base);
static void gee_hash_map_map_iterator_real_unset (GeeMapIterator* base);
static gpointer gee_hash_map_map_iterator_real_get_value (GeeMapIterator* base);
static void gee_hash_map_map_iterator_real_set_value (GeeMapIterator* base,
                                               gconstpointer value);
static GType gee_hash_map_map_iterator_get_type_once (void);
static void _vala_gee_hash_map_map_iterator_get_property (GObject * object,
                                                   guint property_id,
                                                   GValue * value,
                                                   GParamSpec * pspec);
static void _vala_gee_hash_map_map_iterator_set_property (GObject * object,
                                                   guint property_id,
                                                   const GValue * value,
                                                   GParamSpec * pspec);
static GeeHashMapValueIterator* gee_hash_map_value_iterator_new_from_iterator (GType k_type,
                                                                        GBoxedCopyFunc k_dup_func,
                                                                        GDestroyNotify k_destroy_func,
                                                                        GType v_type,
                                                                        GBoxedCopyFunc v_dup_func,
                                                                        GDestroyNotify v_destroy_func,
                                                                        GeeHashMapValueIterator* iter);
static GeeHashMapValueIterator* gee_hash_map_value_iterator_construct_from_iterator (GType object_type,
                                                                              GType k_type,
                                                                              GBoxedCopyFunc k_dup_func,
                                                                              GDestroyNotify k_destroy_func,
                                                                              GType v_type,
                                                                              GBoxedCopyFunc v_dup_func,
                                                                              GDestroyNotify v_destroy_func,
                                                                              GeeHashMapValueIterator* iter);
static gpointer gee_hash_map_value_iterator_real_get (GeeIterator* base);
static void gee_hash_map_value_iterator_real_remove (GeeIterator* base);
static gboolean gee_hash_map_value_iterator_real_foreach (GeeTraversable* base,
                                                   GeeForallFunc f,
                                                   gpointer f_target);
static GeeIterator** gee_hash_map_value_iterator_real_tee (GeeTraversable* base,
                                                    guint forks,
                                                    gint* result_length1);
static GType gee_hash_map_value_iterator_get_type_once (void);
static void _vala_gee_hash_map_value_iterator_get_property (GObject * object,
                                                     guint property_id,
                                                     GValue * value,
                                                     GParamSpec * pspec);
static void _vala_gee_hash_map_value_iterator_set_property (GObject * object,
                                                     guint property_id,
                                                     const GValue * value,
                                                     GParamSpec * pspec);
static GeeHashMapEntryIterator* gee_hash_map_entry_iterator_new_from_iterator (GType k_type,
                                                                        GBoxedCopyFunc k_dup_func,
                                                                        GDestroyNotify k_destroy_func,
                                                                        GType v_type,
                                                                        GBoxedCopyFunc v_dup_func,
                                                                        GDestroyNotify v_destroy_func,
                                                                        GeeHashMapEntryIterator* iter);
static GeeHashMapEntryIterator* gee_hash_map_entry_iterator_construct_from_iterator (GType object_type,
                                                                              GType k_type,
                                                                              GBoxedCopyFunc k_dup_func,
                                                                              GDestroyNotify k_destroy_func,
                                                                              GType v_type,
                                                                              GBoxedCopyFunc v_dup_func,
                                                                              GDestroyNotify v_destroy_func,
                                                                              GeeHashMapEntryIterator* iter);
static GeeMapEntry* gee_hash_map_entry_iterator_real_get (GeeIterator* base);
static void gee_hash_map_entry_iterator_real_remove (GeeIterator* base);
static gboolean gee_hash_map_entry_iterator_real_foreach (GeeTraversable* base,
                                                   GeeForallFunc f,
                                                   gpointer f_target);
static GeeIterator** gee_hash_map_entry_iterator_real_tee (GeeTraversable* base,
                                                    guint forks,
                                                    gint* result_length1);
static GType gee_hash_map_entry_iterator_get_type_once (void);
static void _vala_gee_hash_map_entry_iterator_get_property (GObject * object,
                                                     guint property_id,
                                                     GValue * value,
                                                     GParamSpec * pspec);
static void _vala_gee_hash_map_entry_iterator_set_property (GObject * object,
                                                     guint property_id,
                                                     const GValue * value,
                                                     GParamSpec * pspec);
static void gee_hash_map_finalize (GObject * obj);
static GType gee_hash_map_get_type_once (void);
static void _vala_gee_hash_map_get_property (GObject * object,
                                      guint property_id,
                                      GValue * value,
                                      GParamSpec * pspec);
static void _vala_gee_hash_map_set_property (GObject * object,
                                      guint property_id,
                                      const GValue * value,
                                      GParamSpec * pspec);
static void _vala_array_destroy (gpointer array,
                          gssize array_length,
                          GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array,
                       gssize array_length,
                       GDestroyNotify destroy_func);

static inline gpointer
gee_hash_map_get_instance_private (GeeHashMap* self)
{
	return G_STRUCT_MEMBER_P (self, GeeHashMap_private_offset);
}

/**
 * Constructs a new, empty hash map.
 *
 * If not provided, the functions parameters are requested to the
 * {@link Functions} function factory methods.
 *
 * @param key_hash_func an optional key hash function
 * @param key_equal_func an optional key equality testing function
 * @param value_equal_func an optional value equality testing function
 */
GeeHashMap*
gee_hash_map_construct (GType object_type,
                        GType k_type,
                        GBoxedCopyFunc k_dup_func,
                        GDestroyNotify k_destroy_func,
                        GType v_type,
                        GBoxedCopyFunc v_dup_func,
                        GDestroyNotify v_destroy_func,
                        GeeHashDataFunc key_hash_func,
                        gpointer key_hash_func_target,
                        GDestroyNotify key_hash_func_target_destroy_notify,
                        GeeEqualDataFunc key_equal_func,
                        gpointer key_equal_func_target,
                        GDestroyNotify key_equal_func_target_destroy_notify,
                        GeeEqualDataFunc value_equal_func,
                        gpointer value_equal_func_target,
                        GDestroyNotify value_equal_func_target_destroy_notify)
{
	GeeHashMap * self = NULL;
	GeeHashDataFunc _tmp9_;
	gpointer _tmp9__target;
	GDestroyNotify _tmp9__target_destroy_notify;
	GeeFunctionsHashDataFuncClosure* _tmp10_;
	GeeEqualDataFunc _tmp11_;
	gpointer _tmp11__target;
	GDestroyNotify _tmp11__target_destroy_notify;
	GeeFunctionsEqualDataFuncClosure* _tmp12_;
	GeeEqualDataFunc _tmp13_;
	gpointer _tmp13__target;
	GDestroyNotify _tmp13__target_destroy_notify;
	GeeFunctionsEqualDataFuncClosure* _tmp14_;
	GeeHashMapNode** _tmp15_;
	self = (GeeHashMap*) gee_abstract_map_construct (object_type, k_type, (GBoxedCopyFunc) k_dup_func, (GDestroyNotify) k_destroy_func, v_type, (GBoxedCopyFunc) v_dup_func, (GDestroyNotify) v_destroy_func);
	self->priv->k_type = k_type;
	self->priv->k_dup_func = k_dup_func;
	self->priv->k_destroy_func = k_destroy_func;
	self->priv->v_type = v_type;
	self->priv->v_dup_func = v_dup_func;
	self->priv->v_destroy_func = v_destroy_func;
	if (key_hash_func == NULL) {
		gpointer _tmp0_ = NULL;
		GDestroyNotify _tmp1_ = NULL;
		GeeHashDataFunc _tmp2_;
		_tmp2_ = gee_functions_get_hash_func_for (k_type, &_tmp0_, &_tmp1_);
		(key_hash_func_target_destroy_notify == NULL) ? NULL : (key_hash_func_target_destroy_notify (key_hash_func_target), NULL);
		key_hash_func = NULL;
		key_hash_func_target = NULL;
		key_hash_func_target_destroy_notify = NULL;
		key_hash_func = _tmp2_;
		key_hash_func_target = _tmp0_;
		key_hash_func_target_destroy_notify = _tmp1_;
	}
	if (key_equal_func == NULL) {
		gpointer _tmp3_ = NULL;
		GDestroyNotify _tmp4_ = NULL;
		GeeEqualDataFunc _tmp5_;
		_tmp5_ = gee_functions_get_equal_func_for (k_type, &_tmp3_, &_tmp4_);
		(key_equal_func_target_destroy_notify == NULL) ? NULL : (key_equal_func_target_destroy_notify (key_equal_func_target), NULL);
		key_equal_func = NULL;
		key_equal_func_target = NULL;
		key_equal_func_target_destroy_notify = NULL;
		key_equal_func = _tmp5_;
		key_equal_func_target = _tmp3_;
		key_equal_func_target_destroy_notify = _tmp4_;
	}
	if (value_equal_func == NULL) {
		gpointer _tmp6_ = NULL;
		GDestroyNotify _tmp7_ = NULL;
		GeeEqualDataFunc _tmp8_;
		_tmp8_ = gee_functions_get_equal_func_for (v_type, &_tmp6_, &_tmp7_);
		(value_equal_func_target_destroy_notify == NULL) ? NULL : (value_equal_func_target_destroy_notify (value_equal_func_target), NULL);
		value_equal_func = NULL;
		value_equal_func_target = NULL;
		value_equal_func_target_destroy_notify = NULL;
		value_equal_func = _tmp8_;
		value_equal_func_target = _tmp6_;
		value_equal_func_target_destroy_notify = _tmp7_;
	}
	_tmp9_ = key_hash_func;
	_tmp9__target = key_hash_func_target;
	_tmp9__target_destroy_notify = key_hash_func_target_destroy_notify;
	key_hash_func = NULL;
	key_hash_func_target = NULL;
	key_hash_func_target_destroy_notify = NULL;
	_tmp10_ = gee_functions_hash_data_func_closure_new (k_type, (GBoxedCopyFunc) k_dup_func, (GDestroyNotify) k_destroy_func, _tmp9_, _tmp9__target, _tmp9__target_destroy_notify);
	_gee_functions_hash_data_func_closure_unref0 (self->priv->_key_hash_func);
	self->priv->_key_hash_func = _tmp10_;
	_tmp11_ = key_equal_func;
	_tmp11__target = key_equal_func_target;
	_tmp11__target_destroy_notify = key_equal_func_target_destroy_notify;
	key_equal_func = NULL;
	key_equal_func_target = NULL;
	key_equal_func_target_destroy_notify = NULL;
	_tmp12_ = gee_functions_equal_data_func_closure_new (k_type, (GBoxedCopyFunc) k_dup_func, (GDestroyNotify) k_destroy_func, _tmp11_, _tmp11__target, _tmp11__target_destroy_notify);
	_gee_functions_equal_data_func_closure_unref0 (self->priv->_key_equal_func);
	self->priv->_key_equal_func = _tmp12_;
	_tmp13_ = value_equal_func;
	_tmp13__target = value_equal_func_target;
	_tmp13__target_destroy_notify = value_equal_func_target_destroy_notify;
	value_equal_func = NULL;
	value_equal_func_target = NULL;
	value_equal_func_target_destroy_notify = NULL;
	_tmp14_ = gee_functions_equal_data_func_closure_new (v_type, (GBoxedCopyFunc) v_dup_func, (GDestroyNotify) v_destroy_func, _tmp13_, _tmp13__target, _tmp13__target_destroy_notify);
	_gee_functions_equal_data_func_closure_unref0 (self->priv->_value_equal_func);
	self->priv->_value_equal_func = _tmp14_;
	self->priv->_array_size = GEE_HASH_MAP_MIN_SIZE;
	_tmp15_ = g_new0 (GeeHashMapNode*, self->priv->_array_size + 1);
	self->priv->_nodes = (_vala_array_free (self->priv->_nodes, self->priv->_nodes_length1, (GDestroyNotify) gee_hash_map_node_free), NULL);
	self->priv->_nodes = _tmp15_;
	self->priv->_nodes_length1 = self->priv->_array_size;
	self->priv->__nodes_size_ = self->priv->_nodes_length1;
	(key_hash_func_target_destroy_notify == NULL) ? NULL : (key_hash_func_target_destroy_notify (key_hash_func_target), NULL);
	key_hash_func = NULL;
	key_hash_func_target = NULL;
	key_hash_func_target_destroy_notify = NULL;
	(key_equal_func_target_destroy_notify == NULL) ? NULL : (key_equal_func_target_destroy_notify (key_equal_func_target), NULL);
	key_equal_func = NULL;
	key_equal_func_target = NULL;
	key_equal_func_target_destroy_notify = NULL;
	(value_equal_func_target_destroy_notify == NULL) ? NULL : (value_equal_func_target_destroy_notify (value_equal_func_target), NULL);
	value_equal_func = NULL;
	value_equal_func_target = NULL;
	value_equal_func_target_destroy_notify = NULL;
	return self;
}

GeeHashMap*
gee_hash_map_new (GType k_type,
                  GBoxedCopyFunc k_dup_func,
                  GDestroyNotify k_destroy_func,
                  GType v_type,
                  GBoxedCopyFunc v_dup_func,
                  GDestroyNotify v_destroy_func,
                  GeeHashDataFunc key_hash_func,
                  gpointer key_hash_func_target,
                  GDestroyNotify key_hash_func_target_destroy_notify,
                  GeeEqualDataFunc key_equal_func,
                  gpointer key_equal_func_target,
                  GDestroyNotify key_equal_func_target_destroy_notify,
                  GeeEqualDataFunc value_equal_func,
                  gpointer value_equal_func_target,
                  GDestroyNotify value_equal_func_target_destroy_notify)
{
	return gee_hash_map_construct (GEE_TYPE_HASH_MAP, k_type, k_dup_func, k_destroy_func, v_type, v_dup_func, v_destroy_func, key_hash_func, key_hash_func_target, key_hash_func_target_destroy_notify, key_equal_func, key_equal_func_target, key_equal_func_target_destroy_notify, value_equal_func, value_equal_func_target, value_equal_func_target_destroy_notify);
}

static gpointer
_gee_functions_hash_data_func_closure_ref0 (gpointer self)
{
	return self ? gee_functions_hash_data_func_closure_ref (self) : NULL;
}

static gpointer
_gee_functions_equal_data_func_closure_ref0 (gpointer self)
{
	return self ? gee_functions_equal_data_func_closure_ref (self) : NULL;
}

 G_GNUC_INTERNAL GeeHashMap*
gee_hash_map_construct_with_closures (GType object_type,
                                      GType k_type,
                                      GBoxedCopyFunc k_dup_func,
                                      GDestroyNotify k_destroy_func,
                                      GType v_type,
                                      GBoxedCopyFunc v_dup_func,
                                      GDestroyNotify v_destroy_func,
                                      GeeFunctionsHashDataFuncClosure* key_hash_func,
                                      GeeFunctionsEqualDataFuncClosure* key_equal_func,
                                      GeeFunctionsEqualDataFuncClosure* value_equal_func)
{
	GeeHashMap * self = NULL;
	GeeFunctionsHashDataFuncClosure* _tmp0_;
	GeeFunctionsEqualDataFuncClosure* _tmp1_;
	GeeFunctionsEqualDataFuncClosure* _tmp2_;
	GeeHashMapNode** _tmp3_;
	g_return_val_if_fail (key_hash_func != NULL, NULL);
	g_return_val_if_fail (key_equal_func != NULL, NULL);
	g_return_val_if_fail (value_equal_func != NULL, NULL);
	self = (GeeHashMap*) gee_abstract_map_construct (object_type, k_type, (GBoxedCopyFunc) k_dup_func, (GDestroyNotify) k_destroy_func, v_type, (GBoxedCopyFunc) v_dup_func, (GDestroyNotify) v_destroy_func);
	self->priv->k_type = k_type;
	self->priv->k_dup_func = k_dup_func;
	self->priv->k_destroy_func = k_destroy_func;
	self->priv->v_type = v_type;
	self->priv->v_dup_func = v_dup_func;
	self->priv->v_destroy_func = v_destroy_func;
	_tmp0_ = _gee_functions_hash_data_func_closure_ref0 (key_hash_func);
	_gee_functions_hash_data_func_closure_unref0 (self->priv->_key_hash_func);
	self->priv->_key_hash_func = _tmp0_;
	_tmp1_ = _gee_functions_equal_data_func_closure_ref0 (key_equal_func);
	_gee_functions_equal_data_func_closure_unref0 (self->priv->_key_equal_func);
	self->priv->_key_equal_func = _tmp1_;
	_tmp2_ = _gee_functions_equal_data_func_closure_ref0 (value_equal_func);
	_gee_functions_equal_data_func_closure_unref0 (self->priv->_value_equal_func);
	self->priv->_value_equal_func = _tmp2_;
	self->priv->_array_size = GEE_HASH_MAP_MIN_SIZE;
	_tmp3_ = g_new0 (GeeHashMapNode*, self->priv->_array_size + 1);
	self->priv->_nodes = (_vala_array_free (self->priv->_nodes, self->priv->_nodes_length1, (GDestroyNotify) gee_hash_map_node_free), NULL);
	self->priv->_nodes = _tmp3_;
	self->priv->_nodes_length1 = self->priv->_array_size;
	self->priv->__nodes_size_ = self->priv->_nodes_length1;
	_gee_functions_hash_data_func_closure_unref0 (key_hash_func);
	_gee_functions_equal_data_func_closure_unref0 (key_equal_func);
	_gee_functions_equal_data_func_closure_unref0 (value_equal_func);
	return self;
}

 G_GNUC_INTERNAL GeeHashMap*
gee_hash_map_new_with_closures (GType k_type,
                                GBoxedCopyFunc k_dup_func,
                                GDestroyNotify k_destroy_func,
                                GType v_type,
                                GBoxedCopyFunc v_dup_func,
                                GDestroyNotify v_destroy_func,
                                GeeFunctionsHashDataFuncClosure* key_hash_func,
                                GeeFunctionsEqualDataFuncClosure* key_equal_func,
                                GeeFunctionsEqualDataFuncClosure* value_equal_func)
{
	return gee_hash_map_construct_with_closures (GEE_TYPE_HASH_MAP, k_type, k_dup_func, k_destroy_func, v_type, v_dup_func, v_destroy_func, key_hash_func, key_equal_func, value_equal_func);
}

static GeeHashMapNode**
gee_hash_map_lookup_node (GeeHashMap* self,
                          gconstpointer key)
{
	guint hash_value = 0U;
	GeeHashDataFunc _tmp0_ = NULL;
	gpointer _tmp0__target = NULL;
	GeeHashDataFunc _tmp1_;
	gpointer _tmp1__target;
	GeeHashMapNode** node = NULL;
	GeeHashMapNode** _tmp2_;
	gint _tmp2__length1;
	GeeHashMapNode** _tmp12_;
	GeeHashMapNode** result;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = gee_hash_map_get_key_hash_func (self, &_tmp0__target);
	_tmp1_ = _tmp0_;
	_tmp1__target = _tmp0__target;
	hash_value = _tmp1_ (key, _tmp1__target);
	_tmp2_ = self->priv->_nodes;
	_tmp2__length1 = self->priv->_nodes_length1;
	node = &_tmp2_[hash_value % self->priv->_array_size];
	while (TRUE) {
		gboolean _tmp3_ = FALSE;
		GeeHashMapNode** _tmp4_;
		GeeHashMapNode** _tmp11_;
		_tmp4_ = node;
		if ((*_tmp4_) != NULL) {
			gboolean _tmp5_ = FALSE;
			GeeHashMapNode** _tmp6_;
			_tmp6_ = node;
			if (hash_value != (*_tmp6_)->key_hash) {
				_tmp5_ = TRUE;
			} else {
				GeeEqualDataFunc _tmp7_ = NULL;
				gpointer _tmp7__target = NULL;
				GeeEqualDataFunc _tmp8_;
				gpointer _tmp8__target;
				GeeHashMapNode** _tmp9_;
				gconstpointer _tmp10_;
				_tmp7_ = gee_hash_map_get_key_equal_func (self, &_tmp7__target);
				_tmp8_ = _tmp7_;
				_tmp8__target = _tmp7__target;
				_tmp9_ = node;
				_tmp10_ = (*_tmp9_)->key;
				_tmp5_ = !_tmp8_ (_tmp10_, key, _tmp8__target);
			}
			_tmp3_ = _tmp5_;
		} else {
			_tmp3_ = FALSE;
		}
		if (!_tmp3_) {
			break;
		}
		_tmp11_ = node;
		node = &(*_tmp11_)->next;
	}
	_tmp12_ = node;
	result = _tmp12_;
	return result;
}

/**
 * {@inheritDoc}
 */
static gboolean
gee_hash_map_real_has_key (GeeAbstractMap* base,
                           gconstpointer key)
{
	GeeHashMap * self;
	GeeHashMapNode** node = NULL;
	GeeHashMapNode** _tmp0_;
	gboolean result;
	self = (GeeHashMap*) base;
	_tmp0_ = gee_hash_map_lookup_node (self, key);
	node = _tmp0_;
	result = (*node) != NULL;
	return result;
}

/**
 * {@inheritDoc}
 */
static gboolean
gee_hash_map_real_has (GeeAbstractMap* base,
                       gconstpointer key,
                       gconstpointer value)
{
	GeeHashMap * self;
	GeeHashMapNode** node = NULL;
	GeeHashMapNode** _tmp0_;
	gboolean _tmp1_ = FALSE;
	GeeHashMapNode** _tmp2_;
	gboolean result;
	self = (GeeHashMap*) base;
	_tmp0_ = gee_hash_map_lookup_node (self, key);
	node = _tmp0_;
	_tmp2_ = node;
	if ((*_tmp2_) != NULL) {
		GeeEqualDataFunc _tmp3_ = NULL;
		gpointer _tmp3__target = NULL;
		GeeEqualDataFunc _tmp4_;
		gpointer _tmp4__target;
		GeeHashMapNode** _tmp5_;
		gconstpointer _tmp6_;
		_tmp3_ = gee_hash_map_get_value_equal_func (self, &_tmp3__target);
		_tmp4_ = _tmp3_;
		_tmp4__target = _tmp3__target;
		_tmp5_ = node;
		_tmp6_ = (*_tmp5_)->value;
		_tmp1_ = _tmp4_ (_tmp6_, value, _tmp4__target);
	} else {
		_tmp1_ = FALSE;
	}
	result = _tmp1_;
	return result;
}

/**
 * {@inheritDoc}
 */
static gpointer
gee_hash_map_real_get (GeeAbstractMap* base,
                       gconstpointer key)
{
	GeeHashMap * self;
	GeeHashMapNode* node = NULL;
	GeeHashMapNode** _tmp0_;
	GeeHashMapNode* _tmp1_;
	gpointer result;
	self = (GeeHashMap*) base;
	_tmp0_ = gee_hash_map_lookup_node (self, key);
	node = *_tmp0_;
	_tmp1_ = node;
	if (_tmp1_ != NULL) {
		GeeHashMapNode* _tmp2_;
		gconstpointer _tmp3_;
		gpointer _tmp4_;
		_tmp2_ = node;
		_tmp3_ = _tmp2_->value;
		_tmp4_ = ((_tmp3_ != NULL) && (self->priv->v_dup_func != NULL)) ? self->priv->v_dup_func ((gpointer) _tmp3_) : ((gpointer) _tmp3_);
		result = _tmp4_;
		return result;
	} else {
		result = NULL;
		return result;
	}
}

/**
 * {@inheritDoc}
 */
static void
gee_hash_map_real_set (GeeAbstractMap* base,
                       gconstpointer key,
                       gconstpointer value)
{
	GeeHashMap * self;
	GeeHashMapNode** node = NULL;
	GeeHashMapNode** _tmp0_;
	GeeHashMapNode** _tmp1_;
	gint _tmp11_;
	self = (GeeHashMap*) base;
	_tmp0_ = gee_hash_map_lookup_node (self, key);
	node = _tmp0_;
	_tmp1_ = node;
	if ((*_tmp1_) != NULL) {
		GeeHashMapNode** _tmp2_;
		gpointer _tmp3_;
		_tmp2_ = node;
		_tmp3_ = ((value != NULL) && (self->priv->v_dup_func != NULL)) ? self->priv->v_dup_func ((gpointer) value) : ((gpointer) value);
		(((*_tmp2_)->value == NULL) || (self->priv->v_destroy_func == NULL)) ? NULL : ((*_tmp2_)->value = (self->priv->v_destroy_func ((*_tmp2_)->value), NULL));
		(*_tmp2_)->value = _tmp3_;
	} else {
		guint hash_value = 0U;
		GeeHashDataFunc _tmp4_ = NULL;
		gpointer _tmp4__target = NULL;
		GeeHashDataFunc _tmp5_;
		gpointer _tmp5__target;
		GeeHashMapNode** _tmp6_;
		gpointer _tmp7_;
		gpointer _tmp8_;
		GeeHashMapNode* _tmp9_;
		gint _tmp10_;
		_tmp4_ = gee_hash_map_get_key_hash_func (self, &_tmp4__target);
		_tmp5_ = _tmp4_;
		_tmp5__target = _tmp4__target;
		hash_value = _tmp5_ (key, _tmp5__target);
		_tmp6_ = node;
		_tmp7_ = ((key != NULL) && (self->priv->k_dup_func != NULL)) ? self->priv->k_dup_func ((gpointer) key) : ((gpointer) key);
		_tmp8_ = ((value != NULL) && (self->priv->v_dup_func != NULL)) ? self->priv->v_dup_func ((gpointer) value) : ((gpointer) value);
		_tmp9_ = gee_hash_map_node_new (_tmp7_, _tmp8_, hash_value);
		*_tmp6_ = _tmp9_;
		_tmp10_ = self->priv->_nnodes;
		self->priv->_nnodes = _tmp10_ + 1;
		gee_hash_map_resize (self);
	}
	_tmp11_ = self->priv->_stamp;
	self->priv->_stamp = _tmp11_ + 1;
}

/**
 * {@inheritDoc}
 */
static gboolean
gee_hash_map_real_unset (GeeAbstractMap* base,
                         gconstpointer key,
                         gpointer* value)
{
	GeeHashMap * self;
	gpointer _vala_value = NULL;
	gboolean b = FALSE;
	gpointer _tmp0_ = NULL;
	gboolean _tmp1_;
	gboolean result;
	self = (GeeHashMap*) base;
	_tmp1_ = gee_hash_map_unset_helper (self, key, &_tmp0_);
	((_vala_value == NULL) || (self->priv->v_destroy_func == NULL)) ? NULL : (_vala_value = (self->priv->v_destroy_func (_vala_value), NULL));
	_vala_value = _tmp0_;
	b = _tmp1_;
	if (b) {
		gee_hash_map_resize (self);
	}
	result = b;
	if (value) {
		*value = _vala_value;
	} else {
		((_vala_value == NULL) || (self->priv->v_destroy_func == NULL)) ? NULL : (_vala_value = (self->priv->v_destroy_func (_vala_value), NULL));
	}
	return result;
}

/**
 * {@inheritDoc}
 */
static void
gee_hash_map_real_clear (GeeAbstractMap* base)
{
	GeeHashMap * self;
	self = (GeeHashMap*) base;
	{
		gint i = 0;
		i = 0;
		{
			gboolean _tmp0_ = FALSE;
			_tmp0_ = TRUE;
			while (TRUE) {
				GeeHashMapNode* node = NULL;
				GeeHashMapNode** _tmp2_;
				gint _tmp2__length1;
				GeeHashMapNode* _tmp3_;
				if (!_tmp0_) {
					gint _tmp1_;
					_tmp1_ = i;
					i = _tmp1_ + 1;
				}
				_tmp0_ = FALSE;
				if (!(i < self->priv->_array_size)) {
					break;
				}
				_tmp2_ = self->priv->_nodes;
				_tmp2__length1 = self->priv->_nodes_length1;
				_tmp3_ = _tmp2_[i];
				_tmp2_[i] = NULL;
				node = _tmp3_;
				while (TRUE) {
					GeeHashMapNode* _tmp4_;
					GeeHashMapNode* next = NULL;
					GeeHashMapNode* _tmp5_;
					GeeHashMapNode* _tmp6_;
					GeeHashMapNode* _tmp7_;
					GeeHashMapNode* _tmp8_;
					GeeHashMapNode* _tmp9_;
					_tmp4_ = node;
					if (!(_tmp4_ != NULL)) {
						break;
					}
					_tmp5_ = node;
					_tmp6_ = _tmp5_->next;
					_tmp5_->next = NULL;
					next = _tmp6_;
					_tmp7_ = node;
					((_tmp7_->key == NULL) || (self->priv->k_destroy_func == NULL)) ? NULL : (_tmp7_->key = (self->priv->k_destroy_func (_tmp7_->key), NULL));
					_tmp7_->key = NULL;
					_tmp8_ = node;
					((_tmp8_->value == NULL) || (self->priv->v_destroy_func == NULL)) ? NULL : (_tmp8_->value = (self->priv->v_destroy_func (_tmp8_->value), NULL));
					_tmp8_->value = NULL;
					_tmp9_ = next;
					next = NULL;
					_gee_hash_map_node_free0 (node);
					node = _tmp9_;
					_gee_hash_map_node_free0 (next);
				}
				_gee_hash_map_node_free0 (node);
			}
		}
	}
	self->priv->_nnodes = 0;
	gee_hash_map_resize (self);
}

/**
 * {@inheritDoc}
 */
static GeeMapIterator*
gee_hash_map_real_map_iterator (GeeAbstractMap* base)
{
	GeeHashMap * self;
	GeeHashMapMapIterator* _tmp0_;
	GeeMapIterator* result;
	self = (GeeHashMap*) base;
	_tmp0_ = gee_hash_map_map_iterator_new (self->priv->k_type, (GBoxedCopyFunc) self->priv->k_dup_func, (GDestroyNotify) self->priv->k_destroy_func, self->priv->v_type, (GBoxedCopyFunc) self->priv->v_dup_func, (GDestroyNotify) self->priv->v_destroy_func, self);
	result = (GeeMapIterator*) _tmp0_;
	return result;
}

 G_GNUC_INTERNAL GeeFunctionsHashDataFuncClosure*
gee_hash_map_get_key_hash_func_closure (GeeHashMap* self)
{
	GeeFunctionsHashDataFuncClosure* _tmp0_;
	GeeFunctionsHashDataFuncClosure* _tmp1_;
	GeeFunctionsHashDataFuncClosure* result;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_key_hash_func;
	_tmp1_ = _gee_functions_hash_data_func_closure_ref0 (_tmp0_);
	result = _tmp1_;
	return result;
}

 G_GNUC_INTERNAL GeeFunctionsEqualDataFuncClosure*
gee_hash_map_get_key_equal_func_closure (GeeHashMap* self)
{
	GeeFunctionsEqualDataFuncClosure* _tmp0_;
	GeeFunctionsEqualDataFuncClosure* _tmp1_;
	GeeFunctionsEqualDataFuncClosure* result;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_key_equal_func;
	_tmp1_ = _gee_functions_equal_data_func_closure_ref0 (_tmp0_);
	result = _tmp1_;
	return result;
}

static inline gboolean
gee_hash_map_unset_helper (GeeHashMap* self,
                           gconstpointer key,
                           gpointer* value)
{
	gpointer _vala_value = NULL;
	GeeHashMapNode** node = NULL;
	GeeHashMapNode** _tmp0_;
	GeeHashMapNode** _tmp1_;
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	_tmp0_ = gee_hash_map_lookup_node (self, key);
	node = _tmp0_;
	_tmp1_ = node;
	if ((*_tmp1_) != NULL) {
		GeeHashMapNode* next = NULL;
		GeeHashMapNode** _tmp2_;
		GeeHashMapNode* _tmp3_;
		GeeHashMapNode** _tmp4_;
		gpointer _tmp5_;
		GeeHashMapNode** _tmp6_;
		GeeHashMapNode** _tmp7_;
		GeeHashMapNode** _tmp8_;
		GeeHashMapNode** _tmp9_;
		GeeHashMapNode* _tmp10_;
		gint _tmp11_;
		gint _tmp12_;
		_tmp2_ = node;
		_tmp3_ = (*_tmp2_)->next;
		(*_tmp2_)->next = NULL;
		next = _tmp3_;
		_tmp4_ = node;
		_tmp5_ = (*_tmp4_)->value;
		(*_tmp4_)->value = NULL;
		((_vala_value == NULL) || (self->priv->v_destroy_func == NULL)) ? NULL : (_vala_value = (self->priv->v_destroy_func (_vala_value), NULL));
		_vala_value = _tmp5_;
		_tmp6_ = node;
		(((*_tmp6_)->key == NULL) || (self->priv->k_destroy_func == NULL)) ? NULL : ((*_tmp6_)->key = (self->priv->k_destroy_func ((*_tmp6_)->key), NULL));
		(*_tmp6_)->key = NULL;
		_tmp7_ = node;
		(((*_tmp7_)->value == NULL) || (self->priv->v_destroy_func == NULL)) ? NULL : ((*_tmp7_)->value = (self->priv->v_destroy_func ((*_tmp7_)->value), NULL));
		(*_tmp7_)->value = NULL;
		_tmp8_ = node;
		_gee_hash_map_node_free0 (*_tmp8_);
		_tmp9_ = node;
		_tmp10_ = next;
		next = NULL;
		*_tmp9_ = _tmp10_;
		_tmp11_ = self->priv->_nnodes;
		self->priv->_nnodes = _tmp11_ - 1;
		_tmp12_ = self->priv->_stamp;
		self->priv->_stamp = _tmp12_ + 1;
		result = TRUE;
		_gee_hash_map_node_free0 (next);
		if (value) {
			*value = _vala_value;
		} else {
			((_vala_value == NULL) || (self->priv->v_destroy_func == NULL)) ? NULL : (_vala_value = (self->priv->v_destroy_func (_vala_value), NULL));
		}
		return result;
	} else {
		((_vala_value == NULL) || (self->priv->v_destroy_func == NULL)) ? NULL : (_vala_value = (self->priv->v_destroy_func (_vala_value), NULL));
		_vala_value = NULL;
	}
	result = FALSE;
	if (value) {
		*value = _vala_value;
	} else {
		((_vala_value == NULL) || (self->priv->v_destroy_func == NULL)) ? NULL : (_vala_value = (self->priv->v_destroy_func (_vala_value), NULL));
	}
	return result;
}

static inline void
gee_hash_map_resize (GeeHashMap* self)
{
	gboolean _tmp0_ = FALSE;
	gboolean _tmp1_ = FALSE;
	g_return_if_fail (self != NULL);
	if (self->priv->_array_size >= (3 * self->priv->_nnodes)) {
		_tmp1_ = self->priv->_array_size >= GEE_HASH_MAP_MIN_SIZE;
	} else {
		_tmp1_ = FALSE;
	}
	if (_tmp1_) {
		_tmp0_ = TRUE;
	} else {
		gboolean _tmp2_ = FALSE;
		if ((3 * self->priv->_array_size) <= self->priv->_nnodes) {
			_tmp2_ = self->priv->_array_size < GEE_HASH_MAP_MAX_SIZE;
		} else {
			_tmp2_ = FALSE;
		}
		_tmp0_ = _tmp2_;
	}
	if (_tmp0_) {
		gint new_array_size = 0;
		GeeHashMapNode** new_nodes = NULL;
		GeeHashMapNode** _tmp3_;
		gint new_nodes_length1;
		gint _new_nodes_size_;
		GeeHashMapNode** _tmp19_;
		gint _tmp19__length1;
		new_array_size = (gint) g_spaced_primes_closest ((guint) self->priv->_nnodes);
		new_array_size = CLAMP (new_array_size, GEE_HASH_MAP_MIN_SIZE, GEE_HASH_MAP_MAX_SIZE);
		_tmp3_ = g_new0 (GeeHashMapNode*, new_array_size + 1);
		new_nodes = _tmp3_;
		new_nodes_length1 = new_array_size;
		_new_nodes_size_ = new_nodes_length1;
		{
			gint i = 0;
			i = 0;
			{
				gboolean _tmp4_ = FALSE;
				_tmp4_ = TRUE;
				while (TRUE) {
					GeeHashMapNode* node = NULL;
					GeeHashMapNode* next = NULL;
					if (!_tmp4_) {
						gint _tmp5_;
						_tmp5_ = i;
						i = _tmp5_ + 1;
					}
					_tmp4_ = FALSE;
					if (!(i < self->priv->_array_size)) {
						break;
					}
					next = NULL;
					{
						GeeHashMapNode** _tmp6_;
						gint _tmp6__length1;
						GeeHashMapNode* _tmp7_;
						gboolean _tmp8_ = FALSE;
						_tmp6_ = self->priv->_nodes;
						_tmp6__length1 = self->priv->_nodes_length1;
						_tmp7_ = _tmp6_[i];
						_tmp6_[i] = NULL;
						_gee_hash_map_node_free0 (node);
						node = _tmp7_;
						_tmp8_ = TRUE;
						while (TRUE) {
							GeeHashMapNode* _tmp10_;
							GeeHashMapNode* _tmp11_;
							GeeHashMapNode* _tmp12_;
							guint hash_val = 0U;
							GeeHashMapNode* _tmp13_;
							GeeHashMapNode* _tmp14_;
							GeeHashMapNode** _tmp15_;
							gint _tmp15__length1;
							GeeHashMapNode* _tmp16_;
							GeeHashMapNode** _tmp17_;
							gint _tmp17__length1;
							GeeHashMapNode* _tmp18_;
							if (!_tmp8_) {
								GeeHashMapNode* _tmp9_;
								_tmp9_ = next;
								next = NULL;
								_gee_hash_map_node_free0 (node);
								node = _tmp9_;
							}
							_tmp8_ = FALSE;
							_tmp10_ = node;
							if (!(_tmp10_ != NULL)) {
								break;
							}
							_tmp11_ = node;
							_tmp12_ = _tmp11_->next;
							_tmp11_->next = NULL;
							_gee_hash_map_node_free0 (next);
							next = _tmp12_;
							_tmp13_ = node;
							hash_val = _tmp13_->key_hash % new_array_size;
							_tmp14_ = node;
							_tmp15_ = new_nodes;
							_tmp15__length1 = new_nodes_length1;
							_tmp16_ = _tmp15_[hash_val];
							_tmp15_[hash_val] = NULL;
							_gee_hash_map_node_free0 (_tmp14_->next);
							_tmp14_->next = _tmp16_;
							_tmp17_ = new_nodes;
							_tmp17__length1 = new_nodes_length1;
							_tmp18_ = node;
							node = NULL;
							_gee_hash_map_node_free0 (_tmp17_[hash_val]);
							_tmp17_[hash_val] = _tmp18_;
						}
					}
					_gee_hash_map_node_free0 (next);
					_gee_hash_map_node_free0 (node);
				}
			}
		}
		_tmp19_ = new_nodes;
		_tmp19__length1 = new_nodes_length1;
		new_nodes = NULL;
		new_nodes_length1 = 0;
		self->priv->_nodes = (_vala_array_free (self->priv->_nodes, self->priv->_nodes_length1, (GDestroyNotify) gee_hash_map_node_free), NULL);
		self->priv->_nodes = _tmp19_;
		self->priv->_nodes_length1 = _tmp19__length1;
		self->priv->__nodes_size_ = self->priv->_nodes_length1;
		self->priv->_array_size = new_array_size;
		new_nodes = (_vala_array_free (new_nodes, new_nodes_length1, (GDestroyNotify) gee_hash_map_node_free), NULL);
	}
}

static gint
gee_hash_map_real_get_size (GeeAbstractMap* base)
{
	gint result;
	GeeHashMap* self;
	self = (GeeHashMap*) base;
	result = self->priv->_nnodes;
	return result;
}

static gboolean
gee_hash_map_real_get_read_only (GeeAbstractMap* base)
{
	gboolean result;
	GeeHashMap* self;
	self = (GeeHashMap*) base;
	result = FALSE;
	return result;
}

static gpointer
_g_object_ref0 (gpointer self)
{
	return self ? g_object_ref (self) : NULL;
}

static GeeSet*
gee_hash_map_real_get_keys (GeeAbstractMap* base)
{
	GeeSet* result;
	GeeHashMap* self;
	GeeSet* keys = NULL;
	GeeSet* _tmp0_;
	GeeSet* _tmp1_;
	GeeSet* _tmp2_;
	self = (GeeHashMap*) base;
	_tmp0_ = self->priv->_keys;
	_tmp1_ = _g_object_ref0 (_tmp0_);
	keys = _tmp1_;
	_tmp2_ = self->priv->_keys;
	if (_tmp2_ == NULL) {
		GeeHashMapKeySet* _tmp3_;
		GeeSet* _tmp4_;
		GeeSet* _tmp5_;
		_tmp3_ = gee_hash_map_key_set_new (self->priv->k_type, (GBoxedCopyFunc) self->priv->k_dup_func, (GDestroyNotify) self->priv->k_destroy_func, self->priv->v_type, (GBoxedCopyFunc) self->priv->v_dup_func, (GDestroyNotify) self->priv->v_destroy_func, self);
		_g_object_unref0 (keys);
		keys = (GeeSet*) _tmp3_;
		_tmp4_ = keys;
		self->priv->_keys = _tmp4_;
		_tmp5_ = keys;
		g_object_add_weak_pointer ((GObject*) _tmp5_, (void**) (&self->priv->_keys));
	}
	result = keys;
	return result;
}

static GeeCollection*
gee_hash_map_real_get_values (GeeAbstractMap* base)
{
	GeeCollection* result;
	GeeHashMap* self;
	GeeCollection* values = NULL;
	GeeCollection* _tmp0_;
	GeeCollection* _tmp1_;
	GeeCollection* _tmp2_;
	self = (GeeHashMap*) base;
	_tmp0_ = self->priv->_values;
	_tmp1_ = _g_object_ref0 (_tmp0_);
	values = _tmp1_;
	_tmp2_ = self->priv->_values;
	if (_tmp2_ == NULL) {
		GeeHashMapValueCollection* _tmp3_;
		GeeCollection* _tmp4_;
		GeeCollection* _tmp5_;
		_tmp3_ = gee_hash_map_value_collection_new (self->priv->k_type, (GBoxedCopyFunc) self->priv->k_dup_func, (GDestroyNotify) self->priv->k_destroy_func, self->priv->v_type, (GBoxedCopyFunc) self->priv->v_dup_func, (GDestroyNotify) self->priv->v_destroy_func, self);
		_g_object_unref0 (values);
		values = (GeeCollection*) _tmp3_;
		_tmp4_ = values;
		self->priv->_values = _tmp4_;
		_tmp5_ = values;
		g_object_add_weak_pointer ((GObject*) _tmp5_, (void**) (&self->priv->_values));
	}
	result = values;
	return result;
}

static GeeSet*
gee_hash_map_real_get_entries (GeeAbstractMap* base)
{
	GeeSet* result;
	GeeHashMap* self;
	GeeSet* entries = NULL;
	GeeSet* _tmp0_;
	GeeSet* _tmp1_;
	GeeSet* _tmp2_;
	self = (GeeHashMap*) base;
	_tmp0_ = self->priv->_entries;
	_tmp1_ = _g_object_ref0 (_tmp0_);
	entries = _tmp1_;
	_tmp2_ = self->priv->_entries;
	if (_tmp2_ == NULL) {
		GeeHashMapEntrySet* _tmp3_;
		GeeSet* _tmp4_;
		GeeSet* _tmp5_;
		_tmp3_ = gee_hash_map_entry_set_new (self->priv->k_type, (GBoxedCopyFunc) self->priv->k_dup_func, (GDestroyNotify) self->priv->k_destroy_func, self->priv->v_type, (GBoxedCopyFunc) self->priv->v_dup_func, (GDestroyNotify) self->priv->v_destroy_func, self);
		_g_object_unref0 (entries);
		entries = (GeeSet*) _tmp3_;
		_tmp4_ = entries;
		self->priv->_entries = _tmp4_;
		_tmp5_ = entries;
		g_object_add_weak_pointer ((GObject*) _tmp5_, (void**) (&self->priv->_entries));
	}
	result = entries;
	return result;
}

GeeHashDataFunc
gee_hash_map_get_key_hash_func (GeeHashMap* self,
                                gpointer* result_target)
{
	GeeHashDataFunc result;
	GeeFunctionsHashDataFuncClosure* _tmp0_;
	GeeHashDataFunc _tmp1_;
	gpointer _tmp1__target;
	GeeHashDataFunc _tmp2_;
	gpointer _tmp2__target;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_key_hash_func;
	_tmp1_ = _tmp0_->func;
	_tmp1__target = _tmp0_->func_target;
	_tmp2_ = _tmp1_;
	_tmp2__target = _tmp1__target;
	*result_target = _tmp2__target;
	result = _tmp2_;
	return result;
}

static void
gee_hash_map_set_key_hash_func (GeeHashMap* self,
                                GeeHashDataFunc value,
                                gpointer value_target)
{
	g_return_if_fail (self != NULL);
}

GeeEqualDataFunc
gee_hash_map_get_key_equal_func (GeeHashMap* self,
                                 gpointer* result_target)
{
	GeeEqualDataFunc result;
	GeeFunctionsEqualDataFuncClosure* _tmp0_;
	GeeEqualDataFunc _tmp1_;
	gpointer _tmp1__target;
	GeeEqualDataFunc _tmp2_;
	gpointer _tmp2__target;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_key_equal_func;
	_tmp1_ = _tmp0_->func;
	_tmp1__target = _tmp0_->func_target;
	_tmp2_ = _tmp1_;
	_tmp2__target = _tmp1__target;
	*result_target = _tmp2__target;
	result = _tmp2_;
	return result;
}

static void
gee_hash_map_set_key_equal_func (GeeHashMap* self,
                                 GeeEqualDataFunc value,
                                 gpointer value_target)
{
	g_return_if_fail (self != NULL);
}

GeeEqualDataFunc
gee_hash_map_get_value_equal_func (GeeHashMap* self,
                                   gpointer* result_target)
{
	GeeEqualDataFunc result;
	GeeFunctionsEqualDataFuncClosure* _tmp0_;
	GeeEqualDataFunc _tmp1_;
	gpointer _tmp1__target;
	GeeEqualDataFunc _tmp2_;
	gpointer _tmp2__target;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_value_equal_func;
	_tmp1_ = _tmp0_->func;
	_tmp1__target = _tmp0_->func_target;
	_tmp2_ = _tmp1_;
	_tmp2__target = _tmp1__target;
	*result_target = _tmp2__target;
	result = _tmp2_;
	return result;
}

static void
gee_hash_map_set_value_equal_func (GeeHashMap* self,
                                   GeeEqualDataFunc value,
                                   gpointer value_target)
{
	g_return_if_fail (self != NULL);
}

static GeeHashMapNode*
gee_hash_map_node_new (gpointer k,
                       gpointer v,
                       guint hash)
{
	GeeHashMapNode* self;
	gpointer _tmp0_;
	gpointer _tmp1_;
	self = g_slice_new0 (GeeHashMapNode);
	gee_hash_map_node_instance_init (self);
	_tmp0_ = k;
	k = NULL;
	self->key = _tmp0_;
	_tmp1_ = v;
	v = NULL;
	self->value = _tmp1_;
	self->key_hash = hash;
	self->entry = NULL;
	return self;
}

static void
gee_hash_map_node_instance_init (GeeHashMapNode * self)
{
}

static void
gee_hash_map_node_free (GeeHashMapNode * self)
{
	GeeMapEntry* _tmp0_;
	_tmp0_ = self->entry;
	if (_tmp0_ != NULL) {
		GeeMapEntry* _tmp1_;
		_tmp1_ = self->entry;
		g_object_remove_weak_pointer ((GObject*) _tmp1_, (void**) (&self->entry));
	}
	_gee_hash_map_node_free0 (self->next);
	g_slice_free (GeeHashMapNode, self);
}

static inline gpointer
gee_hash_map_entry_get_instance_private (GeeHashMapEntry* self)
{
	return G_STRUCT_MEMBER_P (self, GeeHashMapEntry_private_offset);
}

static GeeMapEntry*
gee_hash_map_entry_entry_for (GType k_type,
                              GBoxedCopyFunc k_dup_func,
                              GDestroyNotify k_destroy_func,
                              GType v_type,
                              GBoxedCopyFunc v_dup_func,
                              GDestroyNotify v_destroy_func,
                              GeeHashMapNode* node)
{
	GeeMapEntry* _result_ = NULL;
	GeeMapEntry* _tmp0_;
	GeeMapEntry* _tmp1_;
	GeeMapEntry* _tmp2_;
	GeeMapEntry* result;
	g_return_val_if_fail (node != NULL, NULL);
	_tmp0_ = node->entry;
	_tmp1_ = _g_object_ref0 (_tmp0_);
	_result_ = _tmp1_;
	_tmp2_ = node->entry;
	if (_tmp2_ == NULL) {
		GeeHashMapEntry* _tmp3_;
		GeeMapEntry* _tmp4_;
		GeeMapEntry* _tmp5_;
		_tmp3_ = gee_hash_map_entry_new (k_type, (GBoxedCopyFunc) k_dup_func, (GDestroyNotify) k_destroy_func, v_type, (GBoxedCopyFunc) v_dup_func, (GDestroyNotify) v_destroy_func, node);
		_g_object_unref0 (_result_);
		_result_ = (GeeMapEntry*) _tmp3_;
		_tmp4_ = _result_;
		node->entry = _tmp4_;
		_tmp5_ = _result_;
		g_object_add_weak_pointer ((GObject*) _tmp5_, (void**) (&node->entry));
	}
	result = _result_;
	return result;
}

static GeeHashMapEntry*
gee_hash_map_entry_construct (GType object_type,
                              GType k_type,
                              GBoxedCopyFunc k_dup_func,
                              GDestroyNotify k_destroy_func,
                              GType v_type,
                              GBoxedCopyFunc v_dup_func,
                              GDestroyNotify v_destroy_func,
                              GeeHashMapNode* node)
{
	GeeHashMapEntry * self = NULL;
	g_return_val_if_fail (node != NULL, NULL);
	self = (GeeHashMapEntry*) gee_map_entry_construct (object_type, k_type, (GBoxedCopyFunc) k_dup_func, (GDestroyNotify) k_destroy_func, v_type, (GBoxedCopyFunc) v_dup_func, (GDestroyNotify) v_destroy_func);
	self->priv->k_type = k_type;
	self->priv->k_dup_func = k_dup_func;
	self->priv->k_destroy_func = k_destroy_func;
	self->priv->v_type = v_type;
	self->priv->v_dup_func = v_dup_func;
	self->priv->v_destroy_func = v_destroy_func;
	self->priv->_node = node;
	return self;
}

static GeeHashMapEntry*
gee_hash_map_entry_new (GType k_type,
                        GBoxedCopyFunc k_dup_func,
                        GDestroyNotify k_destroy_func,
                        GType v_type,
                        GBoxedCopyFunc v_dup_func,
                        GDestroyNotify v_destroy_func,
                        GeeHashMapNode* node)
{
	return gee_hash_map_entry_construct (GEE_HASH_MAP_TYPE_ENTRY, k_type, k_dup_func, k_destroy_func, v_type, v_dup_func, v_destroy_func, node);
}

static gconstpointer
gee_hash_map_entry_real_get_key (GeeMapEntry* base)
{
	gconstpointer result;
	GeeHashMapEntry* self;
	GeeHashMapNode* _tmp0_;
	gconstpointer _tmp1_;
	self = (GeeHashMapEntry*) base;
	_tmp0_ = self->priv->_node;
	_tmp1_ = _tmp0_->key;
	result = _tmp1_;
	return result;
}

static gconstpointer
gee_hash_map_entry_real_get_value (GeeMapEntry* base)
{
	gconstpointer result;
	GeeHashMapEntry* self;
	GeeHashMapNode* _tmp0_;
	gconstpointer _tmp1_;
	self = (GeeHashMapEntry*) base;
	_tmp0_ = self->priv->_node;
	_tmp1_ = _tmp0_->value;
	result = _tmp1_;
	return result;
}

static void
gee_hash_map_entry_real_set_value (GeeMapEntry* base,
                                   gconstpointer value)
{
	GeeHashMapEntry* self;
	GeeHashMapNode* _tmp0_;
	gpointer _tmp1_;
	self = (GeeHashMapEntry*) base;
	_tmp0_ = self->priv->_node;
	_tmp1_ = ((value != NULL) && (self->priv->v_dup_func != NULL)) ? self->priv->v_dup_func ((gpointer) value) : ((gpointer) value);
	((_tmp0_->value == NULL) || (self->priv->v_destroy_func == NULL)) ? NULL : (_tmp0_->value = (self->priv->v_destroy_func (_tmp0_->value), NULL));
	_tmp0_->value = _tmp1_;
	g_object_notify_by_pspec ((GObject *) self, gee_hash_map_entry_properties[GEE_HASH_MAP_ENTRY_VALUE_PROPERTY]);
}

static gboolean
gee_hash_map_entry_real_get_read_only (GeeMapEntry* base)
{
	gboolean result;
	GeeHashMapEntry* self;
	self = (GeeHashMapEntry*) base;
	result = FALSE;
	return result;
}

static void
gee_hash_map_entry_class_init (GeeHashMapEntryClass * klass,
                               gpointer klass_data)
{
	gee_hash_map_entry_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &GeeHashMapEntry_private_offset);
	GEE_MAP_ENTRY_CLASS (klass)->get_key = gee_hash_map_entry_real_get_key;
	GEE_MAP_ENTRY_CLASS (klass)->get_value = gee_hash_map_entry_real_get_value;
	GEE_MAP_ENTRY_CLASS (klass)->set_value = gee_hash_map_entry_real_set_value;
	GEE_MAP_ENTRY_CLASS (klass)->get_read_only = gee_hash_map_entry_real_get_read_only;
	G_OBJECT_CLASS (klass)->get_property = _vala_gee_hash_map_entry_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gee_hash_map_entry_set_property;
	G_OBJECT_CLASS (klass)->finalize = gee_hash_map_entry_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_K_TYPE, g_param_spec_gtype ("k-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_K_DUP_FUNC, g_param_spec_pointer ("k-dup-func", "dup func", "dup func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_K_DESTROY_FUNC, g_param_spec_pointer ("k-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_V_TYPE, g_param_spec_gtype ("v-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_V_DUP_FUNC, g_param_spec_pointer ("v-dup-func", "dup func", "dup func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_V_DESTROY_FUNC, g_param_spec_pointer ("v-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_KEY_PROPERTY, gee_hash_map_entry_properties[GEE_HASH_MAP_ENTRY_KEY_PROPERTY] = g_param_spec_pointer ("key", "key", "key", G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_VALUE_PROPERTY, gee_hash_map_entry_properties[GEE_HASH_MAP_ENTRY_VALUE_PROPERTY] = g_param_spec_pointer ("value", "value", "value", G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_READ_ONLY_PROPERTY, gee_hash_map_entry_properties[GEE_HASH_MAP_ENTRY_READ_ONLY_PROPERTY] = g_param_spec_boolean ("read-only", "read-only", "read-only", FALSE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
}

static void
gee_hash_map_entry_instance_init (GeeHashMapEntry * self,
                                  gpointer klass)
{
	self->priv = gee_hash_map_entry_get_instance_private (self);
}

static void
gee_hash_map_entry_finalize (GObject * obj)
{
	GeeHashMapEntry * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GEE_HASH_MAP_TYPE_ENTRY, GeeHashMapEntry);
	G_OBJECT_CLASS (gee_hash_map_entry_parent_class)->finalize (obj);
}

static GType
gee_hash_map_entry_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (GeeHashMapEntryClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_hash_map_entry_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeHashMapEntry), 0, (GInstanceInitFunc) gee_hash_map_entry_instance_init, NULL };
	GType gee_hash_map_entry_type_id;
	gee_hash_map_entry_type_id = g_type_register_static (GEE_MAP_TYPE_ENTRY, "GeeHashMapEntry", &g_define_type_info, 0);
	GeeHashMapEntry_private_offset = g_type_add_instance_private (gee_hash_map_entry_type_id, sizeof (GeeHashMapEntryPrivate));
	return gee_hash_map_entry_type_id;
}

static GType
gee_hash_map_entry_get_type (void)
{
	static volatile gsize gee_hash_map_entry_type_id__once = 0;
	if (g_once_init_enter (&gee_hash_map_entry_type_id__once)) {
		GType gee_hash_map_entry_type_id;
		gee_hash_map_entry_type_id = gee_hash_map_entry_get_type_once ();
		g_once_init_leave (&gee_hash_map_entry_type_id__once, gee_hash_map_entry_type_id);
	}
	return gee_hash_map_entry_type_id__once;
}

static void
_vala_gee_hash_map_entry_get_property (GObject * object,
                                       guint property_id,
                                       GValue * value,
                                       GParamSpec * pspec)
{
	GeeHashMapEntry * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_HASH_MAP_TYPE_ENTRY, GeeHashMapEntry);
	switch (property_id) {
		case GEE_HASH_MAP_ENTRY_KEY_PROPERTY:
		g_value_set_pointer (value, gee_map_entry_get_key ((GeeMapEntry*) self));
		break;
		case GEE_HASH_MAP_ENTRY_VALUE_PROPERTY:
		g_value_set_pointer (value, gee_map_entry_get_value ((GeeMapEntry*) self));
		break;
		case GEE_HASH_MAP_ENTRY_READ_ONLY_PROPERTY:
		g_value_set_boolean (value, gee_map_entry_get_read_only ((GeeMapEntry*) self));
		break;
		case GEE_HASH_MAP_ENTRY_K_TYPE:
		g_value_set_gtype (value, self->priv->k_type);
		break;
		case GEE_HASH_MAP_ENTRY_K_DUP_FUNC:
		g_value_set_pointer (value, self->priv->k_dup_func);
		break;
		case GEE_HASH_MAP_ENTRY_K_DESTROY_FUNC:
		g_value_set_pointer (value, self->priv->k_destroy_func);
		break;
		case GEE_HASH_MAP_ENTRY_V_TYPE:
		g_value_set_gtype (value, self->priv->v_type);
		break;
		case GEE_HASH_MAP_ENTRY_V_DUP_FUNC:
		g_value_set_pointer (value, self->priv->v_dup_func);
		break;
		case GEE_HASH_MAP_ENTRY_V_DESTROY_FUNC:
		g_value_set_pointer (value, self->priv->v_destroy_func);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_gee_hash_map_entry_set_property (GObject * object,
                                       guint property_id,
                                       const GValue * value,
                                       GParamSpec * pspec)
{
	GeeHashMapEntry * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_HASH_MAP_TYPE_ENTRY, GeeHashMapEntry);
	switch (property_id) {
		case GEE_HASH_MAP_ENTRY_VALUE_PROPERTY:
		gee_map_entry_set_value ((GeeMapEntry*) self, g_value_get_pointer (value));
		break;
		case GEE_HASH_MAP_ENTRY_K_TYPE:
		self->priv->k_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_ENTRY_K_DUP_FUNC:
		self->priv->k_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_ENTRY_K_DESTROY_FUNC:
		self->priv->k_destroy_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_ENTRY_V_TYPE:
		self->priv->v_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_ENTRY_V_DUP_FUNC:
		self->priv->v_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_ENTRY_V_DESTROY_FUNC:
		self->priv->v_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static inline gpointer
gee_hash_map_key_set_get_instance_private (GeeHashMapKeySet* self)
{
	return G_STRUCT_MEMBER_P (self, GeeHashMapKeySet_private_offset);
}

static GeeHashMapKeySet*
gee_hash_map_key_set_construct (GType object_type,
                                GType k_type,
                                GBoxedCopyFunc k_dup_func,
                                GDestroyNotify k_destroy_func,
                                GType v_type,
                                GBoxedCopyFunc v_dup_func,
                                GDestroyNotify v_destroy_func,
                                GeeHashMap* map)
{
	GeeHashMapKeySet * self = NULL;
	GeeHashMap* _tmp0_;
	g_return_val_if_fail (map != NULL, NULL);
	self = (GeeHashMapKeySet*) gee_abstract_set_construct (object_type, k_type, (GBoxedCopyFunc) k_dup_func, (GDestroyNotify) k_destroy_func);
	self->priv->k_type = k_type;
	self->priv->k_dup_func = k_dup_func;
	self->priv->k_destroy_func = k_destroy_func;
	self->priv->v_type = v_type;
	self->priv->v_dup_func = v_dup_func;
	self->priv->v_destroy_func = v_destroy_func;
	_tmp0_ = _g_object_ref0 (map);
	_g_object_unref0 (self->priv->_map);
	self->priv->_map = _tmp0_;
	return self;
}

static GeeHashMapKeySet*
gee_hash_map_key_set_new (GType k_type,
                          GBoxedCopyFunc k_dup_func,
                          GDestroyNotify k_destroy_func,
                          GType v_type,
                          GBoxedCopyFunc v_dup_func,
                          GDestroyNotify v_destroy_func,
                          GeeHashMap* map)
{
	return gee_hash_map_key_set_construct (GEE_HASH_MAP_TYPE_KEY_SET, k_type, k_dup_func, k_destroy_func, v_type, v_dup_func, v_destroy_func, map);
}

static GeeIterator*
gee_hash_map_key_set_real_iterator (GeeAbstractCollection* base)
{
	GeeHashMapKeySet * self;
	GeeHashMap* _tmp0_;
	GeeHashMapKeyIterator* _tmp1_;
	GeeIterator* result;
	self = (GeeHashMapKeySet*) base;
	_tmp0_ = self->priv->_map;
	_tmp1_ = gee_hash_map_key_iterator_new (self->priv->k_type, (GBoxedCopyFunc) self->priv->k_dup_func, (GDestroyNotify) self->priv->k_destroy_func, self->priv->v_type, (GBoxedCopyFunc) self->priv->v_dup_func, (GDestroyNotify) self->priv->v_destroy_func, _tmp0_);
	result = (GeeIterator*) _tmp1_;
	return result;
}

static gboolean
gee_hash_map_key_set_real_add (GeeAbstractCollection* base,
                               gconstpointer key)
{
	GeeHashMapKeySet * self;
	self = (GeeHashMapKeySet*) base;
	g_assert_not_reached ();
}

static void
gee_hash_map_key_set_real_clear (GeeAbstractCollection* base)
{
	GeeHashMapKeySet * self;
	self = (GeeHashMapKeySet*) base;
	g_assert_not_reached ();
}

static gboolean
gee_hash_map_key_set_real_remove (GeeAbstractCollection* base,
                                  gconstpointer key)
{
	GeeHashMapKeySet * self;
	self = (GeeHashMapKeySet*) base;
	g_assert_not_reached ();
}

static gboolean
gee_hash_map_key_set_real_contains (GeeAbstractCollection* base,
                                    gconstpointer key)
{
	GeeHashMapKeySet * self;
	GeeHashMap* _tmp0_;
	gboolean result;
	self = (GeeHashMapKeySet*) base;
	_tmp0_ = self->priv->_map;
	result = gee_abstract_map_has_key ((GeeAbstractMap*) _tmp0_, key);
	return result;
}

static gint
gee_hash_map_key_set_real_get_size (GeeAbstractCollection* base)
{
	gint result;
	GeeHashMapKeySet* self;
	GeeHashMap* _tmp0_;
	gint _tmp1_;
	gint _tmp2_;
	self = (GeeHashMapKeySet*) base;
	_tmp0_ = self->priv->_map;
	_tmp1_ = gee_abstract_map_get_size ((GeeAbstractMap*) _tmp0_);
	_tmp2_ = _tmp1_;
	result = _tmp2_;
	return result;
}

static gboolean
gee_hash_map_key_set_real_get_read_only (GeeAbstractCollection* base)
{
	gboolean result;
	GeeHashMapKeySet* self;
	self = (GeeHashMapKeySet*) base;
	result = TRUE;
	return result;
}

static void
gee_hash_map_key_set_class_init (GeeHashMapKeySetClass * klass,
                                 gpointer klass_data)
{
	gee_hash_map_key_set_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &GeeHashMapKeySet_private_offset);
	((GeeAbstractCollectionClass *) klass)->iterator = (GeeIterator* (*) (GeeAbstractCollection*)) gee_hash_map_key_set_real_iterator;
	((GeeAbstractCollectionClass *) klass)->add = (gboolean (*) (GeeAbstractCollection*, gconstpointer)) gee_hash_map_key_set_real_add;
	((GeeAbstractCollectionClass *) klass)->clear = (void (*) (GeeAbstractCollection*)) gee_hash_map_key_set_real_clear;
	((GeeAbstractCollectionClass *) klass)->remove = (gboolean (*) (GeeAbstractCollection*, gconstpointer)) gee_hash_map_key_set_real_remove;
	((GeeAbstractCollectionClass *) klass)->contains = (gboolean (*) (GeeAbstractCollection*, gconstpointer)) gee_hash_map_key_set_real_contains;
	GEE_ABSTRACT_COLLECTION_CLASS (klass)->get_size = gee_hash_map_key_set_real_get_size;
	GEE_ABSTRACT_COLLECTION_CLASS (klass)->get_read_only = gee_hash_map_key_set_real_get_read_only;
	G_OBJECT_CLASS (klass)->get_property = _vala_gee_hash_map_key_set_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gee_hash_map_key_set_set_property;
	G_OBJECT_CLASS (klass)->finalize = gee_hash_map_key_set_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_SET_K_TYPE, g_param_spec_gtype ("k-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_SET_K_DUP_FUNC, g_param_spec_pointer ("k-dup-func", "dup func", "dup func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_SET_K_DESTROY_FUNC, g_param_spec_pointer ("k-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_SET_V_TYPE, g_param_spec_gtype ("v-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_SET_V_DUP_FUNC, g_param_spec_pointer ("v-dup-func", "dup func", "dup func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_SET_V_DESTROY_FUNC, g_param_spec_pointer ("v-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_SET_SIZE_PROPERTY, gee_hash_map_key_set_properties[GEE_HASH_MAP_KEY_SET_SIZE_PROPERTY] = g_param_spec_int ("size", "size", "size", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_SET_READ_ONLY_PROPERTY, gee_hash_map_key_set_properties[GEE_HASH_MAP_KEY_SET_READ_ONLY_PROPERTY] = g_param_spec_boolean ("read-only", "read-only", "read-only", FALSE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
}

static void
gee_hash_map_key_set_instance_init (GeeHashMapKeySet * self,
                                    gpointer klass)
{
	self->priv = gee_hash_map_key_set_get_instance_private (self);
}

static void
gee_hash_map_key_set_finalize (GObject * obj)
{
	GeeHashMapKeySet * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GEE_HASH_MAP_TYPE_KEY_SET, GeeHashMapKeySet);
	_g_object_unref0 (self->priv->_map);
	G_OBJECT_CLASS (gee_hash_map_key_set_parent_class)->finalize (obj);
}

static GType
gee_hash_map_key_set_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (GeeHashMapKeySetClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_hash_map_key_set_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeHashMapKeySet), 0, (GInstanceInitFunc) gee_hash_map_key_set_instance_init, NULL };
	GType gee_hash_map_key_set_type_id;
	gee_hash_map_key_set_type_id = g_type_register_static (GEE_TYPE_ABSTRACT_SET, "GeeHashMapKeySet", &g_define_type_info, 0);
	GeeHashMapKeySet_private_offset = g_type_add_instance_private (gee_hash_map_key_set_type_id, sizeof (GeeHashMapKeySetPrivate));
	return gee_hash_map_key_set_type_id;
}

static GType
gee_hash_map_key_set_get_type (void)
{
	static volatile gsize gee_hash_map_key_set_type_id__once = 0;
	if (g_once_init_enter (&gee_hash_map_key_set_type_id__once)) {
		GType gee_hash_map_key_set_type_id;
		gee_hash_map_key_set_type_id = gee_hash_map_key_set_get_type_once ();
		g_once_init_leave (&gee_hash_map_key_set_type_id__once, gee_hash_map_key_set_type_id);
	}
	return gee_hash_map_key_set_type_id__once;
}

static void
_vala_gee_hash_map_key_set_get_property (GObject * object,
                                         guint property_id,
                                         GValue * value,
                                         GParamSpec * pspec)
{
	GeeHashMapKeySet * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_HASH_MAP_TYPE_KEY_SET, GeeHashMapKeySet);
	switch (property_id) {
		case GEE_HASH_MAP_KEY_SET_SIZE_PROPERTY:
		g_value_set_int (value, gee_abstract_collection_get_size ((GeeAbstractCollection*) self));
		break;
		case GEE_HASH_MAP_KEY_SET_READ_ONLY_PROPERTY:
		g_value_set_boolean (value, gee_abstract_collection_get_read_only ((GeeAbstractCollection*) self));
		break;
		case GEE_HASH_MAP_KEY_SET_K_TYPE:
		g_value_set_gtype (value, self->priv->k_type);
		break;
		case GEE_HASH_MAP_KEY_SET_K_DUP_FUNC:
		g_value_set_pointer (value, self->priv->k_dup_func);
		break;
		case GEE_HASH_MAP_KEY_SET_K_DESTROY_FUNC:
		g_value_set_pointer (value, self->priv->k_destroy_func);
		break;
		case GEE_HASH_MAP_KEY_SET_V_TYPE:
		g_value_set_gtype (value, self->priv->v_type);
		break;
		case GEE_HASH_MAP_KEY_SET_V_DUP_FUNC:
		g_value_set_pointer (value, self->priv->v_dup_func);
		break;
		case GEE_HASH_MAP_KEY_SET_V_DESTROY_FUNC:
		g_value_set_pointer (value, self->priv->v_destroy_func);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_gee_hash_map_key_set_set_property (GObject * object,
                                         guint property_id,
                                         const GValue * value,
                                         GParamSpec * pspec)
{
	GeeHashMapKeySet * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_HASH_MAP_TYPE_KEY_SET, GeeHashMapKeySet);
	switch (property_id) {
		case GEE_HASH_MAP_KEY_SET_K_TYPE:
		self->priv->k_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_KEY_SET_K_DUP_FUNC:
		self->priv->k_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_KEY_SET_K_DESTROY_FUNC:
		self->priv->k_destroy_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_KEY_SET_V_TYPE:
		self->priv->v_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_KEY_SET_V_DUP_FUNC:
		self->priv->v_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_KEY_SET_V_DESTROY_FUNC:
		self->priv->v_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static inline gpointer
gee_hash_map_value_collection_get_instance_private (GeeHashMapValueCollection* self)
{
	return G_STRUCT_MEMBER_P (self, GeeHashMapValueCollection_private_offset);
}

static GeeHashMapValueCollection*
gee_hash_map_value_collection_construct (GType object_type,
                                         GType k_type,
                                         GBoxedCopyFunc k_dup_func,
                                         GDestroyNotify k_destroy_func,
                                         GType v_type,
                                         GBoxedCopyFunc v_dup_func,
                                         GDestroyNotify v_destroy_func,
                                         GeeHashMap* map)
{
	GeeHashMapValueCollection * self = NULL;
	GeeHashMap* _tmp0_;
	g_return_val_if_fail (map != NULL, NULL);
	self = (GeeHashMapValueCollection*) gee_abstract_collection_construct (object_type, v_type, (GBoxedCopyFunc) v_dup_func, (GDestroyNotify) v_destroy_func);
	self->priv->k_type = k_type;
	self->priv->k_dup_func = k_dup_func;
	self->priv->k_destroy_func = k_destroy_func;
	self->priv->v_type = v_type;
	self->priv->v_dup_func = v_dup_func;
	self->priv->v_destroy_func = v_destroy_func;
	_tmp0_ = _g_object_ref0 (map);
	_g_object_unref0 (self->priv->_map);
	self->priv->_map = _tmp0_;
	return self;
}

static GeeHashMapValueCollection*
gee_hash_map_value_collection_new (GType k_type,
                                   GBoxedCopyFunc k_dup_func,
                                   GDestroyNotify k_destroy_func,
                                   GType v_type,
                                   GBoxedCopyFunc v_dup_func,
                                   GDestroyNotify v_destroy_func,
                                   GeeHashMap* map)
{
	return gee_hash_map_value_collection_construct (GEE_HASH_MAP_TYPE_VALUE_COLLECTION, k_type, k_dup_func, k_destroy_func, v_type, v_dup_func, v_destroy_func, map);
}

static GeeIterator*
gee_hash_map_value_collection_real_iterator (GeeAbstractCollection* base)
{
	GeeHashMapValueCollection * self;
	GeeHashMap* _tmp0_;
	GeeHashMapValueIterator* _tmp1_;
	GeeIterator* result;
	self = (GeeHashMapValueCollection*) base;
	_tmp0_ = self->priv->_map;
	_tmp1_ = gee_hash_map_value_iterator_new (self->priv->k_type, (GBoxedCopyFunc) self->priv->k_dup_func, (GDestroyNotify) self->priv->k_destroy_func, self->priv->v_type, (GBoxedCopyFunc) self->priv->v_dup_func, (GDestroyNotify) self->priv->v_destroy_func, _tmp0_);
	result = (GeeIterator*) _tmp1_;
	return result;
}

static gboolean
gee_hash_map_value_collection_real_add (GeeAbstractCollection* base,
                                        gconstpointer value)
{
	GeeHashMapValueCollection * self;
	self = (GeeHashMapValueCollection*) base;
	g_assert_not_reached ();
}

static void
gee_hash_map_value_collection_real_clear (GeeAbstractCollection* base)
{
	GeeHashMapValueCollection * self;
	self = (GeeHashMapValueCollection*) base;
	g_assert_not_reached ();
}

static gboolean
gee_hash_map_value_collection_real_remove (GeeAbstractCollection* base,
                                           gconstpointer value)
{
	GeeHashMapValueCollection * self;
	self = (GeeHashMapValueCollection*) base;
	g_assert_not_reached ();
}

static gboolean
gee_hash_map_value_collection_real_contains (GeeAbstractCollection* base,
                                             gconstpointer value)
{
	GeeHashMapValueCollection * self;
	GeeIterator* it = NULL;
	GeeIterator* _tmp0_;
	gboolean result;
	self = (GeeHashMapValueCollection*) base;
	_tmp0_ = gee_abstract_collection_iterator ((GeeAbstractCollection*) self);
	it = _tmp0_;
	while (TRUE) {
		GeeIterator* _tmp1_;
		GeeHashMap* _tmp2_;
		GeeEqualDataFunc _tmp3_ = NULL;
		gpointer _tmp3__target = NULL;
		GeeEqualDataFunc _tmp4_;
		gpointer _tmp4__target;
		GeeIterator* _tmp5_;
		gpointer _tmp6_;
		gpointer _tmp7_;
		gboolean _tmp8_;
		_tmp1_ = it;
		if (!gee_iterator_next (_tmp1_)) {
			break;
		}
		_tmp2_ = self->priv->_map;
		_tmp3_ = gee_hash_map_get_value_equal_func (_tmp2_, &_tmp3__target);
		_tmp4_ = _tmp3_;
		_tmp4__target = _tmp3__target;
		_tmp5_ = it;
		_tmp6_ = gee_iterator_get (_tmp5_);
		_tmp7_ = _tmp6_;
		_tmp8_ = _tmp4_ (_tmp7_, value, _tmp4__target);
		((_tmp7_ == NULL) || (self->priv->v_destroy_func == NULL)) ? NULL : (_tmp7_ = (self->priv->v_destroy_func (_tmp7_), NULL));
		if (_tmp8_) {
			result = TRUE;
			_g_object_unref0 (it);
			return result;
		}
	}
	result = FALSE;
	_g_object_unref0 (it);
	return result;
}

static gint
gee_hash_map_value_collection_real_get_size (GeeAbstractCollection* base)
{
	gint result;
	GeeHashMapValueCollection* self;
	GeeHashMap* _tmp0_;
	gint _tmp1_;
	gint _tmp2_;
	self = (GeeHashMapValueCollection*) base;
	_tmp0_ = self->priv->_map;
	_tmp1_ = gee_abstract_map_get_size ((GeeAbstractMap*) _tmp0_);
	_tmp2_ = _tmp1_;
	result = _tmp2_;
	return result;
}

static gboolean
gee_hash_map_value_collection_real_get_read_only (GeeAbstractCollection* base)
{
	gboolean result;
	GeeHashMapValueCollection* self;
	self = (GeeHashMapValueCollection*) base;
	result = TRUE;
	return result;
}

static void
gee_hash_map_value_collection_class_init (GeeHashMapValueCollectionClass * klass,
                                          gpointer klass_data)
{
	gee_hash_map_value_collection_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &GeeHashMapValueCollection_private_offset);
	((GeeAbstractCollectionClass *) klass)->iterator = (GeeIterator* (*) (GeeAbstractCollection*)) gee_hash_map_value_collection_real_iterator;
	((GeeAbstractCollectionClass *) klass)->add = (gboolean (*) (GeeAbstractCollection*, gconstpointer)) gee_hash_map_value_collection_real_add;
	((GeeAbstractCollectionClass *) klass)->clear = (void (*) (GeeAbstractCollection*)) gee_hash_map_value_collection_real_clear;
	((GeeAbstractCollectionClass *) klass)->remove = (gboolean (*) (GeeAbstractCollection*, gconstpointer)) gee_hash_map_value_collection_real_remove;
	((GeeAbstractCollectionClass *) klass)->contains = (gboolean (*) (GeeAbstractCollection*, gconstpointer)) gee_hash_map_value_collection_real_contains;
	GEE_ABSTRACT_COLLECTION_CLASS (klass)->get_size = gee_hash_map_value_collection_real_get_size;
	GEE_ABSTRACT_COLLECTION_CLASS (klass)->get_read_only = gee_hash_map_value_collection_real_get_read_only;
	G_OBJECT_CLASS (klass)->get_property = _vala_gee_hash_map_value_collection_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gee_hash_map_value_collection_set_property;
	G_OBJECT_CLASS (klass)->finalize = gee_hash_map_value_collection_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUE_COLLECTION_K_TYPE, g_param_spec_gtype ("k-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUE_COLLECTION_K_DUP_FUNC, g_param_spec_pointer ("k-dup-func", "dup func", "dup func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUE_COLLECTION_K_DESTROY_FUNC, g_param_spec_pointer ("k-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUE_COLLECTION_V_TYPE, g_param_spec_gtype ("v-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUE_COLLECTION_V_DUP_FUNC, g_param_spec_pointer ("v-dup-func", "dup func", "dup func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUE_COLLECTION_V_DESTROY_FUNC, g_param_spec_pointer ("v-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUE_COLLECTION_SIZE_PROPERTY, gee_hash_map_value_collection_properties[GEE_HASH_MAP_VALUE_COLLECTION_SIZE_PROPERTY] = g_param_spec_int ("size", "size", "size", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUE_COLLECTION_READ_ONLY_PROPERTY, gee_hash_map_value_collection_properties[GEE_HASH_MAP_VALUE_COLLECTION_READ_ONLY_PROPERTY] = g_param_spec_boolean ("read-only", "read-only", "read-only", FALSE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
}

static void
gee_hash_map_value_collection_instance_init (GeeHashMapValueCollection * self,
                                             gpointer klass)
{
	self->priv = gee_hash_map_value_collection_get_instance_private (self);
}

static void
gee_hash_map_value_collection_finalize (GObject * obj)
{
	GeeHashMapValueCollection * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GEE_HASH_MAP_TYPE_VALUE_COLLECTION, GeeHashMapValueCollection);
	_g_object_unref0 (self->priv->_map);
	G_OBJECT_CLASS (gee_hash_map_value_collection_parent_class)->finalize (obj);
}

static GType
gee_hash_map_value_collection_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (GeeHashMapValueCollectionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_hash_map_value_collection_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeHashMapValueCollection), 0, (GInstanceInitFunc) gee_hash_map_value_collection_instance_init, NULL };
	GType gee_hash_map_value_collection_type_id;
	gee_hash_map_value_collection_type_id = g_type_register_static (GEE_TYPE_ABSTRACT_COLLECTION, "GeeHashMapValueCollection", &g_define_type_info, 0);
	GeeHashMapValueCollection_private_offset = g_type_add_instance_private (gee_hash_map_value_collection_type_id, sizeof (GeeHashMapValueCollectionPrivate));
	return gee_hash_map_value_collection_type_id;
}

static GType
gee_hash_map_value_collection_get_type (void)
{
	static volatile gsize gee_hash_map_value_collection_type_id__once = 0;
	if (g_once_init_enter (&gee_hash_map_value_collection_type_id__once)) {
		GType gee_hash_map_value_collection_type_id;
		gee_hash_map_value_collection_type_id = gee_hash_map_value_collection_get_type_once ();
		g_once_init_leave (&gee_hash_map_value_collection_type_id__once, gee_hash_map_value_collection_type_id);
	}
	return gee_hash_map_value_collection_type_id__once;
}

static void
_vala_gee_hash_map_value_collection_get_property (GObject * object,
                                                  guint property_id,
                                                  GValue * value,
                                                  GParamSpec * pspec)
{
	GeeHashMapValueCollection * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_HASH_MAP_TYPE_VALUE_COLLECTION, GeeHashMapValueCollection);
	switch (property_id) {
		case GEE_HASH_MAP_VALUE_COLLECTION_SIZE_PROPERTY:
		g_value_set_int (value, gee_abstract_collection_get_size ((GeeAbstractCollection*) self));
		break;
		case GEE_HASH_MAP_VALUE_COLLECTION_READ_ONLY_PROPERTY:
		g_value_set_boolean (value, gee_abstract_collection_get_read_only ((GeeAbstractCollection*) self));
		break;
		case GEE_HASH_MAP_VALUE_COLLECTION_K_TYPE:
		g_value_set_gtype (value, self->priv->k_type);
		break;
		case GEE_HASH_MAP_VALUE_COLLECTION_K_DUP_FUNC:
		g_value_set_pointer (value, self->priv->k_dup_func);
		break;
		case GEE_HASH_MAP_VALUE_COLLECTION_K_DESTROY_FUNC:
		g_value_set_pointer (value, self->priv->k_destroy_func);
		break;
		case GEE_HASH_MAP_VALUE_COLLECTION_V_TYPE:
		g_value_set_gtype (value, self->priv->v_type);
		break;
		case GEE_HASH_MAP_VALUE_COLLECTION_V_DUP_FUNC:
		g_value_set_pointer (value, self->priv->v_dup_func);
		break;
		case GEE_HASH_MAP_VALUE_COLLECTION_V_DESTROY_FUNC:
		g_value_set_pointer (value, self->priv->v_destroy_func);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_gee_hash_map_value_collection_set_property (GObject * object,
                                                  guint property_id,
                                                  const GValue * value,
                                                  GParamSpec * pspec)
{
	GeeHashMapValueCollection * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_HASH_MAP_TYPE_VALUE_COLLECTION, GeeHashMapValueCollection);
	switch (property_id) {
		case GEE_HASH_MAP_VALUE_COLLECTION_K_TYPE:
		self->priv->k_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_VALUE_COLLECTION_K_DUP_FUNC:
		self->priv->k_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_VALUE_COLLECTION_K_DESTROY_FUNC:
		self->priv->k_destroy_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_VALUE_COLLECTION_V_TYPE:
		self->priv->v_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_VALUE_COLLECTION_V_DUP_FUNC:
		self->priv->v_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_VALUE_COLLECTION_V_DESTROY_FUNC:
		self->priv->v_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static inline gpointer
gee_hash_map_entry_set_get_instance_private (GeeHashMapEntrySet* self)
{
	return G_STRUCT_MEMBER_P (self, GeeHashMapEntrySet_private_offset);
}

static GeeHashMapEntrySet*
gee_hash_map_entry_set_construct (GType object_type,
                                  GType k_type,
                                  GBoxedCopyFunc k_dup_func,
                                  GDestroyNotify k_destroy_func,
                                  GType v_type,
                                  GBoxedCopyFunc v_dup_func,
                                  GDestroyNotify v_destroy_func,
                                  GeeHashMap* map)
{
	GeeHashMapEntrySet * self = NULL;
	GeeHashMap* _tmp0_;
	g_return_val_if_fail (map != NULL, NULL);
	self = (GeeHashMapEntrySet*) gee_abstract_set_construct (object_type, GEE_MAP_TYPE_ENTRY, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref);
	self->priv->k_type = k_type;
	self->priv->k_dup_func = k_dup_func;
	self->priv->k_destroy_func = k_destroy_func;
	self->priv->v_type = v_type;
	self->priv->v_dup_func = v_dup_func;
	self->priv->v_destroy_func = v_destroy_func;
	_tmp0_ = _g_object_ref0 (map);
	_g_object_unref0 (self->priv->_map);
	self->priv->_map = _tmp0_;
	return self;
}

static GeeHashMapEntrySet*
gee_hash_map_entry_set_new (GType k_type,
                            GBoxedCopyFunc k_dup_func,
                            GDestroyNotify k_destroy_func,
                            GType v_type,
                            GBoxedCopyFunc v_dup_func,
                            GDestroyNotify v_destroy_func,
                            GeeHashMap* map)
{
	return gee_hash_map_entry_set_construct (GEE_HASH_MAP_TYPE_ENTRY_SET, k_type, k_dup_func, k_destroy_func, v_type, v_dup_func, v_destroy_func, map);
}

static GeeIterator*
gee_hash_map_entry_set_real_iterator (GeeAbstractCollection* base)
{
	GeeHashMapEntrySet * self;
	GeeHashMap* _tmp0_;
	GeeHashMapEntryIterator* _tmp1_;
	GeeIterator* result;
	self = (GeeHashMapEntrySet*) base;
	_tmp0_ = self->priv->_map;
	_tmp1_ = gee_hash_map_entry_iterator_new (self->priv->k_type, (GBoxedCopyFunc) self->priv->k_dup_func, (GDestroyNotify) self->priv->k_destroy_func, self->priv->v_type, (GBoxedCopyFunc) self->priv->v_dup_func, (GDestroyNotify) self->priv->v_destroy_func, _tmp0_);
	result = (GeeIterator*) _tmp1_;
	return result;
}

static gboolean
gee_hash_map_entry_set_real_add (GeeAbstractCollection* base,
                                 GeeMapEntry* entry)
{
	GeeHashMapEntrySet * self;
	self = (GeeHashMapEntrySet*) base;
	g_return_val_if_fail (entry != NULL, FALSE);
	g_assert_not_reached ();
}

static void
gee_hash_map_entry_set_real_clear (GeeAbstractCollection* base)
{
	GeeHashMapEntrySet * self;
	self = (GeeHashMapEntrySet*) base;
	g_assert_not_reached ();
}

static gboolean
gee_hash_map_entry_set_real_remove (GeeAbstractCollection* base,
                                    GeeMapEntry* entry)
{
	GeeHashMapEntrySet * self;
	self = (GeeHashMapEntrySet*) base;
	g_return_val_if_fail (entry != NULL, FALSE);
	g_assert_not_reached ();
}

static gboolean
gee_hash_map_entry_set_real_contains (GeeAbstractCollection* base,
                                      GeeMapEntry* entry)
{
	GeeHashMapEntrySet * self;
	GeeHashMap* _tmp0_;
	gconstpointer _tmp1_;
	gconstpointer _tmp2_;
	gconstpointer _tmp3_;
	gconstpointer _tmp4_;
	gboolean result;
	self = (GeeHashMapEntrySet*) base;
	g_return_val_if_fail (entry != NULL, FALSE);
	_tmp0_ = self->priv->_map;
	_tmp1_ = gee_map_entry_get_key (entry);
	_tmp2_ = _tmp1_;
	_tmp3_ = gee_map_entry_get_value (entry);
	_tmp4_ = _tmp3_;
	result = gee_abstract_map_has ((GeeAbstractMap*) _tmp0_, _tmp2_, _tmp4_);
	return result;
}

static gint
gee_hash_map_entry_set_real_get_size (GeeAbstractCollection* base)
{
	gint result;
	GeeHashMapEntrySet* self;
	GeeHashMap* _tmp0_;
	gint _tmp1_;
	gint _tmp2_;
	self = (GeeHashMapEntrySet*) base;
	_tmp0_ = self->priv->_map;
	_tmp1_ = gee_abstract_map_get_size ((GeeAbstractMap*) _tmp0_);
	_tmp2_ = _tmp1_;
	result = _tmp2_;
	return result;
}

static gboolean
gee_hash_map_entry_set_real_get_read_only (GeeAbstractCollection* base)
{
	gboolean result;
	GeeHashMapEntrySet* self;
	self = (GeeHashMapEntrySet*) base;
	result = TRUE;
	return result;
}

static void
gee_hash_map_entry_set_class_init (GeeHashMapEntrySetClass * klass,
                                   gpointer klass_data)
{
	gee_hash_map_entry_set_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &GeeHashMapEntrySet_private_offset);
	((GeeAbstractCollectionClass *) klass)->iterator = (GeeIterator* (*) (GeeAbstractCollection*)) gee_hash_map_entry_set_real_iterator;
	((GeeAbstractCollectionClass *) klass)->add = (gboolean (*) (GeeAbstractCollection*, gconstpointer)) gee_hash_map_entry_set_real_add;
	((GeeAbstractCollectionClass *) klass)->clear = (void (*) (GeeAbstractCollection*)) gee_hash_map_entry_set_real_clear;
	((GeeAbstractCollectionClass *) klass)->remove = (gboolean (*) (GeeAbstractCollection*, gconstpointer)) gee_hash_map_entry_set_real_remove;
	((GeeAbstractCollectionClass *) klass)->contains = (gboolean (*) (GeeAbstractCollection*, gconstpointer)) gee_hash_map_entry_set_real_contains;
	GEE_ABSTRACT_COLLECTION_CLASS (klass)->get_size = gee_hash_map_entry_set_real_get_size;
	GEE_ABSTRACT_COLLECTION_CLASS (klass)->get_read_only = gee_hash_map_entry_set_real_get_read_only;
	G_OBJECT_CLASS (klass)->get_property = _vala_gee_hash_map_entry_set_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gee_hash_map_entry_set_set_property;
	G_OBJECT_CLASS (klass)->finalize = gee_hash_map_entry_set_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_SET_K_TYPE, g_param_spec_gtype ("k-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_SET_K_DUP_FUNC, g_param_spec_pointer ("k-dup-func", "dup func", "dup func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_SET_K_DESTROY_FUNC, g_param_spec_pointer ("k-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_SET_V_TYPE, g_param_spec_gtype ("v-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_SET_V_DUP_FUNC, g_param_spec_pointer ("v-dup-func", "dup func", "dup func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_SET_V_DESTROY_FUNC, g_param_spec_pointer ("v-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_SET_SIZE_PROPERTY, gee_hash_map_entry_set_properties[GEE_HASH_MAP_ENTRY_SET_SIZE_PROPERTY] = g_param_spec_int ("size", "size", "size", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_SET_READ_ONLY_PROPERTY, gee_hash_map_entry_set_properties[GEE_HASH_MAP_ENTRY_SET_READ_ONLY_PROPERTY] = g_param_spec_boolean ("read-only", "read-only", "read-only", FALSE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
}

static void
gee_hash_map_entry_set_instance_init (GeeHashMapEntrySet * self,
                                      gpointer klass)
{
	self->priv = gee_hash_map_entry_set_get_instance_private (self);
}

static void
gee_hash_map_entry_set_finalize (GObject * obj)
{
	GeeHashMapEntrySet * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GEE_HASH_MAP_TYPE_ENTRY_SET, GeeHashMapEntrySet);
	_g_object_unref0 (self->priv->_map);
	G_OBJECT_CLASS (gee_hash_map_entry_set_parent_class)->finalize (obj);
}

static GType
gee_hash_map_entry_set_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (GeeHashMapEntrySetClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_hash_map_entry_set_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeHashMapEntrySet), 0, (GInstanceInitFunc) gee_hash_map_entry_set_instance_init, NULL };
	GType gee_hash_map_entry_set_type_id;
	gee_hash_map_entry_set_type_id = g_type_register_static (GEE_TYPE_ABSTRACT_SET, "GeeHashMapEntrySet", &g_define_type_info, 0);
	GeeHashMapEntrySet_private_offset = g_type_add_instance_private (gee_hash_map_entry_set_type_id, sizeof (GeeHashMapEntrySetPrivate));
	return gee_hash_map_entry_set_type_id;
}

static GType
gee_hash_map_entry_set_get_type (void)
{
	static volatile gsize gee_hash_map_entry_set_type_id__once = 0;
	if (g_once_init_enter (&gee_hash_map_entry_set_type_id__once)) {
		GType gee_hash_map_entry_set_type_id;
		gee_hash_map_entry_set_type_id = gee_hash_map_entry_set_get_type_once ();
		g_once_init_leave (&gee_hash_map_entry_set_type_id__once, gee_hash_map_entry_set_type_id);
	}
	return gee_hash_map_entry_set_type_id__once;
}

static void
_vala_gee_hash_map_entry_set_get_property (GObject * object,
                                           guint property_id,
                                           GValue * value,
                                           GParamSpec * pspec)
{
	GeeHashMapEntrySet * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_HASH_MAP_TYPE_ENTRY_SET, GeeHashMapEntrySet);
	switch (property_id) {
		case GEE_HASH_MAP_ENTRY_SET_SIZE_PROPERTY:
		g_value_set_int (value, gee_abstract_collection_get_size ((GeeAbstractCollection*) self));
		break;
		case GEE_HASH_MAP_ENTRY_SET_READ_ONLY_PROPERTY:
		g_value_set_boolean (value, gee_abstract_collection_get_read_only ((GeeAbstractCollection*) self));
		break;
		case GEE_HASH_MAP_ENTRY_SET_K_TYPE:
		g_value_set_gtype (value, self->priv->k_type);
		break;
		case GEE_HASH_MAP_ENTRY_SET_K_DUP_FUNC:
		g_value_set_pointer (value, self->priv->k_dup_func);
		break;
		case GEE_HASH_MAP_ENTRY_SET_K_DESTROY_FUNC:
		g_value_set_pointer (value, self->priv->k_destroy_func);
		break;
		case GEE_HASH_MAP_ENTRY_SET_V_TYPE:
		g_value_set_gtype (value, self->priv->v_type);
		break;
		case GEE_HASH_MAP_ENTRY_SET_V_DUP_FUNC:
		g_value_set_pointer (value, self->priv->v_dup_func);
		break;
		case GEE_HASH_MAP_ENTRY_SET_V_DESTROY_FUNC:
		g_value_set_pointer (value, self->priv->v_destroy_func);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_gee_hash_map_entry_set_set_property (GObject * object,
                                           guint property_id,
                                           const GValue * value,
                                           GParamSpec * pspec)
{
	GeeHashMapEntrySet * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_HASH_MAP_TYPE_ENTRY_SET, GeeHashMapEntrySet);
	switch (property_id) {
		case GEE_HASH_MAP_ENTRY_SET_K_TYPE:
		self->priv->k_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_ENTRY_SET_K_DUP_FUNC:
		self->priv->k_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_ENTRY_SET_K_DESTROY_FUNC:
		self->priv->k_destroy_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_ENTRY_SET_V_TYPE:
		self->priv->v_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_ENTRY_SET_V_DUP_FUNC:
		self->priv->v_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_ENTRY_SET_V_DESTROY_FUNC:
		self->priv->v_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static inline gpointer
gee_hash_map_node_iterator_get_instance_private (GeeHashMapNodeIterator* self)
{
	return G_STRUCT_MEMBER_P (self, GeeHashMapNodeIterator_private_offset);
}

static GeeHashMapNodeIterator*
gee_hash_map_node_iterator_construct (GType object_type,
                                      GType k_type,
                                      GBoxedCopyFunc k_dup_func,
                                      GDestroyNotify k_destroy_func,
                                      GType v_type,
                                      GBoxedCopyFunc v_dup_func,
                                      GDestroyNotify v_destroy_func,
                                      GeeHashMap* map)
{
	GeeHashMapNodeIterator * self = NULL;
	GeeHashMap* _tmp0_;
	GeeHashMap* _tmp1_;
	g_return_val_if_fail (map != NULL, NULL);
	self = (GeeHashMapNodeIterator*) g_object_new (object_type, "k-type", k_type, "k-dup-func", k_dup_func, "k-destroy-func", k_destroy_func, "v-type", v_type, "v-dup-func", v_dup_func, "v-destroy-func", v_destroy_func, NULL);
	self->priv->k_type = k_type;
	self->priv->k_dup_func = k_dup_func;
	self->priv->k_destroy_func = k_destroy_func;
	self->priv->v_type = v_type;
	self->priv->v_dup_func = v_dup_func;
	self->priv->v_destroy_func = v_destroy_func;
	_tmp0_ = _g_object_ref0 (map);
	_g_object_unref0 (self->_map);
	self->_map = _tmp0_;
	_tmp1_ = self->_map;
	self->_stamp = _tmp1_->priv->_stamp;
	return self;
}

static GeeHashMapNodeIterator*
gee_hash_map_node_iterator_construct_from_iterator (GType object_type,
                                                    GType k_type,
                                                    GBoxedCopyFunc k_dup_func,
                                                    GDestroyNotify k_destroy_func,
                                                    GType v_type,
                                                    GBoxedCopyFunc v_dup_func,
                                                    GDestroyNotify v_destroy_func,
                                                    GeeHashMapNodeIterator* iter)
{
	GeeHashMapNodeIterator * self = NULL;
	GeeHashMap* _tmp0_;
	GeeHashMap* _tmp1_;
	GeeHashMapNode* _tmp2_;
	GeeHashMapNode* _tmp3_;
	g_return_val_if_fail (iter != NULL, NULL);
	self = (GeeHashMapNodeIterator*) g_object_new (object_type, "k-type", k_type, "k-dup-func", k_dup_func, "k-destroy-func", k_destroy_func, "v-type", v_type, "v-dup-func", v_dup_func, "v-destroy-func", v_destroy_func, NULL);
	self->priv->k_type = k_type;
	self->priv->k_dup_func = k_dup_func;
	self->priv->k_destroy_func = k_destroy_func;
	self->priv->v_type = v_type;
	self->priv->v_dup_func = v_dup_func;
	self->priv->v_destroy_func = v_destroy_func;
	_tmp0_ = iter->_map;
	_tmp1_ = _g_object_ref0 (_tmp0_);
	_g_object_unref0 (self->_map);
	self->_map = _tmp1_;
	self->_index = iter->_index;
	_tmp2_ = iter->_node;
	self->_node = _tmp2_;
	_tmp3_ = iter->_next;
	self->_next = _tmp3_;
	self->_stamp = iter->_stamp;
	return self;
}

static gboolean
gee_hash_map_node_iterator_next (GeeHashMapNodeIterator* self)
{
	GeeHashMap* _tmp0_;
	GeeHashMapNode* _tmp1_;
	GeeHashMapNode* _tmp2_;
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	_tmp0_ = self->_map;
	_vala_assert (self->_stamp == _tmp0_->priv->_stamp, "_stamp == _map._stamp");
	if (!gee_hash_map_node_iterator_has_next (self)) {
		result = FALSE;
		return result;
	}
	_tmp1_ = self->_next;
	self->_node = _tmp1_;
	self->_next = NULL;
	_tmp2_ = self->_node;
	result = _tmp2_ != NULL;
	return result;
}

static gboolean
gee_hash_map_node_iterator_has_next (GeeHashMapNodeIterator* self)
{
	GeeHashMap* _tmp0_;
	GeeHashMapNode* _tmp1_;
	GeeHashMapNode* _tmp13_;
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	_tmp0_ = self->_map;
	_vala_assert (self->_stamp == _tmp0_->priv->_stamp, "_stamp == _map._stamp");
	_tmp1_ = self->_next;
	if (_tmp1_ == NULL) {
		GeeHashMapNode* _tmp2_;
		GeeHashMapNode* _tmp3_;
		_tmp2_ = self->_node;
		self->_next = _tmp2_;
		_tmp3_ = self->_next;
		if (_tmp3_ != NULL) {
			GeeHashMapNode* _tmp4_;
			GeeHashMapNode* _tmp5_;
			_tmp4_ = self->_next;
			_tmp5_ = _tmp4_->next;
			self->_next = _tmp5_;
		}
		while (TRUE) {
			gboolean _tmp6_ = FALSE;
			GeeHashMapNode* _tmp7_;
			gint _tmp9_;
			GeeHashMap* _tmp10_;
			GeeHashMapNode** _tmp11_;
			gint _tmp11__length1;
			GeeHashMapNode* _tmp12_;
			_tmp7_ = self->_next;
			if (_tmp7_ == NULL) {
				GeeHashMap* _tmp8_;
				_tmp8_ = self->_map;
				_tmp6_ = (self->_index + 1) < _tmp8_->priv->_array_size;
			} else {
				_tmp6_ = FALSE;
			}
			if (!_tmp6_) {
				break;
			}
			_tmp9_ = self->_index;
			self->_index = _tmp9_ + 1;
			_tmp10_ = self->_map;
			_tmp11_ = _tmp10_->priv->_nodes;
			_tmp11__length1 = _tmp10_->priv->_nodes_length1;
			_tmp12_ = _tmp11_[self->_index];
			self->_next = _tmp12_;
		}
	}
	_tmp13_ = self->_next;
	result = _tmp13_ != NULL;
	return result;
}

static gboolean
gee_hash_map_node_iterator_get_read_only (GeeHashMapNodeIterator* self)
{
	GeeHashMapNodeIteratorClass* _klass_;
	g_return_val_if_fail (self != NULL, FALSE);
	_klass_ = GEE_HASH_MAP_NODE_ITERATOR_GET_CLASS (self);
	if (_klass_->get_read_only) {
		return _klass_->get_read_only (self);
	}
	return FALSE;
}

static gboolean
gee_hash_map_node_iterator_real_get_read_only (GeeHashMapNodeIterator* base)
{
	gboolean result;
	GeeHashMapNodeIterator* self;
	self = base;
	result = TRUE;
	return result;
}

static gboolean
gee_hash_map_node_iterator_get_valid (GeeHashMapNodeIterator* self)
{
	gboolean result;
	GeeHashMapNode* _tmp0_;
	g_return_val_if_fail (self != NULL, FALSE);
	_tmp0_ = self->_node;
	result = _tmp0_ != NULL;
	return result;
}

static void
gee_hash_map_node_iterator_class_init (GeeHashMapNodeIteratorClass * klass,
                                       gpointer klass_data)
{
	gee_hash_map_node_iterator_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &GeeHashMapNodeIterator_private_offset);
	GEE_HASH_MAP_NODE_ITERATOR_CLASS (klass)->get_read_only = gee_hash_map_node_iterator_real_get_read_only;
	G_OBJECT_CLASS (klass)->get_property = _vala_gee_hash_map_node_iterator_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gee_hash_map_node_iterator_set_property;
	G_OBJECT_CLASS (klass)->finalize = gee_hash_map_node_iterator_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_NODE_ITERATOR_K_TYPE, g_param_spec_gtype ("k-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_NODE_ITERATOR_K_DUP_FUNC, g_param_spec_pointer ("k-dup-func", "dup func", "dup func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_NODE_ITERATOR_K_DESTROY_FUNC, g_param_spec_pointer ("k-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_NODE_ITERATOR_V_TYPE, g_param_spec_gtype ("v-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_NODE_ITERATOR_V_DUP_FUNC, g_param_spec_pointer ("v-dup-func", "dup func", "dup func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_NODE_ITERATOR_V_DESTROY_FUNC, g_param_spec_pointer ("v-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_NODE_ITERATOR_READ_ONLY_PROPERTY, gee_hash_map_node_iterator_properties[GEE_HASH_MAP_NODE_ITERATOR_READ_ONLY_PROPERTY] = g_param_spec_boolean ("read-only", "read-only", "read-only", FALSE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_NODE_ITERATOR_VALID_PROPERTY, gee_hash_map_node_iterator_properties[GEE_HASH_MAP_NODE_ITERATOR_VALID_PROPERTY] = g_param_spec_boolean ("valid", "valid", "valid", FALSE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
}

static void
gee_hash_map_node_iterator_instance_init (GeeHashMapNodeIterator * self,
                                          gpointer klass)
{
	self->priv = gee_hash_map_node_iterator_get_instance_private (self);
	self->_index = -1;
}

static void
gee_hash_map_node_iterator_finalize (GObject * obj)
{
	GeeHashMapNodeIterator * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GEE_HASH_MAP_TYPE_NODE_ITERATOR, GeeHashMapNodeIterator);
	_g_object_unref0 (self->_map);
	G_OBJECT_CLASS (gee_hash_map_node_iterator_parent_class)->finalize (obj);
}

static GType
gee_hash_map_node_iterator_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (GeeHashMapNodeIteratorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_hash_map_node_iterator_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeHashMapNodeIterator), 0, (GInstanceInitFunc) gee_hash_map_node_iterator_instance_init, NULL };
	GType gee_hash_map_node_iterator_type_id;
	gee_hash_map_node_iterator_type_id = g_type_register_static (G_TYPE_OBJECT, "GeeHashMapNodeIterator", &g_define_type_info, G_TYPE_FLAG_ABSTRACT);
	GeeHashMapNodeIterator_private_offset = g_type_add_instance_private (gee_hash_map_node_iterator_type_id, sizeof (GeeHashMapNodeIteratorPrivate));
	return gee_hash_map_node_iterator_type_id;
}

static GType
gee_hash_map_node_iterator_get_type (void)
{
	static volatile gsize gee_hash_map_node_iterator_type_id__once = 0;
	if (g_once_init_enter (&gee_hash_map_node_iterator_type_id__once)) {
		GType gee_hash_map_node_iterator_type_id;
		gee_hash_map_node_iterator_type_id = gee_hash_map_node_iterator_get_type_once ();
		g_once_init_leave (&gee_hash_map_node_iterator_type_id__once, gee_hash_map_node_iterator_type_id);
	}
	return gee_hash_map_node_iterator_type_id__once;
}

static void
_vala_gee_hash_map_node_iterator_get_property (GObject * object,
                                               guint property_id,
                                               GValue * value,
                                               GParamSpec * pspec)
{
	GeeHashMapNodeIterator * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_HASH_MAP_TYPE_NODE_ITERATOR, GeeHashMapNodeIterator);
	switch (property_id) {
		case GEE_HASH_MAP_NODE_ITERATOR_READ_ONLY_PROPERTY:
		g_value_set_boolean (value, gee_hash_map_node_iterator_get_read_only (self));
		break;
		case GEE_HASH_MAP_NODE_ITERATOR_VALID_PROPERTY:
		g_value_set_boolean (value, gee_hash_map_node_iterator_get_valid (self));
		break;
		case GEE_HASH_MAP_NODE_ITERATOR_K_TYPE:
		g_value_set_gtype (value, self->priv->k_type);
		break;
		case GEE_HASH_MAP_NODE_ITERATOR_K_DUP_FUNC:
		g_value_set_pointer (value, self->priv->k_dup_func);
		break;
		case GEE_HASH_MAP_NODE_ITERATOR_K_DESTROY_FUNC:
		g_value_set_pointer (value, self->priv->k_destroy_func);
		break;
		case GEE_HASH_MAP_NODE_ITERATOR_V_TYPE:
		g_value_set_gtype (value, self->priv->v_type);
		break;
		case GEE_HASH_MAP_NODE_ITERATOR_V_DUP_FUNC:
		g_value_set_pointer (value, self->priv->v_dup_func);
		break;
		case GEE_HASH_MAP_NODE_ITERATOR_V_DESTROY_FUNC:
		g_value_set_pointer (value, self->priv->v_destroy_func);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_gee_hash_map_node_iterator_set_property (GObject * object,
                                               guint property_id,
                                               const GValue * value,
                                               GParamSpec * pspec)
{
	GeeHashMapNodeIterator * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_HASH_MAP_TYPE_NODE_ITERATOR, GeeHashMapNodeIterator);
	switch (property_id) {
		case GEE_HASH_MAP_NODE_ITERATOR_K_TYPE:
		self->priv->k_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_NODE_ITERATOR_K_DUP_FUNC:
		self->priv->k_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_NODE_ITERATOR_K_DESTROY_FUNC:
		self->priv->k_destroy_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_NODE_ITERATOR_V_TYPE:
		self->priv->v_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_NODE_ITERATOR_V_DUP_FUNC:
		self->priv->v_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_NODE_ITERATOR_V_DESTROY_FUNC:
		self->priv->v_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static inline gpointer
gee_hash_map_key_iterator_get_instance_private (GeeHashMapKeyIterator* self)
{
	return G_STRUCT_MEMBER_P (self, GeeHashMapKeyIterator_private_offset);
}

static GeeHashMapKeyIterator*
gee_hash_map_key_iterator_construct (GType object_type,
                                     GType k_type,
                                     GBoxedCopyFunc k_dup_func,
                                     GDestroyNotify k_destroy_func,
                                     GType v_type,
                                     GBoxedCopyFunc v_dup_func,
                                     GDestroyNotify v_destroy_func,
                                     GeeHashMap* map)
{
	GeeHashMapKeyIterator * self = NULL;
	g_return_val_if_fail (map != NULL, NULL);
	self = (GeeHashMapKeyIterator*) gee_hash_map_node_iterator_construct (object_type, k_type, (GBoxedCopyFunc) k_dup_func, (GDestroyNotify) k_destroy_func, v_type, (GBoxedCopyFunc) v_dup_func, (GDestroyNotify) v_destroy_func, map);
	self->priv->k_type = k_type;
	self->priv->k_dup_func = k_dup_func;
	self->priv->k_destroy_func = k_destroy_func;
	self->priv->v_type = v_type;
	self->priv->v_dup_func = v_dup_func;
	self->priv->v_destroy_func = v_destroy_func;
	return self;
}

static GeeHashMapKeyIterator*
gee_hash_map_key_iterator_new (GType k_type,
                               GBoxedCopyFunc k_dup_func,
                               GDestroyNotify k_destroy_func,
                               GType v_type,
                               GBoxedCopyFunc v_dup_func,
                               GDestroyNotify v_destroy_func,
                               GeeHashMap* map)
{
	return gee_hash_map_key_iterator_construct (GEE_HASH_MAP_TYPE_KEY_ITERATOR, k_type, k_dup_func, k_destroy_func, v_type, v_dup_func, v_destroy_func, map);
}

static GeeHashMapKeyIterator*
gee_hash_map_key_iterator_construct_from_iterator (GType object_type,
                                                   GType k_type,
                                                   GBoxedCopyFunc k_dup_func,
                                                   GDestroyNotify k_destroy_func,
                                                   GType v_type,
                                                   GBoxedCopyFunc v_dup_func,
                                                   GDestroyNotify v_destroy_func,
                                                   GeeHashMapKeyIterator* iter)
{
	GeeHashMapKeyIterator * self = NULL;
	g_return_val_if_fail (iter != NULL, NULL);
	self = (GeeHashMapKeyIterator*) gee_hash_map_node_iterator_construct_from_iterator (object_type, k_type, (GBoxedCopyFunc) k_dup_func, (GDestroyNotify) k_destroy_func, v_type, (GBoxedCopyFunc) v_dup_func, (GDestroyNotify) v_destroy_func, (GeeHashMapNodeIterator*) iter);
	self->priv->k_type = k_type;
	self->priv->k_dup_func = k_dup_func;
	self->priv->k_destroy_func = k_destroy_func;
	self->priv->v_type = v_type;
	self->priv->v_dup_func = v_dup_func;
	self->priv->v_destroy_func = v_destroy_func;
	return self;
}

static GeeHashMapKeyIterator*
gee_hash_map_key_iterator_new_from_iterator (GType k_type,
                                             GBoxedCopyFunc k_dup_func,
                                             GDestroyNotify k_destroy_func,
                                             GType v_type,
                                             GBoxedCopyFunc v_dup_func,
                                             GDestroyNotify v_destroy_func,
                                             GeeHashMapKeyIterator* iter)
{
	return gee_hash_map_key_iterator_construct_from_iterator (GEE_HASH_MAP_TYPE_KEY_ITERATOR, k_type, k_dup_func, k_destroy_func, v_type, v_dup_func, v_destroy_func, iter);
}

static gpointer
gee_hash_map_key_iterator_real_get (GeeIterator* base)
{
	GeeHashMapKeyIterator * self;
	GeeHashMap* _tmp0_;
	GeeHashMapNode* _tmp1_;
	GeeHashMapNode* _tmp2_;
	gconstpointer _tmp3_;
	gpointer _tmp4_;
	gpointer result;
	self = (GeeHashMapKeyIterator*) base;
	_tmp0_ = ((GeeHashMapNodeIterator*) self)->_map;
	_vala_assert (((GeeHashMapNodeIterator*) self)->_stamp == _tmp0_->priv->_stamp, "_stamp == _map._stamp");
	_tmp1_ = ((GeeHashMapNodeIterator*) self)->_node;
	_vala_assert (_tmp1_ != NULL, "_node != null");
	_tmp2_ = ((GeeHashMapNodeIterator*) self)->_node;
	_tmp3_ = _tmp2_->key;
	_tmp4_ = ((_tmp3_ != NULL) && (self->priv->k_dup_func != NULL)) ? self->priv->k_dup_func ((gpointer) _tmp3_) : ((gpointer) _tmp3_);
	result = _tmp4_;
	return result;
}

static void
gee_hash_map_key_iterator_real_remove (GeeIterator* base)
{
	GeeHashMapKeyIterator * self;
	self = (GeeHashMapKeyIterator*) base;
	g_assert_not_reached ();
}

static gboolean
gee_hash_map_key_iterator_real_foreach (GeeTraversable* base,
                                        GeeForallFunc f,
                                        gpointer f_target)
{
	GeeHashMapKeyIterator * self;
	GeeHashMapNode* _tmp0_;
	gboolean result;
	self = (GeeHashMapKeyIterator*) base;
	_tmp0_ = ((GeeHashMapNodeIterator*) self)->_node;
	if (_tmp0_ != NULL) {
		GeeHashMapNode* _tmp1_;
		gconstpointer _tmp2_;
		gpointer _tmp3_;
		GeeHashMapNode* _tmp4_;
		_tmp1_ = ((GeeHashMapNodeIterator*) self)->_node;
		_tmp2_ = _tmp1_->key;
		_tmp3_ = ((_tmp2_ != NULL) && (self->priv->k_dup_func != NULL)) ? self->priv->k_dup_func ((gpointer) _tmp2_) : ((gpointer) _tmp2_);
		if (!f (_tmp3_, f_target)) {
			result = FALSE;
			return result;
		}
		_tmp4_ = ((GeeHashMapNodeIterator*) self)->_next;
		if (_tmp4_ == NULL) {
			GeeHashMapNode* _tmp5_;
			GeeHashMapNode* _tmp6_;
			_tmp5_ = ((GeeHashMapNodeIterator*) self)->_node;
			_tmp6_ = _tmp5_->next;
			((GeeHashMapNodeIterator*) self)->_next = _tmp6_;
		}
	}
	while (TRUE) {
		GeeHashMap* _tmp14_;
		while (TRUE) {
			GeeHashMapNode* _tmp7_;
			GeeHashMapNode* _tmp8_;
			GeeHashMapNode* _tmp9_;
			gconstpointer _tmp10_;
			gpointer _tmp11_;
			GeeHashMapNode* _tmp12_;
			GeeHashMapNode* _tmp13_;
			_tmp7_ = ((GeeHashMapNodeIterator*) self)->_next;
			if (!(_tmp7_ != NULL)) {
				break;
			}
			_tmp8_ = ((GeeHashMapNodeIterator*) self)->_next;
			((GeeHashMapNodeIterator*) self)->_node = _tmp8_;
			_tmp9_ = ((GeeHashMapNodeIterator*) self)->_node;
			_tmp10_ = _tmp9_->key;
			_tmp11_ = ((_tmp10_ != NULL) && (self->priv->k_dup_func != NULL)) ? self->priv->k_dup_func ((gpointer) _tmp10_) : ((gpointer) _tmp10_);
			if (!f (_tmp11_, f_target)) {
				result = FALSE;
				return result;
			}
			_tmp12_ = ((GeeHashMapNodeIterator*) self)->_next;
			_tmp13_ = _tmp12_->next;
			((GeeHashMapNodeIterator*) self)->_next = _tmp13_;
		}
		_tmp14_ = ((GeeHashMapNodeIterator*) self)->_map;
		if ((((GeeHashMapNodeIterator*) self)->_index + 1) < _tmp14_->priv->_array_size) {
			GeeHashMap* _tmp15_;
			GeeHashMapNode** _tmp16_;
			gint _tmp16__length1;
			gint _tmp17_;
			GeeHashMapNode* _tmp18_;
			_tmp15_ = ((GeeHashMapNodeIterator*) self)->_map;
			_tmp16_ = _tmp15_->priv->_nodes;
			_tmp16__length1 = _tmp15_->priv->_nodes_length1;
			((GeeHashMapNodeIterator*) self)->_index = ((GeeHashMapNodeIterator*) self)->_index + 1;
			_tmp17_ = ((GeeHashMapNodeIterator*) self)->_index;
			_tmp18_ = _tmp16_[_tmp17_];
			((GeeHashMapNodeIterator*) self)->_next = _tmp18_;
		} else {
			result = TRUE;
			return result;
		}
	}
}

static GeeIterator**
gee_hash_map_key_iterator_real_tee (GeeTraversable* base,
                                    guint forks,
                                    gint* result_length1)
{
	GeeHashMapKeyIterator * self;
	GeeIterator** result;
	self = (GeeHashMapKeyIterator*) base;
	if (forks == ((guint) 0)) {
		GeeIterator** _tmp0_;
		GeeIterator** _tmp1_;
		gint _tmp1__length1;
		_tmp0_ = g_new0 (GeeIterator*, 0 + 1);
		_tmp1_ = _tmp0_;
		_tmp1__length1 = 0;
		if (result_length1) {
			*result_length1 = _tmp1__length1;
		}
		result = _tmp1_;
		return result;
	} else {
		GeeIterator** _result_ = NULL;
		GeeIterator** _tmp2_;
		gint _result__length1;
		gint __result__size_;
		GeeIterator** _tmp3_;
		gint _tmp3__length1;
		GeeIterator* _tmp4_;
		GeeIterator** _tmp9_;
		gint _tmp9__length1;
		_tmp2_ = g_new0 (GeeIterator*, forks + 1);
		_result_ = _tmp2_;
		_result__length1 = forks;
		__result__size_ = _result__length1;
		_tmp3_ = _result_;
		_tmp3__length1 = _result__length1;
		_tmp4_ = _g_object_ref0 ((GeeIterator*) self);
		_g_object_unref0 (_tmp3_[0]);
		_tmp3_[0] = _tmp4_;
		{
			guint i = 0U;
			i = (guint) 1;
			{
				gboolean _tmp5_ = FALSE;
				_tmp5_ = TRUE;
				while (TRUE) {
					GeeIterator** _tmp7_;
					gint _tmp7__length1;
					GeeHashMapKeyIterator* _tmp8_;
					if (!_tmp5_) {
						guint _tmp6_;
						_tmp6_ = i;
						i = _tmp6_ + 1;
					}
					_tmp5_ = FALSE;
					if (!(i < forks)) {
						break;
					}
					_tmp7_ = _result_;
					_tmp7__length1 = _result__length1;
					_tmp8_ = gee_hash_map_key_iterator_new_from_iterator (self->priv->k_type, (GBoxedCopyFunc) self->priv->k_dup_func, (GDestroyNotify) self->priv->k_destroy_func, self->priv->v_type, (GBoxedCopyFunc) self->priv->v_dup_func, (GDestroyNotify) self->priv->v_destroy_func, self);
					_g_object_unref0 (_tmp7_[i]);
					_tmp7_[i] = (GeeIterator*) _tmp8_;
				}
			}
		}
		_tmp9_ = _result_;
		_tmp9__length1 = _result__length1;
		if (result_length1) {
			*result_length1 = _tmp9__length1;
		}
		result = _tmp9_;
		return result;
	}
}

static void
gee_hash_map_key_iterator_class_init (GeeHashMapKeyIteratorClass * klass,
                                      gpointer klass_data)
{
	gee_hash_map_key_iterator_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &GeeHashMapKeyIterator_private_offset);
	G_OBJECT_CLASS (klass)->get_property = _vala_gee_hash_map_key_iterator_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gee_hash_map_key_iterator_set_property;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_ITERATOR_K_TYPE, g_param_spec_gtype ("k-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_ITERATOR_K_DUP_FUNC, g_param_spec_pointer ("k-dup-func", "dup func", "dup func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_ITERATOR_K_DESTROY_FUNC, g_param_spec_pointer ("k-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_ITERATOR_V_TYPE, g_param_spec_gtype ("v-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_ITERATOR_V_DUP_FUNC, g_param_spec_pointer ("v-dup-func", "dup func", "dup func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_ITERATOR_V_DESTROY_FUNC, g_param_spec_pointer ("v-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
}

static GType
gee_hash_map_key_iterator_gee_traversable_get_g_type (GeeHashMapKeyIterator* self)
{
	return self->priv->k_type;
}

static GBoxedCopyFunc
gee_hash_map_key_iterator_gee_traversable_get_g_dup_func (GeeHashMapKeyIterator* self)
{
	return self->priv->k_dup_func;
}

static GDestroyNotify
gee_hash_map_key_iterator_gee_traversable_get_g_destroy_func (GeeHashMapKeyIterator* self)
{
	return self->priv->k_destroy_func;
}

static void
gee_hash_map_key_iterator_gee_traversable_interface_init (GeeTraversableIface * iface,
                                                          gpointer iface_data)
{
	gee_hash_map_key_iterator_gee_traversable_parent_iface = g_type_interface_peek_parent (iface);
	iface->foreach = (gboolean (*) (GeeTraversable*, GeeForallFunc, gpointer)) gee_hash_map_key_iterator_real_foreach;
	iface->tee = (GeeIterator** (*) (GeeTraversable*, guint, gint*)) gee_hash_map_key_iterator_real_tee;
	iface->get_g_type = (GType (*) (GeeTraversable *)) gee_hash_map_key_iterator_gee_traversable_get_g_type;
	iface->get_g_dup_func = (GBoxedCopyFunc (*) (GeeTraversable *)) gee_hash_map_key_iterator_gee_traversable_get_g_dup_func;
	iface->get_g_destroy_func = (GDestroyNotify (*) (GeeTraversable *)) gee_hash_map_key_iterator_gee_traversable_get_g_destroy_func;
}

static void
gee_hash_map_key_iterator_gee_iterator_interface_init (GeeIteratorIface * iface,
                                                       gpointer iface_data)
{
	gee_hash_map_key_iterator_gee_iterator_parent_iface = g_type_interface_peek_parent (iface);
	iface->get = (gpointer (*) (GeeIterator*)) gee_hash_map_key_iterator_real_get;
	iface->remove = (void (*) (GeeIterator*)) gee_hash_map_key_iterator_real_remove;
	iface->has_next = (gboolean (*) (GeeIterator*)) gee_hash_map_node_iterator_has_next;
	iface->next = (gboolean (*) (GeeIterator*)) gee_hash_map_node_iterator_next;
	iface->get_valid = (gboolean (*) (GeeIterator *)) gee_hash_map_node_iterator_get_valid;
	iface->get_read_only = (gboolean (*) (GeeIterator *)) gee_hash_map_node_iterator_get_read_only;
}

static void
gee_hash_map_key_iterator_instance_init (GeeHashMapKeyIterator * self,
                                         gpointer klass)
{
	self->priv = gee_hash_map_key_iterator_get_instance_private (self);
}

static GType
gee_hash_map_key_iterator_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (GeeHashMapKeyIteratorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_hash_map_key_iterator_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeHashMapKeyIterator), 0, (GInstanceInitFunc) gee_hash_map_key_iterator_instance_init, NULL };
	static const GInterfaceInfo gee_traversable_info = { (GInterfaceInitFunc) gee_hash_map_key_iterator_gee_traversable_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	static const GInterfaceInfo gee_iterator_info = { (GInterfaceInitFunc) gee_hash_map_key_iterator_gee_iterator_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	GType gee_hash_map_key_iterator_type_id;
	gee_hash_map_key_iterator_type_id = g_type_register_static (GEE_HASH_MAP_TYPE_NODE_ITERATOR, "GeeHashMapKeyIterator", &g_define_type_info, 0);
	g_type_add_interface_static (gee_hash_map_key_iterator_type_id, GEE_TYPE_TRAVERSABLE, &gee_traversable_info);
	g_type_add_interface_static (gee_hash_map_key_iterator_type_id, GEE_TYPE_ITERATOR, &gee_iterator_info);
	GeeHashMapKeyIterator_private_offset = g_type_add_instance_private (gee_hash_map_key_iterator_type_id, sizeof (GeeHashMapKeyIteratorPrivate));
	return gee_hash_map_key_iterator_type_id;
}

static GType
gee_hash_map_key_iterator_get_type (void)
{
	static volatile gsize gee_hash_map_key_iterator_type_id__once = 0;
	if (g_once_init_enter (&gee_hash_map_key_iterator_type_id__once)) {
		GType gee_hash_map_key_iterator_type_id;
		gee_hash_map_key_iterator_type_id = gee_hash_map_key_iterator_get_type_once ();
		g_once_init_leave (&gee_hash_map_key_iterator_type_id__once, gee_hash_map_key_iterator_type_id);
	}
	return gee_hash_map_key_iterator_type_id__once;
}

static void
_vala_gee_hash_map_key_iterator_get_property (GObject * object,
                                              guint property_id,
                                              GValue * value,
                                              GParamSpec * pspec)
{
	GeeHashMapKeyIterator * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_HASH_MAP_TYPE_KEY_ITERATOR, GeeHashMapKeyIterator);
	switch (property_id) {
		case GEE_HASH_MAP_KEY_ITERATOR_K_TYPE:
		g_value_set_gtype (value, self->priv->k_type);
		break;
		case GEE_HASH_MAP_KEY_ITERATOR_K_DUP_FUNC:
		g_value_set_pointer (value, self->priv->k_dup_func);
		break;
		case GEE_HASH_MAP_KEY_ITERATOR_K_DESTROY_FUNC:
		g_value_set_pointer (value, self->priv->k_destroy_func);
		break;
		case GEE_HASH_MAP_KEY_ITERATOR_V_TYPE:
		g_value_set_gtype (value, self->priv->v_type);
		break;
		case GEE_HASH_MAP_KEY_ITERATOR_V_DUP_FUNC:
		g_value_set_pointer (value, self->priv->v_dup_func);
		break;
		case GEE_HASH_MAP_KEY_ITERATOR_V_DESTROY_FUNC:
		g_value_set_pointer (value, self->priv->v_destroy_func);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_gee_hash_map_key_iterator_set_property (GObject * object,
                                              guint property_id,
                                              const GValue * value,
                                              GParamSpec * pspec)
{
	GeeHashMapKeyIterator * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_HASH_MAP_TYPE_KEY_ITERATOR, GeeHashMapKeyIterator);
	switch (property_id) {
		case GEE_HASH_MAP_KEY_ITERATOR_K_TYPE:
		self->priv->k_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_KEY_ITERATOR_K_DUP_FUNC:
		self->priv->k_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_KEY_ITERATOR_K_DESTROY_FUNC:
		self->priv->k_destroy_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_KEY_ITERATOR_V_TYPE:
		self->priv->v_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_KEY_ITERATOR_V_DUP_FUNC:
		self->priv->v_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_KEY_ITERATOR_V_DESTROY_FUNC:
		self->priv->v_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static inline gpointer
gee_hash_map_map_iterator_get_instance_private (GeeHashMapMapIterator* self)
{
	return G_STRUCT_MEMBER_P (self, GeeHashMapMapIterator_private_offset);
}

static GeeHashMapMapIterator*
gee_hash_map_map_iterator_construct (GType object_type,
                                     GType k_type,
                                     GBoxedCopyFunc k_dup_func,
                                     GDestroyNotify k_destroy_func,
                                     GType v_type,
                                     GBoxedCopyFunc v_dup_func,
                                     GDestroyNotify v_destroy_func,
                                     GeeHashMap* map)
{
	GeeHashMapMapIterator * self = NULL;
	g_return_val_if_fail (map != NULL, NULL);
	self = (GeeHashMapMapIterator*) gee_hash_map_node_iterator_construct (object_type, k_type, (GBoxedCopyFunc) k_dup_func, (GDestroyNotify) k_destroy_func, v_type, (GBoxedCopyFunc) v_dup_func, (GDestroyNotify) v_destroy_func, map);
	self->priv->k_type = k_type;
	self->priv->k_dup_func = k_dup_func;
	self->priv->k_destroy_func = k_destroy_func;
	self->priv->v_type = v_type;
	self->priv->v_dup_func = v_dup_func;
	self->priv->v_destroy_func = v_destroy_func;
	return self;
}

static GeeHashMapMapIterator*
gee_hash_map_map_iterator_new (GType k_type,
                               GBoxedCopyFunc k_dup_func,
                               GDestroyNotify k_destroy_func,
                               GType v_type,
                               GBoxedCopyFunc v_dup_func,
                               GDestroyNotify v_destroy_func,
                               GeeHashMap* map)
{
	return gee_hash_map_map_iterator_construct (GEE_HASH_MAP_TYPE_MAP_ITERATOR, k_type, k_dup_func, k_destroy_func, v_type, v_dup_func, v_destroy_func, map);
}

static gpointer
gee_hash_map_map_iterator_real_get_key (GeeMapIterator* base)
{
	GeeHashMapMapIterator * self;
	GeeHashMap* _tmp0_;
	GeeHashMapNode* _tmp1_;
	GeeHashMapNode* _tmp2_;
	gconstpointer _tmp3_;
	gpointer _tmp4_;
	gpointer result;
	self = (GeeHashMapMapIterator*) base;
	_tmp0_ = ((GeeHashMapNodeIterator*) self)->_map;
	_vala_assert (((GeeHashMapNodeIterator*) self)->_stamp == _tmp0_->priv->_stamp, "_stamp == _map._stamp");
	_tmp1_ = ((GeeHashMapNodeIterator*) self)->_node;
	_vala_assert (_tmp1_ != NULL, "_node != null");
	_tmp2_ = ((GeeHashMapNodeIterator*) self)->_node;
	_tmp3_ = _tmp2_->key;
	_tmp4_ = ((_tmp3_ != NULL) && (self->priv->k_dup_func != NULL)) ? self->priv->k_dup_func ((gpointer) _tmp3_) : ((gpointer) _tmp3_);
	result = _tmp4_;
	return result;
}

static void
gee_hash_map_map_iterator_real_unset (GeeMapIterator* base)
{
	GeeHashMapMapIterator * self;
	GeeHashMap* _tmp0_;
	GeeHashMapNode* _tmp1_;
	GeeHashMap* _tmp2_;
	GeeHashMapNode* _tmp3_;
	gconstpointer _tmp4_;
	GeeHashMap* _tmp5_;
	self = (GeeHashMapMapIterator*) base;
	_tmp0_ = ((GeeHashMapNodeIterator*) self)->_map;
	_vala_assert (((GeeHashMapNodeIterator*) self)->_stamp == _tmp0_->priv->_stamp, "_stamp == _map._stamp");
	_tmp1_ = ((GeeHashMapNodeIterator*) self)->_node;
	_vala_assert (_tmp1_ != NULL, "_node != null");
	gee_map_iterator_has_next ((GeeMapIterator*) self);
	_tmp2_ = ((GeeHashMapNodeIterator*) self)->_map;
	_tmp3_ = ((GeeHashMapNodeIterator*) self)->_node;
	_tmp4_ = _tmp3_->key;
	gee_hash_map_unset_helper (_tmp2_, _tmp4_, NULL);
	((GeeHashMapNodeIterator*) self)->_node = NULL;
	_tmp5_ = ((GeeHashMapNodeIterator*) self)->_map;
	((GeeHashMapNodeIterator*) self)->_stamp = _tmp5_->priv->_stamp;
}

static gpointer
gee_hash_map_map_iterator_real_get_value (GeeMapIterator* base)
{
	GeeHashMapMapIterator * self;
	GeeHashMap* _tmp0_;
	GeeHashMapNode* _tmp1_;
	GeeHashMapNode* _tmp2_;
	gconstpointer _tmp3_;
	gpointer _tmp4_;
	gpointer result;
	self = (GeeHashMapMapIterator*) base;
	_tmp0_ = ((GeeHashMapNodeIterator*) self)->_map;
	_vala_assert (((GeeHashMapNodeIterator*) self)->_stamp == _tmp0_->priv->_stamp, "_stamp == _map._stamp");
	_tmp1_ = ((GeeHashMapNodeIterator*) self)->_node;
	_vala_assert (_tmp1_ != NULL, "_node != null");
	_tmp2_ = ((GeeHashMapNodeIterator*) self)->_node;
	_tmp3_ = _tmp2_->value;
	_tmp4_ = ((_tmp3_ != NULL) && (self->priv->v_dup_func != NULL)) ? self->priv->v_dup_func ((gpointer) _tmp3_) : ((gpointer) _tmp3_);
	result = _tmp4_;
	return result;
}

static void
gee_hash_map_map_iterator_real_set_value (GeeMapIterator* base,
                                          gconstpointer value)
{
	GeeHashMapMapIterator * self;
	GeeHashMap* _tmp0_;
	GeeHashMapNode* _tmp1_;
	GeeHashMap* _tmp2_;
	GeeHashMapNode* _tmp3_;
	gconstpointer _tmp4_;
	GeeHashMap* _tmp5_;
	self = (GeeHashMapMapIterator*) base;
	_tmp0_ = ((GeeHashMapNodeIterator*) self)->_map;
	_vala_assert (((GeeHashMapNodeIterator*) self)->_stamp == _tmp0_->priv->_stamp, "_stamp == _map._stamp");
	_tmp1_ = ((GeeHashMapNodeIterator*) self)->_node;
	_vala_assert (_tmp1_ != NULL, "_node != null");
	_tmp2_ = ((GeeHashMapNodeIterator*) self)->_map;
	_tmp3_ = ((GeeHashMapNodeIterator*) self)->_node;
	_tmp4_ = _tmp3_->key;
	gee_abstract_map_set ((GeeAbstractMap*) _tmp2_, _tmp4_, value);
	_tmp5_ = ((GeeHashMapNodeIterator*) self)->_map;
	((GeeHashMapNodeIterator*) self)->_stamp = _tmp5_->priv->_stamp;
}

static gboolean
gee_hash_map_map_iterator_real_get_mutable (GeeMapIterator* base)
{
	gboolean result;
	GeeHashMapMapIterator* self;
	self = (GeeHashMapMapIterator*) base;
	result = TRUE;
	return result;
}

static gboolean
gee_hash_map_map_iterator_real_get_read_only (GeeHashMapNodeIterator* base)
{
	gboolean result;
	GeeHashMapMapIterator* self;
	self = (GeeHashMapMapIterator*) base;
	result = FALSE;
	return result;
}

static void
gee_hash_map_map_iterator_class_init (GeeHashMapMapIteratorClass * klass,
                                      gpointer klass_data)
{
	gee_hash_map_map_iterator_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &GeeHashMapMapIterator_private_offset);
	GEE_HASH_MAP_NODE_ITERATOR_CLASS (klass)->get_read_only = gee_hash_map_map_iterator_real_get_read_only;
	G_OBJECT_CLASS (klass)->get_property = _vala_gee_hash_map_map_iterator_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gee_hash_map_map_iterator_set_property;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_MAP_ITERATOR_K_TYPE, g_param_spec_gtype ("k-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_MAP_ITERATOR_K_DUP_FUNC, g_param_spec_pointer ("k-dup-func", "dup func", "dup func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_MAP_ITERATOR_K_DESTROY_FUNC, g_param_spec_pointer ("k-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_MAP_ITERATOR_V_TYPE, g_param_spec_gtype ("v-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_MAP_ITERATOR_V_DUP_FUNC, g_param_spec_pointer ("v-dup-func", "dup func", "dup func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_MAP_ITERATOR_V_DESTROY_FUNC, g_param_spec_pointer ("v-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_MAP_ITERATOR_MUTABLE_PROPERTY, gee_hash_map_map_iterator_properties[GEE_HASH_MAP_MAP_ITERATOR_MUTABLE_PROPERTY] = g_param_spec_boolean ("mutable", "mutable", "mutable", FALSE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_MAP_ITERATOR_READ_ONLY_PROPERTY, gee_hash_map_map_iterator_properties[GEE_HASH_MAP_MAP_ITERATOR_READ_ONLY_PROPERTY] = g_param_spec_boolean ("read-only", "read-only", "read-only", FALSE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
}

static GType
gee_hash_map_map_iterator_gee_map_iterator_get_k_type (GeeHashMapMapIterator* self)
{
	return self->priv->k_type;
}

static GBoxedCopyFunc
gee_hash_map_map_iterator_gee_map_iterator_get_k_dup_func (GeeHashMapMapIterator* self)
{
	return self->priv->k_dup_func;
}

static GDestroyNotify
gee_hash_map_map_iterator_gee_map_iterator_get_k_destroy_func (GeeHashMapMapIterator* self)
{
	return self->priv->k_destroy_func;
}

static GType
gee_hash_map_map_iterator_gee_map_iterator_get_v_type (GeeHashMapMapIterator* self)
{
	return self->priv->v_type;
}

static GBoxedCopyFunc
gee_hash_map_map_iterator_gee_map_iterator_get_v_dup_func (GeeHashMapMapIterator* self)
{
	return self->priv->v_dup_func;
}

static GDestroyNotify
gee_hash_map_map_iterator_gee_map_iterator_get_v_destroy_func (GeeHashMapMapIterator* self)
{
	return self->priv->v_destroy_func;
}

static void
gee_hash_map_map_iterator_gee_map_iterator_interface_init (GeeMapIteratorIface * iface,
                                                           gpointer iface_data)
{
	gee_hash_map_map_iterator_gee_map_iterator_parent_iface = g_type_interface_peek_parent (iface);
	iface->get_key = (gpointer (*) (GeeMapIterator*)) gee_hash_map_map_iterator_real_get_key;
	iface->unset = (void (*) (GeeMapIterator*)) gee_hash_map_map_iterator_real_unset;
	iface->get_value = (gpointer (*) (GeeMapIterator*)) gee_hash_map_map_iterator_real_get_value;
	iface->set_value = (void (*) (GeeMapIterator*, gconstpointer)) gee_hash_map_map_iterator_real_set_value;
	iface->get_k_type = (GType (*) (GeeMapIterator *)) gee_hash_map_map_iterator_gee_map_iterator_get_k_type;
	iface->get_k_dup_func = (GBoxedCopyFunc (*) (GeeMapIterator *)) gee_hash_map_map_iterator_gee_map_iterator_get_k_dup_func;
	iface->get_k_destroy_func = (GDestroyNotify (*) (GeeMapIterator *)) gee_hash_map_map_iterator_gee_map_iterator_get_k_destroy_func;
	iface->get_v_type = (GType (*) (GeeMapIterator *)) gee_hash_map_map_iterator_gee_map_iterator_get_v_type;
	iface->get_v_dup_func = (GBoxedCopyFunc (*) (GeeMapIterator *)) gee_hash_map_map_iterator_gee_map_iterator_get_v_dup_func;
	iface->get_v_destroy_func = (GDestroyNotify (*) (GeeMapIterator *)) gee_hash_map_map_iterator_gee_map_iterator_get_v_destroy_func;
	iface->next = (gboolean (*) (GeeMapIterator*)) gee_hash_map_node_iterator_next;
	iface->has_next = (gboolean (*) (GeeMapIterator*)) gee_hash_map_node_iterator_has_next;
	iface->get_mutable = gee_hash_map_map_iterator_real_get_mutable;
	iface->get_read_only = gee_hash_map_map_iterator_real_get_read_only;
	iface->get_valid = (gboolean (*) (GeeMapIterator *)) gee_hash_map_node_iterator_get_valid;
}

static void
gee_hash_map_map_iterator_instance_init (GeeHashMapMapIterator * self,
                                         gpointer klass)
{
	self->priv = gee_hash_map_map_iterator_get_instance_private (self);
}

static GType
gee_hash_map_map_iterator_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (GeeHashMapMapIteratorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_hash_map_map_iterator_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeHashMapMapIterator), 0, (GInstanceInitFunc) gee_hash_map_map_iterator_instance_init, NULL };
	static const GInterfaceInfo gee_map_iterator_info = { (GInterfaceInitFunc) gee_hash_map_map_iterator_gee_map_iterator_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	GType gee_hash_map_map_iterator_type_id;
	gee_hash_map_map_iterator_type_id = g_type_register_static (GEE_HASH_MAP_TYPE_NODE_ITERATOR, "GeeHashMapMapIterator", &g_define_type_info, 0);
	g_type_add_interface_static (gee_hash_map_map_iterator_type_id, GEE_TYPE_MAP_ITERATOR, &gee_map_iterator_info);
	GeeHashMapMapIterator_private_offset = g_type_add_instance_private (gee_hash_map_map_iterator_type_id, sizeof (GeeHashMapMapIteratorPrivate));
	return gee_hash_map_map_iterator_type_id;
}

static GType
gee_hash_map_map_iterator_get_type (void)
{
	static volatile gsize gee_hash_map_map_iterator_type_id__once = 0;
	if (g_once_init_enter (&gee_hash_map_map_iterator_type_id__once)) {
		GType gee_hash_map_map_iterator_type_id;
		gee_hash_map_map_iterator_type_id = gee_hash_map_map_iterator_get_type_once ();
		g_once_init_leave (&gee_hash_map_map_iterator_type_id__once, gee_hash_map_map_iterator_type_id);
	}
	return gee_hash_map_map_iterator_type_id__once;
}

static void
_vala_gee_hash_map_map_iterator_get_property (GObject * object,
                                              guint property_id,
                                              GValue * value,
                                              GParamSpec * pspec)
{
	GeeHashMapMapIterator * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_HASH_MAP_TYPE_MAP_ITERATOR, GeeHashMapMapIterator);
	switch (property_id) {
		case GEE_HASH_MAP_MAP_ITERATOR_MUTABLE_PROPERTY:
		g_value_set_boolean (value, gee_map_iterator_get_mutable ((GeeMapIterator*) self));
		break;
		case GEE_HASH_MAP_MAP_ITERATOR_READ_ONLY_PROPERTY:
		g_value_set_boolean (value, gee_hash_map_node_iterator_get_read_only ((GeeHashMapNodeIterator*) self));
		break;
		case GEE_HASH_MAP_MAP_ITERATOR_K_TYPE:
		g_value_set_gtype (value, self->priv->k_type);
		break;
		case GEE_HASH_MAP_MAP_ITERATOR_K_DUP_FUNC:
		g_value_set_pointer (value, self->priv->k_dup_func);
		break;
		case GEE_HASH_MAP_MAP_ITERATOR_K_DESTROY_FUNC:
		g_value_set_pointer (value, self->priv->k_destroy_func);
		break;
		case GEE_HASH_MAP_MAP_ITERATOR_V_TYPE:
		g_value_set_gtype (value, self->priv->v_type);
		break;
		case GEE_HASH_MAP_MAP_ITERATOR_V_DUP_FUNC:
		g_value_set_pointer (value, self->priv->v_dup_func);
		break;
		case GEE_HASH_MAP_MAP_ITERATOR_V_DESTROY_FUNC:
		g_value_set_pointer (value, self->priv->v_destroy_func);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_gee_hash_map_map_iterator_set_property (GObject * object,
                                              guint property_id,
                                              const GValue * value,
                                              GParamSpec * pspec)
{
	GeeHashMapMapIterator * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_HASH_MAP_TYPE_MAP_ITERATOR, GeeHashMapMapIterator);
	switch (property_id) {
		case GEE_HASH_MAP_MAP_ITERATOR_K_TYPE:
		self->priv->k_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_MAP_ITERATOR_K_DUP_FUNC:
		self->priv->k_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_MAP_ITERATOR_K_DESTROY_FUNC:
		self->priv->k_destroy_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_MAP_ITERATOR_V_TYPE:
		self->priv->v_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_MAP_ITERATOR_V_DUP_FUNC:
		self->priv->v_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_MAP_ITERATOR_V_DESTROY_FUNC:
		self->priv->v_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static inline gpointer
gee_hash_map_value_iterator_get_instance_private (GeeHashMapValueIterator* self)
{
	return G_STRUCT_MEMBER_P (self, GeeHashMapValueIterator_private_offset);
}

static GeeHashMapValueIterator*
gee_hash_map_value_iterator_construct (GType object_type,
                                       GType k_type,
                                       GBoxedCopyFunc k_dup_func,
                                       GDestroyNotify k_destroy_func,
                                       GType v_type,
                                       GBoxedCopyFunc v_dup_func,
                                       GDestroyNotify v_destroy_func,
                                       GeeHashMap* map)
{
	GeeHashMapValueIterator * self = NULL;
	g_return_val_if_fail (map != NULL, NULL);
	self = (GeeHashMapValueIterator*) gee_hash_map_node_iterator_construct (object_type, k_type, (GBoxedCopyFunc) k_dup_func, (GDestroyNotify) k_destroy_func, v_type, (GBoxedCopyFunc) v_dup_func, (GDestroyNotify) v_destroy_func, map);
	self->priv->k_type = k_type;
	self->priv->k_dup_func = k_dup_func;
	self->priv->k_destroy_func = k_destroy_func;
	self->priv->v_type = v_type;
	self->priv->v_dup_func = v_dup_func;
	self->priv->v_destroy_func = v_destroy_func;
	return self;
}

static GeeHashMapValueIterator*
gee_hash_map_value_iterator_new (GType k_type,
                                 GBoxedCopyFunc k_dup_func,
                                 GDestroyNotify k_destroy_func,
                                 GType v_type,
                                 GBoxedCopyFunc v_dup_func,
                                 GDestroyNotify v_destroy_func,
                                 GeeHashMap* map)
{
	return gee_hash_map_value_iterator_construct (GEE_HASH_MAP_TYPE_VALUE_ITERATOR, k_type, k_dup_func, k_destroy_func, v_type, v_dup_func, v_destroy_func, map);
}

static GeeHashMapValueIterator*
gee_hash_map_value_iterator_construct_from_iterator (GType object_type,
                                                     GType k_type,
                                                     GBoxedCopyFunc k_dup_func,
                                                     GDestroyNotify k_destroy_func,
                                                     GType v_type,
                                                     GBoxedCopyFunc v_dup_func,
                                                     GDestroyNotify v_destroy_func,
                                                     GeeHashMapValueIterator* iter)
{
	GeeHashMapValueIterator * self = NULL;
	g_return_val_if_fail (iter != NULL, NULL);
	self = (GeeHashMapValueIterator*) gee_hash_map_node_iterator_construct_from_iterator (object_type, k_type, (GBoxedCopyFunc) k_dup_func, (GDestroyNotify) k_destroy_func, v_type, (GBoxedCopyFunc) v_dup_func, (GDestroyNotify) v_destroy_func, (GeeHashMapNodeIterator*) iter);
	self->priv->k_type = k_type;
	self->priv->k_dup_func = k_dup_func;
	self->priv->k_destroy_func = k_destroy_func;
	self->priv->v_type = v_type;
	self->priv->v_dup_func = v_dup_func;
	self->priv->v_destroy_func = v_destroy_func;
	return self;
}

static GeeHashMapValueIterator*
gee_hash_map_value_iterator_new_from_iterator (GType k_type,
                                               GBoxedCopyFunc k_dup_func,
                                               GDestroyNotify k_destroy_func,
                                               GType v_type,
                                               GBoxedCopyFunc v_dup_func,
                                               GDestroyNotify v_destroy_func,
                                               GeeHashMapValueIterator* iter)
{
	return gee_hash_map_value_iterator_construct_from_iterator (GEE_HASH_MAP_TYPE_VALUE_ITERATOR, k_type, k_dup_func, k_destroy_func, v_type, v_dup_func, v_destroy_func, iter);
}

static gpointer
gee_hash_map_value_iterator_real_get (GeeIterator* base)
{
	GeeHashMapValueIterator * self;
	GeeHashMap* _tmp0_;
	GeeHashMapNode* _tmp1_;
	GeeHashMapNode* _tmp2_;
	gconstpointer _tmp3_;
	gpointer _tmp4_;
	gpointer result;
	self = (GeeHashMapValueIterator*) base;
	_tmp0_ = ((GeeHashMapNodeIterator*) self)->_map;
	_vala_assert (((GeeHashMapNodeIterator*) self)->_stamp == _tmp0_->priv->_stamp, "_stamp == _map._stamp");
	_tmp1_ = ((GeeHashMapNodeIterator*) self)->_node;
	_vala_assert (_tmp1_ != NULL, "_node != null");
	_tmp2_ = ((GeeHashMapNodeIterator*) self)->_node;
	_tmp3_ = _tmp2_->value;
	_tmp4_ = ((_tmp3_ != NULL) && (self->priv->v_dup_func != NULL)) ? self->priv->v_dup_func ((gpointer) _tmp3_) : ((gpointer) _tmp3_);
	result = _tmp4_;
	return result;
}

static void
gee_hash_map_value_iterator_real_remove (GeeIterator* base)
{
	GeeHashMapValueIterator * self;
	self = (GeeHashMapValueIterator*) base;
	g_assert_not_reached ();
}

static gboolean
gee_hash_map_value_iterator_real_foreach (GeeTraversable* base,
                                          GeeForallFunc f,
                                          gpointer f_target)
{
	GeeHashMapValueIterator * self;
	GeeHashMapNode* _tmp0_;
	gboolean result;
	self = (GeeHashMapValueIterator*) base;
	_tmp0_ = ((GeeHashMapNodeIterator*) self)->_node;
	if (_tmp0_ != NULL) {
		GeeHashMapNode* _tmp1_;
		gconstpointer _tmp2_;
		gpointer _tmp3_;
		GeeHashMapNode* _tmp4_;
		_tmp1_ = ((GeeHashMapNodeIterator*) self)->_node;
		_tmp2_ = _tmp1_->value;
		_tmp3_ = ((_tmp2_ != NULL) && (self->priv->v_dup_func != NULL)) ? self->priv->v_dup_func ((gpointer) _tmp2_) : ((gpointer) _tmp2_);
		if (!f (_tmp3_, f_target)) {
			result = FALSE;
			return result;
		}
		_tmp4_ = ((GeeHashMapNodeIterator*) self)->_next;
		if (_tmp4_ == NULL) {
			GeeHashMapNode* _tmp5_;
			GeeHashMapNode* _tmp6_;
			_tmp5_ = ((GeeHashMapNodeIterator*) self)->_node;
			_tmp6_ = _tmp5_->next;
			((GeeHashMapNodeIterator*) self)->_next = _tmp6_;
		}
	}
	while (TRUE) {
		GeeHashMap* _tmp14_;
		while (TRUE) {
			GeeHashMapNode* _tmp7_;
			GeeHashMapNode* _tmp8_;
			GeeHashMapNode* _tmp9_;
			gconstpointer _tmp10_;
			gpointer _tmp11_;
			GeeHashMapNode* _tmp12_;
			GeeHashMapNode* _tmp13_;
			_tmp7_ = ((GeeHashMapNodeIterator*) self)->_next;
			if (!(_tmp7_ != NULL)) {
				break;
			}
			_tmp8_ = ((GeeHashMapNodeIterator*) self)->_next;
			((GeeHashMapNodeIterator*) self)->_node = _tmp8_;
			_tmp9_ = ((GeeHashMapNodeIterator*) self)->_node;
			_tmp10_ = _tmp9_->value;
			_tmp11_ = ((_tmp10_ != NULL) && (self->priv->v_dup_func != NULL)) ? self->priv->v_dup_func ((gpointer) _tmp10_) : ((gpointer) _tmp10_);
			if (!f (_tmp11_, f_target)) {
				result = FALSE;
				return result;
			}
			_tmp12_ = ((GeeHashMapNodeIterator*) self)->_next;
			_tmp13_ = _tmp12_->next;
			((GeeHashMapNodeIterator*) self)->_next = _tmp13_;
		}
		_tmp14_ = ((GeeHashMapNodeIterator*) self)->_map;
		if ((((GeeHashMapNodeIterator*) self)->_index + 1) < _tmp14_->priv->_array_size) {
			GeeHashMap* _tmp15_;
			GeeHashMapNode** _tmp16_;
			gint _tmp16__length1;
			gint _tmp17_;
			GeeHashMapNode* _tmp18_;
			_tmp15_ = ((GeeHashMapNodeIterator*) self)->_map;
			_tmp16_ = _tmp15_->priv->_nodes;
			_tmp16__length1 = _tmp15_->priv->_nodes_length1;
			((GeeHashMapNodeIterator*) self)->_index = ((GeeHashMapNodeIterator*) self)->_index + 1;
			_tmp17_ = ((GeeHashMapNodeIterator*) self)->_index;
			_tmp18_ = _tmp16_[_tmp17_];
			((GeeHashMapNodeIterator*) self)->_next = _tmp18_;
		} else {
			result = TRUE;
			return result;
		}
	}
}

static GeeIterator**
gee_hash_map_value_iterator_real_tee (GeeTraversable* base,
                                      guint forks,
                                      gint* result_length1)
{
	GeeHashMapValueIterator * self;
	GeeIterator** result;
	self = (GeeHashMapValueIterator*) base;
	if (forks == ((guint) 0)) {
		GeeIterator** _tmp0_;
		GeeIterator** _tmp1_;
		gint _tmp1__length1;
		_tmp0_ = g_new0 (GeeIterator*, 0 + 1);
		_tmp1_ = _tmp0_;
		_tmp1__length1 = 0;
		if (result_length1) {
			*result_length1 = _tmp1__length1;
		}
		result = _tmp1_;
		return result;
	} else {
		GeeIterator** _result_ = NULL;
		GeeIterator** _tmp2_;
		gint _result__length1;
		gint __result__size_;
		GeeIterator** _tmp3_;
		gint _tmp3__length1;
		GeeIterator* _tmp4_;
		GeeIterator** _tmp9_;
		gint _tmp9__length1;
		_tmp2_ = g_new0 (GeeIterator*, forks + 1);
		_result_ = _tmp2_;
		_result__length1 = forks;
		__result__size_ = _result__length1;
		_tmp3_ = _result_;
		_tmp3__length1 = _result__length1;
		_tmp4_ = _g_object_ref0 ((GeeIterator*) self);
		_g_object_unref0 (_tmp3_[0]);
		_tmp3_[0] = _tmp4_;
		{
			guint i = 0U;
			i = (guint) 1;
			{
				gboolean _tmp5_ = FALSE;
				_tmp5_ = TRUE;
				while (TRUE) {
					GeeIterator** _tmp7_;
					gint _tmp7__length1;
					GeeHashMapValueIterator* _tmp8_;
					if (!_tmp5_) {
						guint _tmp6_;
						_tmp6_ = i;
						i = _tmp6_ + 1;
					}
					_tmp5_ = FALSE;
					if (!(i < forks)) {
						break;
					}
					_tmp7_ = _result_;
					_tmp7__length1 = _result__length1;
					_tmp8_ = gee_hash_map_value_iterator_new_from_iterator (self->priv->k_type, (GBoxedCopyFunc) self->priv->k_dup_func, (GDestroyNotify) self->priv->k_destroy_func, self->priv->v_type, (GBoxedCopyFunc) self->priv->v_dup_func, (GDestroyNotify) self->priv->v_destroy_func, self);
					_g_object_unref0 (_tmp7_[i]);
					_tmp7_[i] = (GeeIterator*) _tmp8_;
				}
			}
		}
		_tmp9_ = _result_;
		_tmp9__length1 = _result__length1;
		if (result_length1) {
			*result_length1 = _tmp9__length1;
		}
		result = _tmp9_;
		return result;
	}
}

static void
gee_hash_map_value_iterator_class_init (GeeHashMapValueIteratorClass * klass,
                                        gpointer klass_data)
{
	gee_hash_map_value_iterator_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &GeeHashMapValueIterator_private_offset);
	G_OBJECT_CLASS (klass)->get_property = _vala_gee_hash_map_value_iterator_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gee_hash_map_value_iterator_set_property;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUE_ITERATOR_K_TYPE, g_param_spec_gtype ("k-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUE_ITERATOR_K_DUP_FUNC, g_param_spec_pointer ("k-dup-func", "dup func", "dup func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUE_ITERATOR_K_DESTROY_FUNC, g_param_spec_pointer ("k-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUE_ITERATOR_V_TYPE, g_param_spec_gtype ("v-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUE_ITERATOR_V_DUP_FUNC, g_param_spec_pointer ("v-dup-func", "dup func", "dup func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUE_ITERATOR_V_DESTROY_FUNC, g_param_spec_pointer ("v-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
}

static GType
gee_hash_map_value_iterator_gee_traversable_get_g_type (GeeHashMapValueIterator* self)
{
	return self->priv->v_type;
}

static GBoxedCopyFunc
gee_hash_map_value_iterator_gee_traversable_get_g_dup_func (GeeHashMapValueIterator* self)
{
	return self->priv->v_dup_func;
}

static GDestroyNotify
gee_hash_map_value_iterator_gee_traversable_get_g_destroy_func (GeeHashMapValueIterator* self)
{
	return self->priv->v_destroy_func;
}

static void
gee_hash_map_value_iterator_gee_traversable_interface_init (GeeTraversableIface * iface,
                                                            gpointer iface_data)
{
	gee_hash_map_value_iterator_gee_traversable_parent_iface = g_type_interface_peek_parent (iface);
	iface->foreach = (gboolean (*) (GeeTraversable*, GeeForallFunc, gpointer)) gee_hash_map_value_iterator_real_foreach;
	iface->tee = (GeeIterator** (*) (GeeTraversable*, guint, gint*)) gee_hash_map_value_iterator_real_tee;
	iface->get_g_type = (GType (*) (GeeTraversable *)) gee_hash_map_value_iterator_gee_traversable_get_g_type;
	iface->get_g_dup_func = (GBoxedCopyFunc (*) (GeeTraversable *)) gee_hash_map_value_iterator_gee_traversable_get_g_dup_func;
	iface->get_g_destroy_func = (GDestroyNotify (*) (GeeTraversable *)) gee_hash_map_value_iterator_gee_traversable_get_g_destroy_func;
}

static void
gee_hash_map_value_iterator_gee_iterator_interface_init (GeeIteratorIface * iface,
                                                         gpointer iface_data)
{
	gee_hash_map_value_iterator_gee_iterator_parent_iface = g_type_interface_peek_parent (iface);
	iface->get = (gpointer (*) (GeeIterator*)) gee_hash_map_value_iterator_real_get;
	iface->remove = (void (*) (GeeIterator*)) gee_hash_map_value_iterator_real_remove;
	iface->has_next = (gboolean (*) (GeeIterator*)) gee_hash_map_node_iterator_has_next;
	iface->next = (gboolean (*) (GeeIterator*)) gee_hash_map_node_iterator_next;
	iface->get_valid = (gboolean (*) (GeeIterator *)) gee_hash_map_node_iterator_get_valid;
	iface->get_read_only = (gboolean (*) (GeeIterator *)) gee_hash_map_node_iterator_get_read_only;
}

static void
gee_hash_map_value_iterator_instance_init (GeeHashMapValueIterator * self,
                                           gpointer klass)
{
	self->priv = gee_hash_map_value_iterator_get_instance_private (self);
}

static GType
gee_hash_map_value_iterator_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (GeeHashMapValueIteratorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_hash_map_value_iterator_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeHashMapValueIterator), 0, (GInstanceInitFunc) gee_hash_map_value_iterator_instance_init, NULL };
	static const GInterfaceInfo gee_traversable_info = { (GInterfaceInitFunc) gee_hash_map_value_iterator_gee_traversable_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	static const GInterfaceInfo gee_iterator_info = { (GInterfaceInitFunc) gee_hash_map_value_iterator_gee_iterator_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	GType gee_hash_map_value_iterator_type_id;
	gee_hash_map_value_iterator_type_id = g_type_register_static (GEE_HASH_MAP_TYPE_NODE_ITERATOR, "GeeHashMapValueIterator", &g_define_type_info, 0);
	g_type_add_interface_static (gee_hash_map_value_iterator_type_id, GEE_TYPE_TRAVERSABLE, &gee_traversable_info);
	g_type_add_interface_static (gee_hash_map_value_iterator_type_id, GEE_TYPE_ITERATOR, &gee_iterator_info);
	GeeHashMapValueIterator_private_offset = g_type_add_instance_private (gee_hash_map_value_iterator_type_id, sizeof (GeeHashMapValueIteratorPrivate));
	return gee_hash_map_value_iterator_type_id;
}

static GType
gee_hash_map_value_iterator_get_type (void)
{
	static volatile gsize gee_hash_map_value_iterator_type_id__once = 0;
	if (g_once_init_enter (&gee_hash_map_value_iterator_type_id__once)) {
		GType gee_hash_map_value_iterator_type_id;
		gee_hash_map_value_iterator_type_id = gee_hash_map_value_iterator_get_type_once ();
		g_once_init_leave (&gee_hash_map_value_iterator_type_id__once, gee_hash_map_value_iterator_type_id);
	}
	return gee_hash_map_value_iterator_type_id__once;
}

static void
_vala_gee_hash_map_value_iterator_get_property (GObject * object,
                                                guint property_id,
                                                GValue * value,
                                                GParamSpec * pspec)
{
	GeeHashMapValueIterator * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_HASH_MAP_TYPE_VALUE_ITERATOR, GeeHashMapValueIterator);
	switch (property_id) {
		case GEE_HASH_MAP_VALUE_ITERATOR_K_TYPE:
		g_value_set_gtype (value, self->priv->k_type);
		break;
		case GEE_HASH_MAP_VALUE_ITERATOR_K_DUP_FUNC:
		g_value_set_pointer (value, self->priv->k_dup_func);
		break;
		case GEE_HASH_MAP_VALUE_ITERATOR_K_DESTROY_FUNC:
		g_value_set_pointer (value, self->priv->k_destroy_func);
		break;
		case GEE_HASH_MAP_VALUE_ITERATOR_V_TYPE:
		g_value_set_gtype (value, self->priv->v_type);
		break;
		case GEE_HASH_MAP_VALUE_ITERATOR_V_DUP_FUNC:
		g_value_set_pointer (value, self->priv->v_dup_func);
		break;
		case GEE_HASH_MAP_VALUE_ITERATOR_V_DESTROY_FUNC:
		g_value_set_pointer (value, self->priv->v_destroy_func);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_gee_hash_map_value_iterator_set_property (GObject * object,
                                                guint property_id,
                                                const GValue * value,
                                                GParamSpec * pspec)
{
	GeeHashMapValueIterator * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_HASH_MAP_TYPE_VALUE_ITERATOR, GeeHashMapValueIterator);
	switch (property_id) {
		case GEE_HASH_MAP_VALUE_ITERATOR_K_TYPE:
		self->priv->k_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_VALUE_ITERATOR_K_DUP_FUNC:
		self->priv->k_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_VALUE_ITERATOR_K_DESTROY_FUNC:
		self->priv->k_destroy_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_VALUE_ITERATOR_V_TYPE:
		self->priv->v_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_VALUE_ITERATOR_V_DUP_FUNC:
		self->priv->v_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_VALUE_ITERATOR_V_DESTROY_FUNC:
		self->priv->v_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static inline gpointer
gee_hash_map_entry_iterator_get_instance_private (GeeHashMapEntryIterator* self)
{
	return G_STRUCT_MEMBER_P (self, GeeHashMapEntryIterator_private_offset);
}

static GeeHashMapEntryIterator*
gee_hash_map_entry_iterator_construct (GType object_type,
                                       GType k_type,
                                       GBoxedCopyFunc k_dup_func,
                                       GDestroyNotify k_destroy_func,
                                       GType v_type,
                                       GBoxedCopyFunc v_dup_func,
                                       GDestroyNotify v_destroy_func,
                                       GeeHashMap* map)
{
	GeeHashMapEntryIterator * self = NULL;
	g_return_val_if_fail (map != NULL, NULL);
	self = (GeeHashMapEntryIterator*) gee_hash_map_node_iterator_construct (object_type, k_type, (GBoxedCopyFunc) k_dup_func, (GDestroyNotify) k_destroy_func, v_type, (GBoxedCopyFunc) v_dup_func, (GDestroyNotify) v_destroy_func, map);
	self->priv->k_type = k_type;
	self->priv->k_dup_func = k_dup_func;
	self->priv->k_destroy_func = k_destroy_func;
	self->priv->v_type = v_type;
	self->priv->v_dup_func = v_dup_func;
	self->priv->v_destroy_func = v_destroy_func;
	return self;
}

static GeeHashMapEntryIterator*
gee_hash_map_entry_iterator_new (GType k_type,
                                 GBoxedCopyFunc k_dup_func,
                                 GDestroyNotify k_destroy_func,
                                 GType v_type,
                                 GBoxedCopyFunc v_dup_func,
                                 GDestroyNotify v_destroy_func,
                                 GeeHashMap* map)
{
	return gee_hash_map_entry_iterator_construct (GEE_HASH_MAP_TYPE_ENTRY_ITERATOR, k_type, k_dup_func, k_destroy_func, v_type, v_dup_func, v_destroy_func, map);
}

static GeeHashMapEntryIterator*
gee_hash_map_entry_iterator_construct_from_iterator (GType object_type,
                                                     GType k_type,
                                                     GBoxedCopyFunc k_dup_func,
                                                     GDestroyNotify k_destroy_func,
                                                     GType v_type,
                                                     GBoxedCopyFunc v_dup_func,
                                                     GDestroyNotify v_destroy_func,
                                                     GeeHashMapEntryIterator* iter)
{
	GeeHashMapEntryIterator * self = NULL;
	g_return_val_if_fail (iter != NULL, NULL);
	self = (GeeHashMapEntryIterator*) gee_hash_map_node_iterator_construct_from_iterator (object_type, k_type, (GBoxedCopyFunc) k_dup_func, (GDestroyNotify) k_destroy_func, v_type, (GBoxedCopyFunc) v_dup_func, (GDestroyNotify) v_destroy_func, (GeeHashMapNodeIterator*) iter);
	self->priv->k_type = k_type;
	self->priv->k_dup_func = k_dup_func;
	self->priv->k_destroy_func = k_destroy_func;
	self->priv->v_type = v_type;
	self->priv->v_dup_func = v_dup_func;
	self->priv->v_destroy_func = v_destroy_func;
	return self;
}

static GeeHashMapEntryIterator*
gee_hash_map_entry_iterator_new_from_iterator (GType k_type,
                                               GBoxedCopyFunc k_dup_func,
                                               GDestroyNotify k_destroy_func,
                                               GType v_type,
                                               GBoxedCopyFunc v_dup_func,
                                               GDestroyNotify v_destroy_func,
                                               GeeHashMapEntryIterator* iter)
{
	return gee_hash_map_entry_iterator_construct_from_iterator (GEE_HASH_MAP_TYPE_ENTRY_ITERATOR, k_type, k_dup_func, k_destroy_func, v_type, v_dup_func, v_destroy_func, iter);
}

static GeeMapEntry*
gee_hash_map_entry_iterator_real_get (GeeIterator* base)
{
	GeeHashMapEntryIterator * self;
	GeeHashMap* _tmp0_;
	GeeHashMapNode* _tmp1_;
	GeeHashMapNode* _tmp2_;
	GeeMapEntry* _tmp3_;
	GeeMapEntry* result;
	self = (GeeHashMapEntryIterator*) base;
	_tmp0_ = ((GeeHashMapNodeIterator*) self)->_map;
	_vala_assert (((GeeHashMapNodeIterator*) self)->_stamp == _tmp0_->priv->_stamp, "_stamp == _map._stamp");
	_tmp1_ = ((GeeHashMapNodeIterator*) self)->_node;
	_vala_assert (_tmp1_ != NULL, "_node != null");
	_tmp2_ = ((GeeHashMapNodeIterator*) self)->_node;
	_tmp3_ = gee_hash_map_entry_entry_for (self->priv->k_type, (GBoxedCopyFunc) self->priv->k_dup_func, (GDestroyNotify) self->priv->k_destroy_func, self->priv->v_type, (GBoxedCopyFunc) self->priv->v_dup_func, (GDestroyNotify) self->priv->v_destroy_func, _tmp2_);
	result = _tmp3_;
	return result;
}

static void
gee_hash_map_entry_iterator_real_remove (GeeIterator* base)
{
	GeeHashMapEntryIterator * self;
	self = (GeeHashMapEntryIterator*) base;
	g_assert_not_reached ();
}

static gboolean
gee_hash_map_entry_iterator_real_foreach (GeeTraversable* base,
                                          GeeForallFunc f,
                                          gpointer f_target)
{
	GeeHashMapEntryIterator * self;
	GeeHashMapNode* _tmp0_;
	gboolean result;
	self = (GeeHashMapEntryIterator*) base;
	_tmp0_ = ((GeeHashMapNodeIterator*) self)->_node;
	if (_tmp0_ != NULL) {
		GeeHashMapNode* _tmp1_;
		GeeMapEntry* _tmp2_;
		GeeHashMapNode* _tmp3_;
		_tmp1_ = ((GeeHashMapNodeIterator*) self)->_node;
		_tmp2_ = gee_hash_map_entry_entry_for (self->priv->k_type, (GBoxedCopyFunc) self->priv->k_dup_func, (GDestroyNotify) self->priv->k_destroy_func, self->priv->v_type, (GBoxedCopyFunc) self->priv->v_dup_func, (GDestroyNotify) self->priv->v_destroy_func, _tmp1_);
		if (!f (_tmp2_, f_target)) {
			result = FALSE;
			return result;
		}
		_tmp3_ = ((GeeHashMapNodeIterator*) self)->_next;
		if (_tmp3_ == NULL) {
			GeeHashMapNode* _tmp4_;
			GeeHashMapNode* _tmp5_;
			_tmp4_ = ((GeeHashMapNodeIterator*) self)->_node;
			_tmp5_ = _tmp4_->next;
			((GeeHashMapNodeIterator*) self)->_next = _tmp5_;
		}
	}
	while (TRUE) {
		GeeHashMap* _tmp12_;
		while (TRUE) {
			GeeHashMapNode* _tmp6_;
			GeeHashMapNode* _tmp7_;
			GeeHashMapNode* _tmp8_;
			GeeMapEntry* _tmp9_;
			GeeHashMapNode* _tmp10_;
			GeeHashMapNode* _tmp11_;
			_tmp6_ = ((GeeHashMapNodeIterator*) self)->_next;
			if (!(_tmp6_ != NULL)) {
				break;
			}
			_tmp7_ = ((GeeHashMapNodeIterator*) self)->_next;
			((GeeHashMapNodeIterator*) self)->_node = _tmp7_;
			_tmp8_ = ((GeeHashMapNodeIterator*) self)->_node;
			_tmp9_ = gee_hash_map_entry_entry_for (self->priv->k_type, (GBoxedCopyFunc) self->priv->k_dup_func, (GDestroyNotify) self->priv->k_destroy_func, self->priv->v_type, (GBoxedCopyFunc) self->priv->v_dup_func, (GDestroyNotify) self->priv->v_destroy_func, _tmp8_);
			if (!f (_tmp9_, f_target)) {
				result = FALSE;
				return result;
			}
			_tmp10_ = ((GeeHashMapNodeIterator*) self)->_next;
			_tmp11_ = _tmp10_->next;
			((GeeHashMapNodeIterator*) self)->_next = _tmp11_;
		}
		_tmp12_ = ((GeeHashMapNodeIterator*) self)->_map;
		if ((((GeeHashMapNodeIterator*) self)->_index + 1) < _tmp12_->priv->_array_size) {
			GeeHashMap* _tmp13_;
			GeeHashMapNode** _tmp14_;
			gint _tmp14__length1;
			gint _tmp15_;
			GeeHashMapNode* _tmp16_;
			_tmp13_ = ((GeeHashMapNodeIterator*) self)->_map;
			_tmp14_ = _tmp13_->priv->_nodes;
			_tmp14__length1 = _tmp13_->priv->_nodes_length1;
			((GeeHashMapNodeIterator*) self)->_index = ((GeeHashMapNodeIterator*) self)->_index + 1;
			_tmp15_ = ((GeeHashMapNodeIterator*) self)->_index;
			_tmp16_ = _tmp14_[_tmp15_];
			((GeeHashMapNodeIterator*) self)->_next = _tmp16_;
		} else {
			result = TRUE;
			return result;
		}
	}
}

static GeeIterator**
gee_hash_map_entry_iterator_real_tee (GeeTraversable* base,
                                      guint forks,
                                      gint* result_length1)
{
	GeeHashMapEntryIterator * self;
	GeeIterator** result;
	self = (GeeHashMapEntryIterator*) base;
	if (forks == ((guint) 0)) {
		GeeIterator** _tmp0_;
		GeeIterator** _tmp1_;
		gint _tmp1__length1;
		_tmp0_ = g_new0 (GeeIterator*, 0 + 1);
		_tmp1_ = _tmp0_;
		_tmp1__length1 = 0;
		if (result_length1) {
			*result_length1 = _tmp1__length1;
		}
		result = _tmp1_;
		return result;
	} else {
		GeeIterator** _result_ = NULL;
		GeeIterator** _tmp2_;
		gint _result__length1;
		gint __result__size_;
		GeeIterator** _tmp3_;
		gint _tmp3__length1;
		GeeIterator* _tmp4_;
		GeeIterator** _tmp9_;
		gint _tmp9__length1;
		_tmp2_ = g_new0 (GeeIterator*, forks + 1);
		_result_ = _tmp2_;
		_result__length1 = forks;
		__result__size_ = _result__length1;
		_tmp3_ = _result_;
		_tmp3__length1 = _result__length1;
		_tmp4_ = _g_object_ref0 ((GeeIterator*) self);
		_g_object_unref0 (_tmp3_[0]);
		_tmp3_[0] = _tmp4_;
		{
			guint i = 0U;
			i = (guint) 1;
			{
				gboolean _tmp5_ = FALSE;
				_tmp5_ = TRUE;
				while (TRUE) {
					GeeIterator** _tmp7_;
					gint _tmp7__length1;
					GeeHashMapEntryIterator* _tmp8_;
					if (!_tmp5_) {
						guint _tmp6_;
						_tmp6_ = i;
						i = _tmp6_ + 1;
					}
					_tmp5_ = FALSE;
					if (!(i < forks)) {
						break;
					}
					_tmp7_ = _result_;
					_tmp7__length1 = _result__length1;
					_tmp8_ = gee_hash_map_entry_iterator_new_from_iterator (self->priv->k_type, (GBoxedCopyFunc) self->priv->k_dup_func, (GDestroyNotify) self->priv->k_destroy_func, self->priv->v_type, (GBoxedCopyFunc) self->priv->v_dup_func, (GDestroyNotify) self->priv->v_destroy_func, self);
					_g_object_unref0 (_tmp7_[i]);
					_tmp7_[i] = (GeeIterator*) _tmp8_;
				}
			}
		}
		_tmp9_ = _result_;
		_tmp9__length1 = _result__length1;
		if (result_length1) {
			*result_length1 = _tmp9__length1;
		}
		result = _tmp9_;
		return result;
	}
}

static void
gee_hash_map_entry_iterator_class_init (GeeHashMapEntryIteratorClass * klass,
                                        gpointer klass_data)
{
	gee_hash_map_entry_iterator_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &GeeHashMapEntryIterator_private_offset);
	G_OBJECT_CLASS (klass)->get_property = _vala_gee_hash_map_entry_iterator_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gee_hash_map_entry_iterator_set_property;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_ITERATOR_K_TYPE, g_param_spec_gtype ("k-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_ITERATOR_K_DUP_FUNC, g_param_spec_pointer ("k-dup-func", "dup func", "dup func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_ITERATOR_K_DESTROY_FUNC, g_param_spec_pointer ("k-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_ITERATOR_V_TYPE, g_param_spec_gtype ("v-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_ITERATOR_V_DUP_FUNC, g_param_spec_pointer ("v-dup-func", "dup func", "dup func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_ITERATOR_V_DESTROY_FUNC, g_param_spec_pointer ("v-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
}

static GType
gee_hash_map_entry_iterator_gee_traversable_get_g_type (GeeHashMapEntryIterator* self)
{
	return GEE_MAP_TYPE_ENTRY;
}

static GBoxedCopyFunc
gee_hash_map_entry_iterator_gee_traversable_get_g_dup_func (GeeHashMapEntryIterator* self)
{
	return g_object_ref;
}

static GDestroyNotify
gee_hash_map_entry_iterator_gee_traversable_get_g_destroy_func (GeeHashMapEntryIterator* self)
{
	return g_object_unref;
}

static void
gee_hash_map_entry_iterator_gee_traversable_interface_init (GeeTraversableIface * iface,
                                                            gpointer iface_data)
{
	gee_hash_map_entry_iterator_gee_traversable_parent_iface = g_type_interface_peek_parent (iface);
	iface->foreach = (gboolean (*) (GeeTraversable*, GeeForallFunc, gpointer)) gee_hash_map_entry_iterator_real_foreach;
	iface->tee = (GeeIterator** (*) (GeeTraversable*, guint, gint*)) gee_hash_map_entry_iterator_real_tee;
	iface->get_g_type = (GType (*) (GeeTraversable *)) gee_hash_map_entry_iterator_gee_traversable_get_g_type;
	iface->get_g_dup_func = (GBoxedCopyFunc (*) (GeeTraversable *)) gee_hash_map_entry_iterator_gee_traversable_get_g_dup_func;
	iface->get_g_destroy_func = (GDestroyNotify (*) (GeeTraversable *)) gee_hash_map_entry_iterator_gee_traversable_get_g_destroy_func;
}

static void
gee_hash_map_entry_iterator_gee_iterator_interface_init (GeeIteratorIface * iface,
                                                         gpointer iface_data)
{
	gee_hash_map_entry_iterator_gee_iterator_parent_iface = g_type_interface_peek_parent (iface);
	iface->get = (gpointer (*) (GeeIterator*)) gee_hash_map_entry_iterator_real_get;
	iface->remove = (void (*) (GeeIterator*)) gee_hash_map_entry_iterator_real_remove;
	iface->has_next = (gboolean (*) (GeeIterator*)) gee_hash_map_node_iterator_has_next;
	iface->next = (gboolean (*) (GeeIterator*)) gee_hash_map_node_iterator_next;
	iface->get_valid = (gboolean (*) (GeeIterator *)) gee_hash_map_node_iterator_get_valid;
	iface->get_read_only = (gboolean (*) (GeeIterator *)) gee_hash_map_node_iterator_get_read_only;
}

static void
gee_hash_map_entry_iterator_instance_init (GeeHashMapEntryIterator * self,
                                           gpointer klass)
{
	self->priv = gee_hash_map_entry_iterator_get_instance_private (self);
}

static GType
gee_hash_map_entry_iterator_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (GeeHashMapEntryIteratorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_hash_map_entry_iterator_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeHashMapEntryIterator), 0, (GInstanceInitFunc) gee_hash_map_entry_iterator_instance_init, NULL };
	static const GInterfaceInfo gee_traversable_info = { (GInterfaceInitFunc) gee_hash_map_entry_iterator_gee_traversable_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	static const GInterfaceInfo gee_iterator_info = { (GInterfaceInitFunc) gee_hash_map_entry_iterator_gee_iterator_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	GType gee_hash_map_entry_iterator_type_id;
	gee_hash_map_entry_iterator_type_id = g_type_register_static (GEE_HASH_MAP_TYPE_NODE_ITERATOR, "GeeHashMapEntryIterator", &g_define_type_info, 0);
	g_type_add_interface_static (gee_hash_map_entry_iterator_type_id, GEE_TYPE_TRAVERSABLE, &gee_traversable_info);
	g_type_add_interface_static (gee_hash_map_entry_iterator_type_id, GEE_TYPE_ITERATOR, &gee_iterator_info);
	GeeHashMapEntryIterator_private_offset = g_type_add_instance_private (gee_hash_map_entry_iterator_type_id, sizeof (GeeHashMapEntryIteratorPrivate));
	return gee_hash_map_entry_iterator_type_id;
}

static GType
gee_hash_map_entry_iterator_get_type (void)
{
	static volatile gsize gee_hash_map_entry_iterator_type_id__once = 0;
	if (g_once_init_enter (&gee_hash_map_entry_iterator_type_id__once)) {
		GType gee_hash_map_entry_iterator_type_id;
		gee_hash_map_entry_iterator_type_id = gee_hash_map_entry_iterator_get_type_once ();
		g_once_init_leave (&gee_hash_map_entry_iterator_type_id__once, gee_hash_map_entry_iterator_type_id);
	}
	return gee_hash_map_entry_iterator_type_id__once;
}

static void
_vala_gee_hash_map_entry_iterator_get_property (GObject * object,
                                                guint property_id,
                                                GValue * value,
                                                GParamSpec * pspec)
{
	GeeHashMapEntryIterator * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_HASH_MAP_TYPE_ENTRY_ITERATOR, GeeHashMapEntryIterator);
	switch (property_id) {
		case GEE_HASH_MAP_ENTRY_ITERATOR_K_TYPE:
		g_value_set_gtype (value, self->priv->k_type);
		break;
		case GEE_HASH_MAP_ENTRY_ITERATOR_K_DUP_FUNC:
		g_value_set_pointer (value, self->priv->k_dup_func);
		break;
		case GEE_HASH_MAP_ENTRY_ITERATOR_K_DESTROY_FUNC:
		g_value_set_pointer (value, self->priv->k_destroy_func);
		break;
		case GEE_HASH_MAP_ENTRY_ITERATOR_V_TYPE:
		g_value_set_gtype (value, self->priv->v_type);
		break;
		case GEE_HASH_MAP_ENTRY_ITERATOR_V_DUP_FUNC:
		g_value_set_pointer (value, self->priv->v_dup_func);
		break;
		case GEE_HASH_MAP_ENTRY_ITERATOR_V_DESTROY_FUNC:
		g_value_set_pointer (value, self->priv->v_destroy_func);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_gee_hash_map_entry_iterator_set_property (GObject * object,
                                                guint property_id,
                                                const GValue * value,
                                                GParamSpec * pspec)
{
	GeeHashMapEntryIterator * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_HASH_MAP_TYPE_ENTRY_ITERATOR, GeeHashMapEntryIterator);
	switch (property_id) {
		case GEE_HASH_MAP_ENTRY_ITERATOR_K_TYPE:
		self->priv->k_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_ENTRY_ITERATOR_K_DUP_FUNC:
		self->priv->k_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_ENTRY_ITERATOR_K_DESTROY_FUNC:
		self->priv->k_destroy_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_ENTRY_ITERATOR_V_TYPE:
		self->priv->v_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_ENTRY_ITERATOR_V_DUP_FUNC:
		self->priv->v_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_ENTRY_ITERATOR_V_DESTROY_FUNC:
		self->priv->v_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
gee_hash_map_class_init (GeeHashMapClass * klass,
                         gpointer klass_data)
{
	gee_hash_map_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &GeeHashMap_private_offset);
	((GeeAbstractMapClass *) klass)->has_key = (gboolean (*) (GeeAbstractMap*, gconstpointer)) gee_hash_map_real_has_key;
	((GeeAbstractMapClass *) klass)->has = (gboolean (*) (GeeAbstractMap*, gconstpointer, gconstpointer)) gee_hash_map_real_has;
	((GeeAbstractMapClass *) klass)->get = (gpointer (*) (GeeAbstractMap*, gconstpointer)) gee_hash_map_real_get;
	((GeeAbstractMapClass *) klass)->set = (void (*) (GeeAbstractMap*, gconstpointer, gconstpointer)) gee_hash_map_real_set;
	((GeeAbstractMapClass *) klass)->unset = (gboolean (*) (GeeAbstractMap*, gconstpointer, gpointer*)) gee_hash_map_real_unset;
	((GeeAbstractMapClass *) klass)->clear = (void (*) (GeeAbstractMap*)) gee_hash_map_real_clear;
	((GeeAbstractMapClass *) klass)->map_iterator = (GeeMapIterator* (*) (GeeAbstractMap*)) gee_hash_map_real_map_iterator;
	GEE_ABSTRACT_MAP_CLASS (klass)->get_size = gee_hash_map_real_get_size;
	GEE_ABSTRACT_MAP_CLASS (klass)->get_read_only = gee_hash_map_real_get_read_only;
	GEE_ABSTRACT_MAP_CLASS (klass)->get_keys = gee_hash_map_real_get_keys;
	GEE_ABSTRACT_MAP_CLASS (klass)->get_values = gee_hash_map_real_get_values;
	GEE_ABSTRACT_MAP_CLASS (klass)->get_entries = gee_hash_map_real_get_entries;
	G_OBJECT_CLASS (klass)->get_property = _vala_gee_hash_map_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gee_hash_map_set_property;
	G_OBJECT_CLASS (klass)->finalize = gee_hash_map_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_K_TYPE, g_param_spec_gtype ("k-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_K_DUP_FUNC, g_param_spec_pointer ("k-dup-func", "dup func", "dup func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_K_DESTROY_FUNC, g_param_spec_pointer ("k-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_V_TYPE, g_param_spec_gtype ("v-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_V_DUP_FUNC, g_param_spec_pointer ("v-dup-func", "dup func", "dup func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_V_DESTROY_FUNC, g_param_spec_pointer ("v-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	/**
	 * {@inheritDoc}
	 */
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_SIZE_PROPERTY, gee_hash_map_properties[GEE_HASH_MAP_SIZE_PROPERTY] = g_param_spec_int ("size", "size", "size", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
	/**
	 * {@inheritDoc}
	 */
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_READ_ONLY_PROPERTY, gee_hash_map_properties[GEE_HASH_MAP_READ_ONLY_PROPERTY] = g_param_spec_boolean ("read-only", "read-only", "read-only", FALSE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
	/**
	 * {@inheritDoc}
	 */
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEYS_PROPERTY, gee_hash_map_properties[GEE_HASH_MAP_KEYS_PROPERTY] = g_param_spec_object ("keys", "keys", "keys", GEE_TYPE_SET, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
	/**
	 * {@inheritDoc}
	 */
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUES_PROPERTY, gee_hash_map_properties[GEE_HASH_MAP_VALUES_PROPERTY] = g_param_spec_object ("values", "values", "values", GEE_TYPE_COLLECTION, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
	/**
	 * {@inheritDoc}
	 */
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRIES_PROPERTY, gee_hash_map_properties[GEE_HASH_MAP_ENTRIES_PROPERTY] = g_param_spec_object ("entries", "entries", "entries", GEE_TYPE_SET, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
}

static void
gee_hash_map_instance_init (GeeHashMap * self,
                            gpointer klass)
{
	self->priv = gee_hash_map_get_instance_private (self);
	self->priv->_stamp = 0;
}

static void
gee_hash_map_finalize (GObject * obj)
{
	GeeHashMap * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GEE_TYPE_HASH_MAP, GeeHashMap);
	gee_abstract_map_clear ((GeeAbstractMap*) self);
	self->priv->_nodes = (_vala_array_free (self->priv->_nodes, self->priv->_nodes_length1, (GDestroyNotify) gee_hash_map_node_free), NULL);
	_gee_functions_hash_data_func_closure_unref0 (self->priv->_key_hash_func);
	_gee_functions_equal_data_func_closure_unref0 (self->priv->_key_equal_func);
	_gee_functions_equal_data_func_closure_unref0 (self->priv->_value_equal_func);
	G_OBJECT_CLASS (gee_hash_map_parent_class)->finalize (obj);
}

/**
 * Hash table implementation of the {@link Map} interface.
 *
 * This implementation is better fit for highly heterogeneous key values.
 * In case of high key hashes redundancy or higher amount of data prefer using
 * tree implementation like {@link TreeMap}.
 *
 * @see TreeMap
 */
static GType
gee_hash_map_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (GeeHashMapClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_hash_map_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeHashMap), 0, (GInstanceInitFunc) gee_hash_map_instance_init, NULL };
	GType gee_hash_map_type_id;
	gee_hash_map_type_id = g_type_register_static (GEE_TYPE_ABSTRACT_MAP, "GeeHashMap", &g_define_type_info, 0);
	GeeHashMap_private_offset = g_type_add_instance_private (gee_hash_map_type_id, sizeof (GeeHashMapPrivate));
	return gee_hash_map_type_id;
}

GType
gee_hash_map_get_type (void)
{
	static volatile gsize gee_hash_map_type_id__once = 0;
	if (g_once_init_enter (&gee_hash_map_type_id__once)) {
		GType gee_hash_map_type_id;
		gee_hash_map_type_id = gee_hash_map_get_type_once ();
		g_once_init_leave (&gee_hash_map_type_id__once, gee_hash_map_type_id);
	}
	return gee_hash_map_type_id__once;
}

static void
_vala_gee_hash_map_get_property (GObject * object,
                                 guint property_id,
                                 GValue * value,
                                 GParamSpec * pspec)
{
	GeeHashMap * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_TYPE_HASH_MAP, GeeHashMap);
	switch (property_id) {
		case GEE_HASH_MAP_SIZE_PROPERTY:
		g_value_set_int (value, gee_abstract_map_get_size ((GeeAbstractMap*) self));
		break;
		case GEE_HASH_MAP_READ_ONLY_PROPERTY:
		g_value_set_boolean (value, gee_abstract_map_get_read_only ((GeeAbstractMap*) self));
		break;
		case GEE_HASH_MAP_KEYS_PROPERTY:
		g_value_take_object (value, gee_abstract_map_get_keys ((GeeAbstractMap*) self));
		break;
		case GEE_HASH_MAP_VALUES_PROPERTY:
		g_value_take_object (value, gee_abstract_map_get_values ((GeeAbstractMap*) self));
		break;
		case GEE_HASH_MAP_ENTRIES_PROPERTY:
		g_value_take_object (value, gee_abstract_map_get_entries ((GeeAbstractMap*) self));
		break;
		case GEE_HASH_MAP_K_TYPE:
		g_value_set_gtype (value, self->priv->k_type);
		break;
		case GEE_HASH_MAP_K_DUP_FUNC:
		g_value_set_pointer (value, self->priv->k_dup_func);
		break;
		case GEE_HASH_MAP_K_DESTROY_FUNC:
		g_value_set_pointer (value, self->priv->k_destroy_func);
		break;
		case GEE_HASH_MAP_V_TYPE:
		g_value_set_gtype (value, self->priv->v_type);
		break;
		case GEE_HASH_MAP_V_DUP_FUNC:
		g_value_set_pointer (value, self->priv->v_dup_func);
		break;
		case GEE_HASH_MAP_V_DESTROY_FUNC:
		g_value_set_pointer (value, self->priv->v_destroy_func);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_gee_hash_map_set_property (GObject * object,
                                 guint property_id,
                                 const GValue * value,
                                 GParamSpec * pspec)
{
	GeeHashMap * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_TYPE_HASH_MAP, GeeHashMap);
	switch (property_id) {
		case GEE_HASH_MAP_K_TYPE:
		self->priv->k_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_K_DUP_FUNC:
		self->priv->k_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_K_DESTROY_FUNC:
		self->priv->k_destroy_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_V_TYPE:
		self->priv->v_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_V_DUP_FUNC:
		self->priv->v_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_V_DESTROY_FUNC:
		self->priv->v_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_array_destroy (gpointer array,
                     gssize array_length,
                     GDestroyNotify destroy_func)
{
	if ((array != NULL) && (destroy_func != NULL)) {
		gssize i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}

static void
_vala_array_free (gpointer array,
                  gssize array_length,
                  GDestroyNotify destroy_func)
{
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}

